@@ -9,7 +9,7 @@ use strict;
 use Carp;
 
 use vars qw($VERSION);
-$VERSION = '4.014'; # was: sprintf '4.%03d', q$Revision: #13 $ =~ /\D(\d+)\s*$/;
+$VERSION = sprintf '4.%03d', q$Revision: #13 $ =~ /\D(\d+)\s*$/;
 
 use base  qw(Tk::Toplevel);
 
@@ -23,18 +23,14 @@ sub Populate {
     $buttons = ['OK'] unless defined $buttons;
     my $default_button = delete $args->{'-default_button'};
     $default_button = $buttons->[0] unless defined $default_button;
-    my $cancel_button = delete $args->{'-cancel_button'};
-    if (!$cancel_button && @$buttons == 1) {
-	$cancel_button = $buttons->[0];
-    }
 
     $cw->{'selected_button'} = '';
     $cw->transient($cw->Parent->toplevel);
     $cw->withdraw;
-    if ($cancel_button) {
-	$cw->protocol('WM_DELETE_WINDOW' => sub { $cw->{'cancel_button'}->invoke });
+    if (@$buttons == 1) {
+	$cw->protocol('WM_DELETE_WINDOW' => sub { $cw->{'default_button'}->invoke });
     } else {
-	$cw->protocol('WM_DELETE_WINDOW' => sub { $cw->{'selected_button'} = undef });
+	$cw->protocol('WM_DELETE_WINDOW' => sub {});
     }
 
     # create the two frames
@@ -57,24 +53,19 @@ sub Populate {
           $b->configure(-width => 10, -pady => 0);
          }
 	if ($bl eq $default_button) {
-	    my $db = $bot->Frame(-relief => 'sunken', -bd => 1);
-	    $b->raise($db);
-	    $b->pack(-in => $db, -padx => '2', -pady => '2');
-	    $db->pack(-side => 'left', -expand => 1, -padx => 1, -pady => 1);
+            if ($Tk::platform eq 'MSWin32') {
+                $b->pack(-side => 'left', -expand => 1,  -padx => 1, -pady => 1);
+            } else {
+	        my $db = $bot->Frame(-relief => 'sunken', -bd => 1);
+	        $b->raise($db);
+	        $b->pack(-in => $db, -padx => '2', -pady => '2');
+	        $db->pack(-side => 'left', -expand => 1, -padx => 1, -pady => 1);
+            }
 	    $cw->{'default_button'} = $b;
 	    $cw->bind('<Return>' => [ $b, 'Invoke']);
 	} else {
 	    $b->pack(-side => 'left', -expand => 1,  -padx => 1, -pady => 1);
 	}
-	if (defined $cancel_button && $bl eq $cancel_button) {
-	    $cw->{'cancel_button'} = $b;
-	}
-    }
-    if (defined $default_button && !$cw->{'default_button'}) {
-	warn "Default button `$default_button' does not exist.\n";
-    }
-    if (defined $cancel_button && !$cw->{'cancel_button'}) {
-	warn "Cancel button `$cancel_button' does not exist.\n";
     }
     $cw->ConfigSpecs(-command    => ['CALLBACK', undef, undef, undef ],
                      -foreground => ['DESCENDANTS', 'foreground','Foreground', 'black'],
@@ -97,8 +88,8 @@ sub Wait
  my $cw = shift;
  $cw->Callback(-showcommand => $cw);
  $cw->waitVariable(\$cw->{'selected_button'});
- $cw->grabRelease if Tk::Exists($cw);
- $cw->withdraw if Tk::Exists($cw);
+ $cw->grabRelease;
+ $cw->withdraw;
  $cw->Callback(-command => $cw->{'selected_button'});
 }
 
