@@ -5,25 +5,20 @@ package Tk::DirTree;
 #
 # Chris Dean <ctdean@cogit.com>
 
-use strict;
 use vars qw($VERSION);
-$VERSION = '4.020';
+$VERSION = sprintf '4.%03d', q$Revision: #14 $ =~ /\D(\d+)\s*$/;
 
 use Tk;
 use Tk::Derived;
 use Tk::Tree;
 use Cwd;
 use DirHandle;
-use File::Spec qw();
 
 use base  qw(Tk::Derived Tk::Tree);
 use strict;
 
 Construct Tk::Widget 'DirTree';
 
-my $sep = $^O eq 'MSWin32' ? '\\' : '/';
-
-*_fs_encode = eval { require Encode; 1 } ? sub { Encode::encode("iso-8859-1", $_[0]) } : sub { $_[0] };
 
 sub Populate {
     my( $cw, $args ) = @_;
@@ -37,14 +32,12 @@ sub Populate {
         -directory      => [qw/SETMETHOD directory Directory ./],
         -value          => '-directory' );
 
-    $cw->configure( -separator => $sep,
-		    -itemtype => 'imagetext',
-		  );
+    $cw->configure( -separator => '/', -itemtype => 'imagetext' );
 }
 
 sub DirCmd {
     my( $w, $dir, $showhidden ) = @_;
-    $dir .= $sep if $dir =~ /^[a-z]:$/i and $^O eq 'MSWin32';
+    $dir .= "/" if $dir =~ /^[a-z]:$/i and $^O eq 'MSWin32';
     my $h = DirHandle->new( $dir ) or return();
     my @names = grep( $_ ne '.' && $_ ne '..', $h->read );
     @names = grep( ! /^[.]/, @names ) unless $showhidden;
@@ -66,7 +59,6 @@ sub fullpath
   {
    warn "Cannot cd to $path:$!"
   }
- $path = File::Spec->canonpath($path);
  return $path;
 }
 
@@ -83,7 +75,7 @@ sub set_dir {
     my( $w, $val ) = @_;
     my $fulldir = fullpath( $val );
 
-    my $parent = $sep;
+    my $parent = '/';
     if ($^O eq 'MSWin32')
      {
       if ($fulldir =~ s/^([a-z]:)//i)
@@ -94,10 +86,11 @@ sub set_dir {
     $w->add_to_tree( $parent, $parent)  unless $w->infoExists($parent);
 
     my @dirs = ($parent);
-    foreach my $name (split( /\Q$sep\E/, $fulldir )) {
+    foreach my $name (split( /[\/\\]/, $fulldir )) {
         next unless length $name;
         push @dirs, $name;
-	my $dir = File::Spec->catfile( @dirs );
+        my $dir = join( '/', @dirs );
+	$dir =~ s|^//|/|;
         $w->add_to_tree( $dir, $name, $parent )
             unless $w->infoExists( $dir );
         $parent = $dir;
@@ -113,10 +106,10 @@ sub OpenCmd {
     my( $w, $dir ) = @_;
 
     my $parent = $dir;
+    $dir = '' if $dir eq '/';
     foreach my $name ($w->dirnames( $parent )) {
         next if ($name eq '.' || $name eq '..');
-        my $subdir = File::Spec->catfile( $dir, $name );
-	$subdir = _fs_encode($subdir);
+        my $subdir = "$dir/$name";
         next unless -d $subdir;
         if( $w->infoExists( $subdir ) ) {
             $w->show( -entry => $subdir );
@@ -131,7 +124,6 @@ sub OpenCmd {
 sub add_to_tree {
     my( $w, $dir, $name, $parent ) = @_;
 
-    my $dir8 = _fs_encode($dir);
     my $image = $w->cget('-image');
     if ( !UNIVERSAL::isa($image, 'Tk::Image') ) {
 	$image = $w->Getimage( $image );
@@ -142,9 +134,7 @@ sub add_to_tree {
     my @args = (-image => $image, -text => $name);
     if( $parent ) {             # Add in alphabetical order.
         foreach my $sib ($w->infoChildren( $parent )) {
-	    use if $^O ne 'MSWin32', "locale"; # dumps core under Windows under some (japanese?) locales, see http://www.nntp.perl.org/group/perl.cpan.testers/2008/11/msg2550386.html
-	    my $sib8 = _fs_encode($sib);
-	    if ($sib8 gt $dir8) {
+            if( $sib gt $dir ) {
                 push @args, (-before => $sib);
                 last;
             }
@@ -160,7 +150,7 @@ sub has_subdir {
     foreach my $name ($w->dirnames( $dir )) {
         next if ($name eq '.' || $name eq '..');
         next if ($name =~ /^\.+$/);
-        return( 1 ) if -d File::Spec->catfile( $dir, $name );
+        return( 1 ) if -d "$dir/$name";
     }
     return( 0 );
 }
@@ -178,7 +168,7 @@ sub dirnames {
 
     sub Populate {
 	my($w, $args) = @_;
-	$w->{curr_dir} = delete $args->{-initialdir};
+	$w->{curr_dir} = $args->{-initialdir};
 	if (!defined $w->{curr_dir}) {
 	    require Cwd;
 	    $w->{curr_dir} = Cwd::cwd();
@@ -197,7 +187,7 @@ sub dirnames {
 	my $f = $w->Frame->pack(-fill => "x", -side => "bottom");
 
 	my $d;
-	$d = $w->Scrolled('DirTree',
+	$d = $f->Scrolled('DirTree',
 			  -scrollbars => 'osoe',
 			  -width => 35,
 			  -height => 20,
@@ -205,6 +195,9 @@ sub dirnames {
 			  -exportselection => 1,
 			  -browsecmd => sub {
 			      $w->{curr_dir} = shift;
+			      if ($^O ne 'MSWin32') {
+				  $w->{curr_dir} =~ s|^//|/|; # bugfix
+			      }
 			  },
 
 			  # With this version of -command a double-click will
@@ -217,7 +210,7 @@ sub dirnames {
 			  #-command   => sub { $d->opencmd($_[0]) },
 			 )->pack(-fill => "both", -expand => 1);
 	# Set the initial directory
-	$d->set_dir($w->{curr_dir});
+	exists &Tk::DirTree::chdir ? $d->chdir($w->{curr_dir}) : $d->set_dir($w->{curr_dir});
 
 	$f->Button(-text => 'Ok',
 		   -command => sub { $w->{ok} =  1 })->pack(-side => 'left');
