@@ -9,8 +9,7 @@ package Tk::NoteBook;
 
 use vars qw($VERSION);
 
-#$VERSION = sprintf '4.%03d', q$Revision: #9 $ =~ /\D(\d+)\s*$/;
-$VERSION = '4.010';
+$VERSION = sprintf '4.%03d', q$Revision: #9 $ =~ /\D(\d+)\s*$/;
 require Tk::NBFrame;
 
 use base  qw(Tk::Derived Tk::NBFrame);
@@ -232,7 +231,7 @@ sub delete
   {
    $childw->bind('<Destroy>', undef);
    $childw->destroy;
-   @{$w->{'windows'}} = grep($_ ne $child, @{$w->{'windows'}});
+   @{$w->{'windows'}} = grep($_ !~ /$child/, @{$w->{'windows'}});
    $w->{'nWindows'}--;
    $w->SUPER::delete($child);
    # see if the child to be deleted was the top child
@@ -450,3 +449,4 @@ sub InitTabSize {
 
 __END__
 
+=cut
