@@ -7,7 +7,7 @@ package Tk::Button;
 # This program is free software; you can redistribute it and/or
 
 use vars qw($VERSION);
-$VERSION = '4.009'; # $Id: //depot/Tkutf8/Tk/Button.pm#8 $
+$VERSION = '4.008'; # $Id: //depot/Tkutf8/Tk/Button.pm#8 $
 
 # modify it under the same terms as Perl itself, subject
 # to additional disclaimer in license.terms due to partial
@@ -136,12 +136,6 @@ sub Invoke
   }
 }
 
-# Used for Tk::Widget::AmpWidget
-sub AmpWidgetPostHook
-{
- my $w = shift;
- $w->bind('<<AltUnderlined>>' => ['invoke']);
-}
 
 
 1;
