@@ -33,16 +33,13 @@
 #       be called directly. Call tk_getOpenFile or tk_getSaveFile instead.
 #
 
-use 5.005; # qr//
-
 package Tk::FBox;
 require Tk::Toplevel;
 
 use strict;
 use vars qw($VERSION $updirImage $folderImage $fileImage);
 
-#$VERSION = sprintf '4.%03d', q$Revision: #18 $ =~ /\D(\d+)\s*$/;
-$VERSION = '4.021';
+$VERSION = sprintf '4.%03d', q$Revision: #18 $ =~ /\D(\d+)\s*$/;
 
 use base qw(Tk::Toplevel);
 
@@ -81,8 +78,6 @@ sub Populate {
 
     $w->SUPER::Populate($args);
 
-    $w->{'encoding'} = $w->getEncoding('iso_8859_1');
-
     # f1: the frame with the directory option menu
     my $f1 = $w->Frame;
     my $lab = $f1->Label(-text => 'Directory:', -underline => 0);
@@ -332,7 +327,7 @@ sub Show {
 #XXX use Tk::setFocusGrab when it's available
     my $oldFocus = $w->focusCurrent;
     my $oldGrab = $w->grabCurrent;
-    my $grabStatus; $grabStatus = $oldGrab->grabStatus if ($oldGrab);
+    my $grabStatus = $oldGrab->grabStatus if ($oldGrab);
     $w->grab;
     my $ent = $w->{'ent'};
     $ent->focus;
@@ -407,14 +402,14 @@ sub Update {
     my $folder = $folderImage->{$w->MainWindow};
     my $file   = $fileImage->{$w->MainWindow};
     my $appPWD = _cwd();
-    if (!ext_chdir($w->_get_select_path)) {
+    if (!ext_chdir($w->{'selectPath'})) {
 	# We cannot change directory to $data(selectPath). $data(selectPath)
 	# should have been checked before tkFDialog_Update is called, so
 	# we normally won't come to here. Anyways, give an error and abort
 	# action.
 	$w->messageBox(-type => 'OK',
 		       -message => 'Cannot change to the directory "' .
-		       $w->_get_select_path . "\".\nPermission denied.",
+		       $w->{'selectPath'} . "\".\nPermission denied.",
 		       -icon => 'warning',
 		      );
 	ext_chdir($appPWD);
@@ -453,7 +448,7 @@ sub Update {
 	    if ($fltcb) {
 		next if !$fltcb->($w, $f, $cwd);
 	    } else {
-		next if !-d $f && $f !~ m!$flt!s;
+		next if -f $f && $f !~ m!$flt!;
 	    }
 	    if (-d $f) {
 		$icons->Add($folder, $f);
@@ -470,7 +465,7 @@ sub Update {
     # Update the Directory: option menu
     my @list;
     my $dir = '';
-    foreach my $subdir (TclFileSplit($w->_get_select_path)) {
+    foreach my $subdir (TclFileSplit($w->{'selectPath'})) {
 	$dir = TclFileJoin($dir, $subdir);
 	push @list, $dir;
     }
@@ -559,7 +554,7 @@ sub ResolveFile {
     # If the file has no extension, append the default.  Be careful not
     # to do this for directories, otherwise typing a dirname in the box
     # will give back "dirname.extension" instead of trying to change dir.
-    if (!-d $path && $path !~ /\..+$/s && defined $defaultext) {
+    if (!-d $path && $path !~ /\..+$/ && defined $defaultext) {
 	$path = "$path$defaultext";
     }
     # Cannot just test for existance here as non-existing files are
@@ -639,6 +634,8 @@ sub EntFocusOut {
 #
 sub ActivateEnt {
     my $w = shift;
+    my $ent = $w->{'ent'};
+    my $text = $ent->get;
     if ($w->cget(-multiple)) {
 	# For the multiple case we have to be careful to get the file
 	# names as a true list, watching out for a single file with a
@@ -646,11 +643,9 @@ sub ActivateEnt {
 
 	$w->{'selectFile'} = [];
 	for my $item ($w->{'icons'}->Curselection) {
-	    $w->VerifyFileName($w->_get_from_icons($item));
+	    $w->VerifyFileName($w->{'icons'}->Get($item));
 	}
     } else {
-	my $ent = $w->{'ent'};
-	my $text = $w->_encode_filename($ent->get);
 	$w->VerifyFileName($text);
     }
 }
@@ -662,7 +657,7 @@ sub VerifyFileName {
 #XXX leave this here?
 #    $text =~ s/^\s+//;
 #    $text =~ s/\s+$//;
-    my($flag, $path, $file) = ResolveFile($w->_get_select_path, $text,
+    my($flag, $path, $file) = ResolveFile($w->{'selectPath'}, $text,
 					  $w->cget(-defaultextension));
     my $ent = $w->{'ent'};
     if ($flag eq 'OK') {
@@ -732,8 +727,8 @@ sub InvokeBtn {
 #
 sub UpDirCmd {
     my $w = shift;
-    $w->SetPath(File::Basename::dirname($w->_get_select_path))
-      unless ($w->_get_select_path eq '/');
+    $w->SetPath(File::Basename::dirname($w->{'selectPath'}))
+      unless ($w->{'selectPath'} eq '/');
 }
 
 # Join a file name to a path name. The "file join" command will break
@@ -747,13 +742,14 @@ sub JoinFile {
     }
 }
 
-# XXX replace with File::Spec if possible
+# XXX replace with File::Spec when perl/Tk depends on 5.005
 sub TclFileJoin {
     my $path = '';
     foreach (@_) {
 	if (m|^/|) {
 	    $path = $_;
-	} elsif (m|^[a-z]:/|i) {  # DOS-ish
+	}
+	elsif (m|^[a-z]:/|i) {  # DOS-ish
 	    $path = $_;
 	} elsif ($_ eq '~') {
 	    $path = _get_homedir();
@@ -796,17 +792,16 @@ sub OkCmd {
 
     my $filenames = [];
     for my $item ($w->{'icons'}->Curselection) {
-	push @$filenames, $w->_get_from_icons($item);
+	push @$filenames, $w->{'icons'}->Get($item);
     }
 
     my $filename = $filenames->[0];
-    $filename = "" if !defined $filename;
     if ($w->cget('-type') eq 'dir' && $from ne "iconlist") {
-	my $file = $filename eq '' ? $w->_get_select_path : JoinFile($w->_get_select_path, $filename);
+	my $file = $filename eq '' ? $w->{'selectPath'} : JoinFile($w->{'selectPath'}, $filename);
 	$w->Done($file);
     } elsif ((@$filenames && !$w->cget('-multiple')) ||
 	($w->cget('-multiple') && @$filenames == 1)) {
-	my $file = JoinFile($w->_get_select_path, $filename);
+	my $file = JoinFile($w->{'selectPath'}, $filename);
 	if (-d $file) {
 	    $w->ListInvoke($filename);
 	    return;
@@ -831,14 +826,14 @@ sub ListBrowse {
 
     my $text = [];
     for my $item ($w->{'icons'}->Curselection) {
-	push @$text, $w->_get_from_icons($item);
+	push @$text, $w->{'icons'}->Get($item);
     }
     return if @$text == 0;
     my $isDir;
     if (@$text > 1) {
 	my $newtext = [];
 	for my $file (@$text) {
-	    my $fullfile = JoinFile($w->_get_select_path, $file);
+	    my $fullfile = JoinFile($w->{'selectPath'}, $file);
 	    if (!-d $fullfile) {
 		push @$newtext, $file;
 	    }
@@ -846,7 +841,7 @@ sub ListBrowse {
 	$text = $newtext;
 	$isDir = 0;
     } else {
-	my $file = JoinFile($w->_get_select_path, $text->[0]);
+	my $file = JoinFile($w->{'selectPath'}, $text->[0]);
 	$isDir = -d $file;
     }
     my $ent = $w->{'ent'};
@@ -871,7 +866,7 @@ sub ListBrowse {
 sub ListInvoke {
     my($w, @filenames) = @_;
     return if !@filenames;
-    my $file = JoinFile($w->_get_select_path, $filenames[0]);
+    my $file = JoinFile($w->{'selectPath'}, $filenames[0]);
     if (-d $file) {
 	my $appPWD = _cwd();
 	if (!ext_chdir($file)) {
@@ -907,10 +902,10 @@ sub Done {
 	if ($w->cget('-multiple')) {
 	    $selectFilePath = [];
 	    for my $f (@{ $w->{'selectFile'} }) {
-		push @$selectFilePath, JoinFile($w->_get_select_path, $f);
+		push @$selectFilePath, JoinFile($w->{'selectPath'}, $f);
 	    }
 	} else {
-	    $selectFilePath = JoinFile($w->_get_select_path,
+	    $selectFilePath = JoinFile($w->{'selectPath'},
 				       $w->{'selectFile'});
 	}
 	if ($w->cget(-type) eq 'save' and
@@ -993,9 +988,9 @@ sub ext_chdir {
     my $dir = shift;
     if ($dir eq '~') {
 	chdir _get_homedir();
-    } elsif ($dir =~ m|^~/(.*)|s) {
+    } elsif ($dir =~ m|^~/(.*)|) {
 	chdir _get_homedir() . "/" . $1;
-    } elsif ($dir =~ m|^~([^/]+(.*))|s) {
+    } elsif ($dir =~ m|^~([^/]+(.*))|) {
 	chdir _get_homedir($1) . $2;
     } else {
 	chdir $dir;
@@ -1034,28 +1029,15 @@ sub _untaint {
 
 sub _rx_to_glob {
     my $arg = shift;
-    $arg =~ s!([.+^()|\${}\[\]\\])!\\$1!g;
-    $arg = join('|', map {"^$_\\z"} split(' ', $arg));
-    $arg =~ s!\*!.*!g;$arg =~ s!\?!.!g;
-    $arg = qr/$arg/s;
+    $arg = join('|', split(' ', $arg));
+    $arg =~ s!([\.\+])!\\$1!g;
+    $arg =~ s!\*!.*!g;
+    $arg = "^" . $arg . "\$";
+    if ($] >= 5.005) {
+	$arg = qr/$arg/;
+    }
     $arg;
 }
 
-sub _get_from_icons {
-    my($w, $item) = @_;
-    $w->_encode_filename($w->{'icons'}->Get($item));
-}
-
-sub _get_select_path {
-    my($w) = @_;
-    $w->_encode_filename($w->{'selectPath'});
-}
-
-sub _encode_filename {
-    my($w, $filename) = @_;
-    $filename = $w->{encoding}->encode($filename);
-    $filename;
-}
-
 1;
 
