@@ -9,7 +9,7 @@ use Carp;
 use File::Basename;
 
 use vars qw($VERSION);
-$VERSION = '4.024';
+$VERSION = sprintf '4.%03d', q$Revision: #21 $ =~ /\D(\d+)\s*$/;
 
 # warn __FILE__." $VERSION\n";
 
@@ -259,12 +259,10 @@ sub perldepend
       {
        push(@def,qw(-D__WIN32__)) unless $self->{'DEFINE'} =~ /-D__WIN32__/;
        push(@def,qw(-DWIN32)) if $self->{'NAME'} eq 'Tk::pTk';
-       push(@def,qw(-dWINARCH_MSWIN32));
       }
      elsif ($win_arch eq 'x')
       {
        push(@def,qw(-U_WIN32));
-       push(@def,qw(-DWINARCH_X));
       }
     }
    foreach (@inc)
@@ -297,20 +295,15 @@ sub const_config
   {
    $self->{'LDDLFLAGS'} =~ s/-flat_namespace//;
    $self->{'LDDLFLAGS'} =~ s/-undefined\s+suppress//;
-## These lines seem to be not necessary for Panther, both
-## builds with and without shared libperl, and seem to
-## be dangerous for other MacOSX versions using perl builds
-## without shared libperl, so disabled completely.
-## See http://rt.cpan.org/Public/Bug/Display.html?id=39593
-#   if ( -e "$Config{'archlib'}/CORE/$Config{'libperl'}" ) {
-#    $self->{'LDDLFLAGS'} .= " -L\${PERL_ARCHLIB}/CORE -lperl ";
-#   }
-#   elsif ( -e "/System/Library/Perl/darwin/CORE/libperl.dylib" ) {
-#    $self->{'LDDLFLAGS'} .= " -L/System/Library/Perl/darwin/CORE -lperl ";
-#   }
-#   else {
-#    warn "Can't find libperl.dylib";
-#   }
+   if ( -e "$Config{'archlib'}/CORE/$Config{'libperl'}" ) {
+    $self->{'LDDLFLAGS'} .= " -L\${PERL_ARCHLIB}/CORE -lperl ";
+   }
+   elsif ( -e "/System/Library/Perl/darwin/CORE/libperl.dylib" ) {
+    $self->{'LDDLFLAGS'} .= " -L/System/Library/Perl/darwin/CORE -lperl ";
+   }
+   else {
+    warn "Can't find libperl.dylib";
+   }
    $self->{'LDFLAGS'} =~ s/-flat_namespace//;
    $self->{'LDFLAGS'} =~ s/-undefined\s+suppress//;
   } elsif ($^O =~ /(openbsd)/i)
@@ -551,11 +544,11 @@ sub TkExtMakefile
      $i = "-I$tk $i";
      $i = "-I$ptk $i";
     }
-   if ($IsWin32 && $Config{'cc'} =~ /gcc/i)
+   if ($IsWin32 && $Config{'cc'} =~ /^gcc/i)
     {
      my $base  = $Config{'libpth'};
-     #$base =~ s#lib$#i386-mingw32/lib#;
-     my $extra = "-L$base -limm32 -lcomctl32 -lcomdlg32 -lgdi32";
+     $base =~ s#lib$#i386-mingw32/lib#;
+     my $extra = "-L$base -lcomdlg32 -lgdi32";
      my $libs = $att{'LIBS'}->[0];
      $att{'LIBS'}->[0] = "$extra $libs";
     }
@@ -567,15 +560,15 @@ sub TkExtMakefile
      push(@opt,'LD' => 'gcc -shared');
      if ($win_arch eq 'MSWin32')
       {
-       my $extra = "-L/lib/w32api -limm32 -lcomctl32 -lcomdlg32 -lgdi32";
+       my $extra = "-L/lib/w32api -lcomdlg32 -lgdi32";
        my $libs = $att{'LIBS'}->[0];
        $att{'LIBS'}->[0] = "$extra $libs";
-       $att{'DEFINE'} .= ' -D__WIN32__ -D_WIN32 -DWINARCH_MSWIN32';
+       $att{'DEFINE'} .= ' -D__WIN32__ -D_WIN32';
        $att{'DEFINE'} .= ' -DWIN32' if($att{'NAME'} eq 'Tk::pTk');
       }
      elsif ($win_arch eq 'x')
       {
-       $att{'DEFINE'} .= ' -U_WIN32 -DWINARCH_X';
+       $att{'DEFINE'} .= ' -U_WIN32';
       }
     }
    if (delete $att{'ptk_include'})
