@@ -21,7 +21,7 @@ require Tk::Menu::Item;
 
 
 use vars qw($VERSION);
-$VERSION = '4.022'; # was: sprintf '4.%03d', q$Revision: #21 $ =~ /\D(\d+)\s*$/;
+$VERSION = sprintf '4.%03d', q$Revision: #21 $ =~ /\D(\d+)\s*$/;
 
 use strict;
 
@@ -404,9 +404,8 @@ sub Motion
 sub ButtonDown
 {
  my $menu = shift;
- return if (!$menu->viewable);
  $menu->postcascade('active');
- if (defined $Tk::postedMb && $Tk::postedMb->viewable)
+ if (defined $Tk::postedMb)
   {
    $Tk::postedMb->grabGlobal
   }
@@ -516,8 +515,8 @@ sub Invoke
   }
  elsif ($w->typeIS('active','tearoff'))
   {
-   $w->tearOffMenu();
    $w->Unpost();
+   $w->tearOffMenu();
   }
  elsif ($w->typeIS('active','menubar'))
   {
@@ -603,14 +602,15 @@ sub NextMenu
    my $m2 = $menu->parent;
    if ($m2->IsMenu)
     {
-     $menu->activate('none');
-     $menu->GenerateMenuSelect;
-     $m2->SetFocus;
-
-     $m2->postcascade('none');
-
      if ($m2->cget('-type') ne 'menubar')
       {
+       $menu->activate('none');
+       $menu->GenerateMenuSelect;
+       $m2->SetFocus;
+       # This code unposts any posted submenu in the parent.
+       my $tmp = $m2->index('active');
+       $m2->activate('none');
+       $m2->activate($tmp);
        return;
       }
     }
@@ -796,7 +796,7 @@ sub FirstEntry
      if ($menu->type($i) eq 'cascade')
       {
        my $cascade = $menu->entrycget($i,'-menu');
-       if (0 && defined $cascade)
+       if (defined $cascade)
         {
          $menu->postcascade($i);
          $cascade->FirstEntry;
@@ -892,13 +892,10 @@ sub Post
  my $entry = shift;
  Unpost(undef) if (defined($Tk::popup) || defined($Tk::postedMb));
  $menu->PostOverPoint($x,$y,$entry);
- if ($Tk::platform eq 'unix' && $menu->viewable)
-  {
-   $menu->grabGlobal;
-   $Tk::popup = $menu;
-   $Tk::focus = $menu->focusCurrent;
-   $menu->focus();
-  }
+ $menu->grabGlobal;
+ $Tk::popup = $menu;
+ $Tk::focus = $menu->focusCurrent;
+ $menu->focus();
 }
 
 sub SetFocus
@@ -964,7 +961,7 @@ sub tearOffMenu
  # original: if the parent is a menu, then use the text of the active
  # entry. If it's a menubutton then use its text.
  my $title = $w->cget('-title');
- # print ref($w),' ',$w->PathName," ",$menu->PathName," $w\n";
+ # print ref($w),' ',$w->PathName," $w\n";
  unless (defined $title && length($title))
   {
    $parent = $w->parent;
@@ -982,12 +979,6 @@ sub tearOffMenu
   }
  $menu->title($title) if (defined $title && length($title));
  $menu->post($x,$y);
-
- if (!Tk::Exists($menu))
-  {
-   return;
-  }
-
  # Set tkPriv(focus) on entry: otherwise the focus will get lost
  # after keyboard invocation of a sub-menu (it will stay on the
  # submenu).
@@ -998,11 +989,6 @@ sub tearOffMenu
  # will get saved in $Tk::focus
  # $menu->bind('<Enter>','EnterFocus');
  $menu->Callback('-tearoffcommand');
-
- # Strangely tear-off menus do not work in tkpod and Tk804.027.
- # Explicitely setting normal state helps here - why?
- $menu->state("normal");
-
  return $menu;
 }
 
@@ -1152,48 +1138,6 @@ sub BalloonInfo
   }
 }
 
-sub MasterMenu
-{
- my ($menu) = @_;
- my $pathname = $menu->PathName;
- my $master_menu;
- if ($pathname =~ m{#})
-  {
-   my $master_pathname = (split m{\.}, $pathname)[-1];
-   $master_pathname =~ s{#}{.}g;
-   $master_menu = $menu->Widget($master_pathname);
-   if (!Tk::Exists($master_menu))
-    {
-     warn "Cannot find master menu <$master_pathname>";
-    }
-  }
- $master_menu;
-}
-
-
-# ::tk::AmpMenuArgs --
-# Processes arguments for a menu entry, turning -label option into
-# -label and -underline options, returned by ::tk::UnderlineAmpersand.
-#
-sub AmpArgs
-{
- my ($w, $add, $type, %args) = @_;
- my @options;
- while(my($opt,$val) = each %args)
-  {
-   if ($opt eq "-label")
-    {
-     my ($newtext,$under) = $w->UnderlineAmpersand($val);
-     push @options, -label => $newtext, -underline => $under;
-    }
-   else
-    {
-     push @options, $opt, $val;
-    }
-  }
- $w->$type(@options);
-}
-
 1;
 
 __END__
