@@ -1,7 +1,7 @@
 package Tk::ProgressBar;
 
 use vars qw($VERSION);
-$VERSION = '4.013'; # was: sprintf '4.%03d', q$Revision: #10 $ =~ /\D(\d+)\s*$/;
+$VERSION = sprintf '4.%03d', q$Revision: #10 $ =~ /\D(\d+)\s*$/;
 
 use Tk;
 use Tk::Canvas;
@@ -132,7 +132,7 @@ sub _arrange {
 	my $colors = $c->{Configure}{'-colors'} || [];
 	my $bdir = $from < $to ? $dir : 0 - $dir;
 
-	$c->delete('all');
+	$c->delete($c->find('all'));
 
 	$c->createRectangle(0,0,$w+$x*2,$h+$y*2,
 		-fill =>  $c->{Configure}{'-troughcolor'},
@@ -298,7 +298,7 @@ sub value {
 sub variable {
     my $c = shift;
     my $oldvarref = $c->{'-variable'};
-    my $oldval; $oldval = $$oldvarref if $oldvarref;
+    my $oldval = $$oldvarref if $oldvarref;
     if(@_) {
 	my $varref = shift;
         if ($oldvarref)
@@ -334,8 +334,8 @@ Tk::ProgressBar - A graphical progress bar
     use Tk::ProgressBar;
 
     $progress = $parent->ProgressBar(
-	-width => 20,
-	-length => 200,
+	-width => 200,
+	-length => 20,
         -anchor => 's',
 	-from => 0,
 	-to => 100,
@@ -450,7 +450,7 @@ full progress bar will be displayed. Defaults to 100.
 =item B<-variable>
 
 Specifies the reference to a scalar variable to link to the ProgressBar.
-Whenever the value of the variable changes, the ProgressBar will update
+Whenever the value of the variable changes, the ProgressBar will upate
 to reflect this value. (See also the B<value> method below.)
 
 =item B<-value>
