@@ -5,8 +5,9 @@ package Tk::Table;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '4.015'; # was: sprintf '4.%03d', q$Revision: #12 $ =~ /\D(\d+)\s*$/;
+$VERSION = sprintf '4.%03d', q$Revision: #12 $ =~ /\D(\d+)\s*$/;
 
+use Tk::Pretty;
 use AutoLoader;
 use base qw(Tk::Frame);
 
@@ -169,7 +170,6 @@ sub Layout
   {
    $t->{xsb} = $t->Scrollbar(-orient => 'horizontal', -command => ['xview' => $t]) unless (defined $t->{xsb});
    $xsb   = $t->{xsb};
-   $t->Advertise('xscrollbar' => $xsb);
    $xs[3] = $xsb->ReqHeight;
    if ($sb =~ /n/)
     {
@@ -191,7 +191,6 @@ sub Layout
   {
    $t->{ysb} = $t->Scrollbar(-orient => 'vertical', -command => ['yview' => $t]) unless (defined $t->{ysb});
    $ysb    = $t->{ysb};
-   $t->Advertise('yscrollbar' => $ysb);
    $ys[2]  = $ysb->ReqWidth;
    if ($sb =~ /w/)
     {
@@ -392,8 +391,8 @@ sub clear {
     my $self = shift;
     my $rows = $self->cget(-rows);
     my $cols = $self->cget(-columns);
-    foreach my $r (0 .. $rows-1) {
-	foreach my $c (0 .. $cols-1) {
+    foreach my $r (1 .. $rows) {
+	foreach my $c (1 .. $cols) {
 	    my $old = $self->get( $r, $c );
 	    next unless $old;
 	    $self->LostSlave($old);
@@ -595,3 +594,5 @@ sub see
  return $see;
 }
 
+=cut
+
