@@ -1,4 +1,4 @@
-# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2003 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::Wm;
@@ -14,11 +14,14 @@ use strict qw(vars);
 
 
 use vars qw($VERSION);
-$VERSION = '4.008'; # $Id: //depot/Tkutf8/Tk/Wm.pm#8 $
+$VERSION = sprintf '4.%03d', q$Revision: #14 $ =~ /\D(\d+)\s*$/;
 
-use Tk::Submethods ( 'wm' => [qw(grid tracing)] );
+use Tk::Submethods;
 
-Direct Tk::Submethods ('wm' => [qw(aspect client colormapwindows command
+*{Tk::Wm::wmGrid}    = sub { shift->wm("grid", @_) };
+*{Tk::Wm::wmTracing} = sub { shift->wm("tracing", @_) };
+
+Direct Tk::Submethods ('wm' => [qw(aspect attributes client colormapwindows command
                        deiconify focusmodel frame geometry group
                        iconbitmap iconify iconimage iconmask iconname
                        iconwindow maxsize minsize overrideredirect positionfrom
@@ -75,8 +78,8 @@ sub Post
  $X = int($X);
  $Y = int($Y);
  $w->positionfrom('user');
- # $w->geometry("+$X+$Y");
- $w->MoveToplevelWindow($X,$Y);
+ $w->geometry("+$X+$Y");
+ # $w->MoveToplevelWindow($X,$Y);
  $w->deiconify;
  $w->raise;
 }
@@ -124,6 +127,8 @@ sub Popup
  my ($X,$Y) = AnchorAdjust($w->cget('-overanchor'),$rx,$ry,$rw,$rh);
  ($X,$Y)    = AnchorAdjust($w->cget('-popanchor'),$X,$Y,-$mw,-$mh);
  # adjust to not cross screen borders
+ if ($X < 0) { $X = 0 }
+ if ($Y < 0) { $Y = 0 }
  if ($mw > $w->screenwidth)  { $X = 0 }
  if ($mh > $w->screenheight) { $Y = 0 }
  $w->Post($X,$Y);
