@@ -1,9 +1,9 @@
 /*
-  Copyright (c) 1997-1998 Nick Ing-Simmons. All rights reserved.
+  Copyright (c) 1997-2004 Nick Ing-Simmons. All rights reserved.
   This program is free software; you can redistribute it and/or
   modify it under the same terms as Perl itself.
 */
-/* #define PERL_NO_GET_CONTEXT */
+#define PERL_NO_GET_CONTEXT
 
 #include <EXTERN.h>
 #include <perl.h>
@@ -81,21 +81,23 @@ Tcl_IsShared(Tcl_Obj *objPtr)
 void
 Tcl_IncrRefCount(Tcl_Obj *objPtr)
 {
+ dTHX;
  SvREFCNT_inc(objPtr);
 }
 
 void
 Tcl_DecrRefCount(Tcl_Obj *objPtr)
 {
+ dTHX;
  SvREFCNT_dec(objPtr);
 }
 
-static SV *ForceScalar(SV *sv);
+static SV *ForceScalar(pTHX_ SV *sv);
 
-static SV *ForceScalarLvalue(SV *sv);
+static SV *ForceScalarLvalue(pTHX_ SV *sv);
 
 static void
-Scalarize(SV *sv, AV *av)
+Scalarize(pTHX_ SV *sv, AV *av)
 {
  int n    = av_len(av)+1;
  if (n == 0)
@@ -127,11 +129,13 @@ Scalarize(SV *sv, AV *av)
         {
          SV *el = *svp;
          int temp = 0;
-         if (SvROK(el) && SvTYPE(SvRV(el)) == SVt_PVAV)
+         if (SvROK(el) && !SvOBJECT(SvRV(el)) && SvTYPE(SvRV(el)) == SVt_PVAV)
           {
            el = newSVpv("",0);
            temp = 1;
-           Scalarize(el,(AV *) SvRV(*svp));
+           if ((AV *) SvRV(*svp) == av)
+            abort();
+           Scalarize(aTHX_ el,(AV *) SvRV(*svp));
           }
          Tcl_DStringAppendElement(&ds,Tcl_GetString(el));
          if (temp)
@@ -146,37 +150,37 @@ Scalarize(SV *sv, AV *av)
 }
 
 static SV *
-ForceScalar(SV *sv)
+ForceScalar(pTHX_ SV *sv)
 {
  if (SvGMAGICAL(sv))
   mg_get(sv);
  if (SvTYPE(sv) == SVt_PVAV)
   {
    AV *av = (AV *) sv;
-   SV *nsv = newSVpv("",0);
-   Scalarize(nsv, (AV *) av);
+   SV *newsv = newSVpv("",0);
+   Scalarize(aTHX_ newsv, (AV *) av);
    av_clear(av);
-   av_store(av,0,nsv);
-   return nsv;
+   av_store(av,0,newsv);
+   return newsv;
   }
  else
   {
-   if (SvROK(sv) && SvTYPE(SvRV(sv)) == SVt_PVAV)
+   if (SvROK(sv) && !SvOBJECT(SvRV(sv)) && SvTYPE(SvRV(sv)) == SVt_PVAV)
     {
      /* Callbacks and lists often get stringified by mistake due to
         Tcl/Tk's string fixation - don't change the real value
       */
-     SV *nsv = newSVpv("",0);
-     Scalarize(nsv, (AV *) SvRV(sv));
-     return sv_2mortal(nsv);
+     SV *newsv = newSVpv("",0);
+     Scalarize(aTHX_ newsv, (AV *) SvRV(sv));
+     return sv_2mortal(newsv);
     }
    else if (!SvOK(sv))
     {
      /* Map undef to null string */
      if (SvREADONLY(sv))
       {
-       SV *nsv = newSVpv("",0);
-       return sv_2mortal(nsv);
+       SV *newsv = newSVpv("",0);
+       return sv_2mortal(newsv);
       }
      else
       sv_setpvn(sv,"",0);
@@ -186,15 +190,15 @@ ForceScalar(SV *sv)
 }
 
 static SV *
-ForceScalarLvalue(SV *sv)
+ForceScalarLvalue(pTHX_ SV *sv)
 {
  if (SvTYPE(sv) == SVt_PVAV)
   {
    AV *av = (AV *) sv;
-   SV *nsv = newSVpv("",0);
+   SV *newsv = newSVpv("",0);
    av_clear(av);
-   av_store(av,0,nsv);
-   return nsv;
+   av_store(av,0,newsv);
+   return newsv;
   }
  else
   {
@@ -205,33 +209,38 @@ ForceScalarLvalue(SV *sv)
 void
 Tcl_SetBooleanObj (Tcl_Obj *objPtr, int value)
 {
- sv_setiv(ForceScalarLvalue(objPtr),value != 0);
+ dTHX;
+ sv_setiv(ForceScalarLvalue(aTHX_ objPtr),value != 0);
 }
 
 void
 Tcl_SetDoubleObj (Tcl_Obj *objPtr, double value)
 {
- sv_setnv(ForceScalarLvalue(objPtr),value);
+ dTHX;
+ sv_setnv(ForceScalarLvalue(aTHX_ objPtr),value);
 }
 
 void
 Tcl_SetIntObj (Tcl_Obj *objPtr, int value)
 {
- sv_setiv(ForceScalarLvalue(objPtr),value);
+ dTHX;
+ sv_setiv(ForceScalarLvalue(aTHX_ objPtr),value);
 }
 
 void
 Tcl_SetLongObj (Tcl_Obj *objPtr, long value)
 {
- sv_setiv(ForceScalarLvalue(objPtr),value);
+ dTHX;
+ sv_setiv(ForceScalarLvalue(aTHX_ objPtr),value);
 }
 
 void
 Tcl_SetStringObj (Tcl_Obj *objPtr, CONST char *bytes, int length)
 {
+ dTHX;
  if (length < 0)
   length = strlen(bytes);
- objPtr = ForceScalarLvalue(objPtr);
+ objPtr = ForceScalarLvalue(aTHX_ objPtr);
  sv_setpvn(objPtr, bytes, length);
  sv_maybe_utf8(objPtr);
 }
@@ -239,7 +248,8 @@ Tcl_SetStringObj (Tcl_Obj *objPtr, CONST char *bytes, int length)
 int
 Tcl_GetLongFromObj (Tcl_Interp *interp, Tcl_Obj *obj, long *longPtr)
 {
- SV *sv = ForceScalar(obj);
+ dTHX;
+ SV *sv = ForceScalar(aTHX_ obj);
  if (SvIOK(sv) || looks_like_number(sv))
   *longPtr = SvIV(sv);
  else
@@ -253,7 +263,8 @@ Tcl_GetLongFromObj (Tcl_Interp *interp, Tcl_Obj *obj, long *longPtr)
 int
 Tcl_GetBooleanFromObj (Tcl_Interp *interp, Tcl_Obj *obj, int *boolPtr)
 {
- SV *sv = ForceScalar(obj);
+ dTHX;
+ SV *sv = ForceScalar(aTHX_ obj);
  static char *yes[] = {"y", "yes", "true", "on", NULL};
  static char *no[] =  {"n", "no", "false", "off", NULL};
  if (SvPOK(sv))
@@ -286,7 +297,8 @@ Tcl_GetBooleanFromObj (Tcl_Interp *interp, Tcl_Obj *obj, int *boolPtr)
 int
 Tcl_GetIntFromObj (Tcl_Interp *interp, Tcl_Obj *obj, int *intPtr)
 {
- SV *sv = ForceScalar(obj);
+ dTHX;
+ SV *sv = ForceScalar(aTHX_ obj);
  if (SvIOK(sv) || looks_like_number(sv))
   *intPtr = SvIV(sv);
  else
@@ -300,7 +312,8 @@ Tcl_GetIntFromObj (Tcl_Interp *interp, Tcl_Obj *obj, int *intPtr)
 int
 Tcl_GetDoubleFromObj (Tcl_Interp *interp, Tcl_Obj *obj, double *doublePtr)
 {
- SV *sv = ForceScalar(obj);
+ dTHX;
+ SV *sv = ForceScalar(aTHX_ obj);
  if (SvNOK(sv) || looks_like_number(sv))
   *doublePtr = SvNV(sv);
  else
@@ -314,36 +327,42 @@ Tcl_GetDoubleFromObj (Tcl_Interp *interp, Tcl_Obj *obj, double *doublePtr)
 Tcl_Obj *
 Tcl_NewIntObj (int value)
 {
+ dTHX;
  return newSViv(value);
 }
 
 Tcl_Obj *
 Tcl_NewBooleanObj (int value)
 {
+ dTHX;
  return newSViv(value);
 }
 
 Tcl_Obj *
 Tcl_NewObj(void)
 {
+ dTHX;
  return newSVsv(&PL_sv_undef);
 }
 
 Tcl_Obj *
 Tcl_NewLongObj(long value)
 {
+ dTHX;
  return newSViv(value);
 }
 
 Tcl_Obj *
 Tcl_NewDoubleObj(double value)
 {
+ dTHX;
  return newSVnv(value);
 }
 
 Tcl_Obj *
 Tcl_NewStringObj (CONST char *bytes, int length)
 {
+ dTHX;
  if (bytes)
   {
    SV *sv;
@@ -360,6 +379,7 @@ Tcl_NewStringObj (CONST char *bytes, int length)
 Tcl_Obj *
 Tcl_NewListObj (int objc, Tcl_Obj *CONST objv[])
 {
+ dTHX;
  AV *av = newAV();
  if (objc)
   {
@@ -373,7 +393,7 @@ Tcl_NewListObj (int objc, Tcl_Obj *CONST objv[])
         */
        if (SvREFCNT(sv) <= 0 || SvTEMP(sv))
         {
-         LangDebug(__FUNCTION__ " %d:\n",objc);
+         LangDebug("%s %d:\n",__FUNCTION__, objc);
          sv_dump(sv);
         }
        av_store(av,objc,sv);
@@ -386,83 +406,121 @@ Tcl_NewListObj (int objc, Tcl_Obj *CONST objv[])
 static char *
 LangString(SV *sv)
 {
- STRLEN na;
+ dTHX;
  if (!sv)
   return "";
  if (SvGMAGICAL(sv)) mg_get(sv);
  if (SvPOK(sv))
-  return SvPV(sv, na);
+  {
+   if (!SvUTF8(sv))
+    sv_utf8_upgrade(sv);
+   return SvPV_nolen(sv);
+  }
  else
   {
    if (SvROK(sv))
     {
      SV *rv = SvRV(sv);
-     if (SvTYPE(rv) == SVt_PVCV || SvTYPE(rv) == SVt_PVAV)
-      return SvPV(sv, na);
-     else
+     STRLEN len;
+     char *s;
+     if (SvOBJECT(rv))
       {
-       if (SvOBJECT(rv))
+       /* Special case "our" objects and certainb legacy hacks ... */
+       if (SvTYPE(rv) == SVt_PVHV)
         {
-         if (SvTYPE(rv) == SVt_PVHV)
+         SV **p = hv_fetch((HV *) rv,"_TkValue_",9,0);
+         if (p)
           {
-           SV **p = hv_fetch((HV *) rv,"_TkValue_",9,0);
-           if (p)
+           return SvPV_nolen(*p);
+          }
+         else
+          {
+           Lang_CmdInfo *info = WindowCommand(sv, NULL, 0);
+           if (info)
             {
-             return SvPV(*p,na);
-            }
-           else
-            {
-             Lang_CmdInfo *info = WindowCommand(sv, NULL, 0);
-             if (info)
+             if (info->tkwin)
               {
-               if (info->tkwin)
-                {
-                 char *val = Tk_PathName(info->tkwin);
-                 hv_store((HV *) rv,"_TkValue_",9,Tcl_NewStringObj(val,strlen(val)),0);
-                 return val;
-                }
-               if (info->image)
-                {
-                 return SvPV(info->image,na);
-                }
+               char *val = Tk_PathName(info->tkwin);
+               hv_store((HV *) rv,"_TkValue_",9,Tcl_NewStringObj(val,strlen(val)),0);
+               return val;
+              }
+             if (info->image)
+              {
+               return SvPV_nolen(info->image);
               }
             }
           }
-         else if (SvPOK(rv))
-          {
-#ifdef SvUTF8
-           if (!SvUTF8(sv))
-            sv_utf8_upgrade(rv);
-#endif
-           return SvPV(rv,na);
-          }
-         else
-          LangDumpVec("Odd object type", 1, &rv);
+        }
+       else if (SvPOK(rv))
+        {
+         /* ref to string is special cased for some reason ? */
+         if (!SvUTF8(rv))
+          sv_utf8_upgrade(rv);
+         return SvPV_nolen(rv);
+        }
+      } /* Object */
+     s = SvPV(sv, len);
+     if (!is_utf8_string(s,len))
+      {
+       sv_setpvn(sv,s,len);
+       sv_utf8_upgrade(sv);
+       s = SvPV(sv, len);
+      }
+     if (!is_utf8_string(s,len))
+      {
+       LangDebug("%s @ %d not utf8 '%.*s'\n",__FUNCTION__,__LINE__,(int) len, s);
+       sv_dump(sv);
+       abort();
+      }
+     return s;
+    } /* reference */
+   else if (SvOK(sv))
+    {
+     if (SvROK(sv) && SvPOK(SvRV(sv)) && !SvUTF8(SvRV(sv)))
+      sv_utf8_upgrade(SvRV(sv));
+     else if (SvPOKp(sv) && !SvPOK(sv))
+      {
+       if (SvTYPE(sv) == SVt_PVLV && !SvUTF8(sv))
+        {
+         /* LVs e.g. substr() don't upgrade */
+         SV *copy = newSVsv(sv);
+         sv_utf8_upgrade(copy);
+         sv_setsv(sv,copy);
+         SvREFCNT_dec(copy);
+        }
+       else
+        {
+         /* Slaven's for magical (tied) SVs with only SvPOKp */
+         SvPOK_on(sv);
+         sv_utf8_upgrade(sv);
+         SvPOK_off(sv);
+         SvPOKp_on(sv);
         }
       }
-    }
-   if (SvOK(sv))
-    {
-#ifdef SvUTF8
-     if (!SvUTF8(sv))
-      sv_utf8_upgrade(sv);
-#endif
-     return SvPV(sv, na);
+     return SvPVutf8_nolen(sv);
     }
    else
     return "";
   }
 }
 
+void utf8Whoops(pTHX_ SV *objPtr)
+{
+ sv_utf8_upgrade(objPtr);
+ sv_dump(objPtr);
+}
+
 char *
 Tcl_GetStringFromObj (Tcl_Obj *objPtr, int *lengthPtr)
 {
  if (objPtr)
   {
+   dTHX;
    char *s;
-   if ((SvROK(objPtr) && SvTYPE(SvRV(objPtr)) == SVt_PVAV) ||
-        (SvTYPE(objPtr) == SVt_PVAV))
-    objPtr = ForceScalar(objPtr);
+   if ((SvROK(objPtr) && !SvOBJECT(SvRV(objPtr))
+        && SvTYPE(SvRV(objPtr)) == SVt_PVAV) ||
+       (SvTYPE(objPtr) == SVt_PVAV))
+    objPtr = ForceScalar(aTHX_ objPtr);
    if (SvPOK(objPtr))
     {
      STRLEN len;
@@ -474,9 +532,21 @@ Tcl_GetStringFromObj (Tcl_Obj *objPtr, int *lengthPtr)
 #ifdef SvUTF8
      if (!is_utf8_string(s,len))
       {
-       LangDebug(__FUNCTION__ " not utf8\n");
+       LangDebug("%s @ %d not utf8\n",__FUNCTION__,__LINE__);
        sv_dump(objPtr);
-       croak("Not UTF-8");
+       utf8Whoops(aTHX_ objPtr);
+       s = SvPV(objPtr, len);
+       if (!is_utf8_string(s,len))
+        {
+         U8 *p = (U8 *) s;
+	 U8 *e = p + len;
+	 while (p < e)
+	  {
+	   if (*p > 0x7F)
+	    *p = '?';
+	   p++;
+	  }
+	}
       }
 #endif
      if (lengthPtr)
@@ -488,7 +558,7 @@ Tcl_GetStringFromObj (Tcl_Obj *objPtr, int *lengthPtr)
 #ifdef SvUTF8
      if (!is_utf8_string(s,strlen(s)))
       {
-       LangDebug(__FUNCTION__ " not utf8\n");
+       LangDebug("%s @ %d not utf8\n",__FUNCTION__,__LINE__);
        sv_dump(objPtr);
        abort();
       }
@@ -508,9 +578,18 @@ Tcl_GetString(Tcl_Obj *objPtr)
  return Tcl_GetStringFromObj(objPtr, NULL);
 }
 
+unsigned char *
+Tcl_GetByteArrayFromObj(Tcl_Obj * objPtr, int * lengthPtr)
+{
+ /* FIXME: presumably should downgrade from UTF-8,
+    what frees it ?
+  */
+ return (unsigned char *) Tcl_GetStringFromObj (objPtr, lengthPtr);
+}
+
 
 AV *
-ForceList(Tcl_Interp *interp, Tcl_Obj *sv)
+ForceList(pTHX_ Tcl_Interp *interp, Tcl_Obj *sv)
 {
  if (SvTYPE(sv) == SVt_PVAV)
   {
@@ -613,7 +692,8 @@ ForceList(Tcl_Interp *interp, Tcl_Obj *sv)
 void
 Tcl_SetListObj(Tcl_Obj * objPtr,int objc, Tcl_Obj *CONST objv[])
 {
- AV *av = ForceList(NULL,objPtr);
+ dTHX;
+ AV *av = ForceList(aTHX_ NULL,objPtr);
  av_clear(av);
  while (objc-- > 0)
   {
@@ -628,7 +708,8 @@ int
 Tcl_ListObjAppendElement (Tcl_Interp *interp, Tcl_Obj *listPtr,
 			    Tcl_Obj *objPtr)
 {
- AV *av = ForceList(interp,listPtr);
+ dTHX;
+ AV *av = ForceList(aTHX_ interp,listPtr);
  if (!objPtr)
   objPtr = &PL_sv_undef;
  if (av)
@@ -639,8 +720,29 @@ Tcl_ListObjAppendElement (Tcl_Interp *interp, Tcl_Obj *listPtr,
  return TCL_ERROR;
 }
 
+void
+Tcl_AppendElement(interp, string)
+Tcl_Interp *interp;
+CONST char *string;
+{
+ dTHX;
+ Tcl_Obj *result = Tcl_GetObjResult(interp);
+ Tcl_Obj *value  = Tcl_NewStringObj(string,-1);
+ if (1 || SvOK(result))
+  {
+   Tcl_ListObjAppendElement(interp,result,value);
+  }
+ else
+  {
+   SvSetMagicSV(result, value);
+   LangDumpVec(__FUNCTION__,1,&result);
+  }
+}
+
+
+
 AV *
-MaybeForceList(Tcl_Interp *interp, Tcl_Obj *sv)
+MaybeForceList(pTHX_ Tcl_Interp *interp, Tcl_Obj *sv)
 {
  AV *av;
  int object = sv_isobject(sv);
@@ -658,18 +760,20 @@ MaybeForceList(Tcl_Interp *interp, Tcl_Obj *sv)
  else if (SvREADONLY(sv))
   {
    /* returns mortal list anyway */
-   return ForceList(interp,sv);
+   return ForceList(aTHX_ interp,sv);
   }
  else
   {
    SvREADONLY_on(sv);
-   av = ForceList(interp,sv);
+   av = ForceList(aTHX_ interp,sv);
    SvREADONLY_off(sv);
    /* If there was more than one element set the SV */
    if (av && av_len(av) > 0)
     {
      /* AV is mortal - so we want newRV not MakeReference as we need extra REFCNT */
-     SvSetMagicSV(sv,newRV((SV *) av));
+     SV *ref = newRV((SV *) av);
+     SvSetMagicSV(sv,ref);
+     SvREFCNT_dec(ref);
     }
    return av;
   }
@@ -681,7 +785,8 @@ Tcl_ListObjGetElements (Tcl_Interp *interp, Tcl_Obj *listPtr,
 {
  if (listPtr)
   {
-   AV *av = MaybeForceList(interp,listPtr);
+   dTHX;
+   AV *av = MaybeForceList(aTHX_ interp,listPtr);
    if (av)
     {
      *objcPtr = av_len(av)+1;
@@ -698,7 +803,8 @@ int
 Tcl_ListObjIndex (Tcl_Interp *interp,  Tcl_Obj *listPtr, int index,
 			    Tcl_Obj **objPtrPtr)
 {
- AV *av = ForceList(interp,listPtr);
+ dTHX;
+ AV *av = ForceList(aTHX_ interp,listPtr);
  if (av)
   {
    SV **svp = av_fetch(av, index, 0);
@@ -715,7 +821,8 @@ Tcl_ListObjIndex (Tcl_Interp *interp,  Tcl_Obj *listPtr, int index,
 int
 Tcl_ListObjLength (Tcl_Interp *interp, Tcl_Obj *listPtr, int *intPtr)
 {
- AV *av = ForceList(interp,listPtr);
+ dTHX;
+ AV *av = ForceList(aTHX_ interp,listPtr);
  if (av)
   {
    *intPtr = av_len(av)+1;
@@ -728,12 +835,20 @@ int
 Tcl_ListObjReplace (Tcl_Interp *interp, Tcl_Obj *listPtr, int first, int count,
 			    int objc, Tcl_Obj *CONST objv[])
 {
- AV *av = ForceList(interp,listPtr);
+ dTHX;
+ AV *av = ForceList(aTHX_ interp,listPtr);
  if (av)
   {
    int len = av_len(av)+1;
-   int newlen = len-count+objc;
+   int newlen;
    int i;
+   if (first < 0)
+    first = 0;
+   if (first > len)
+     first = len;
+   if (first + count > len)
+    count = first-len;
+   newlen = len-count+objc;
    if (newlen > len)
     {
      /* Move entries beyond old range up to make room for new */
@@ -741,7 +856,8 @@ Tcl_ListObjReplace (Tcl_Interp *interp, Tcl_Obj *listPtr, int first, int count,
      for (i=len-1; i >= (first+count); i--)
       {
        SV **svp = av_fetch(av,i,0);
-       av_store(av,i+newlen-len,SvREFCNT_inc(*svp));
+       if (svp)
+        av_store(av,i+newlen-len,SvREFCNT_inc(*svp));
       }
     }
    else if (newlen < len)
@@ -750,7 +866,8 @@ Tcl_ListObjReplace (Tcl_Interp *interp, Tcl_Obj *listPtr, int first, int count,
      for (i=first+count; i < len; i++)
       {
        SV **svp = av_fetch(av,i,0);
-       av_store(av,i+newlen-len,SvREFCNT_inc(*svp));
+       if (svp)
+        av_store(av,i+newlen-len,SvREFCNT_inc(*svp));
       }
 #ifdef AvFILLp
      AvFILLp(av) = newlen-1;
@@ -774,10 +891,11 @@ Tcl_ListObjReplace (Tcl_Interp *interp, Tcl_Obj *listPtr, int first, int count,
 int
 Tcl_ListObjAppendList(Tcl_Interp * interp, Tcl_Obj * listPtr,Tcl_Obj * elemListPtr)
 {
+ dTHX;
  Tcl_Obj **objv;
  int objc = 0;
  int code;
- AV *av = ForceList(interp,listPtr);
+ AV *av = ForceList(aTHX_ interp,listPtr);
  if ((code = Tcl_ListObjGetElements(interp,elemListPtr,&objc,&objv)) == TCL_OK)
   {
    dTHX;
@@ -802,6 +920,7 @@ Tcl_ConcatObj (int objc, Tcl_Obj *CONST objv[])
     set way Tcl_NewListObj() is expecting. So correct that
     then call Tcl_NewListObj().
   */
+ dTHX;
  int i;
  for (i=0; i < objc; i++)
   {
@@ -845,9 +964,10 @@ Tcl_DStringAppendElement(dsPtr, string)
 void
 Tcl_AppendStringsToObj (Tcl_Obj *obj,...)
 {
+ dTHX;
  va_list ap;
  char *s;
- SV *sv = ForceScalar(obj);
+ SV *sv = ForceScalar(aTHX_ obj);
  va_start(ap,obj);
  while ((s = va_arg(ap,char *)))
   {
@@ -855,7 +975,9 @@ Tcl_AppendStringsToObj (Tcl_Obj *obj,...)
   }
  va_end(ap);
  if (sv != obj && SvROK(obj))
-  SvSetMagicSV(obj,sv);
+  {
+   SvSetMagicSV(obj,sv);
+  }
 }
 
 /*
@@ -964,7 +1086,8 @@ Tcl_AppendToObj(objPtr, bytes, length)
 				 * "bytes". If < 0, then append all bytes
 				 * up to NULL byte. */
 {
- SV *sv = ForceScalar(objPtr);
+ dTHX;
+ SV *sv = ForceScalar(aTHX_ objPtr);
  int hi;
  if (length < 0)
   length = strlen(bytes);
@@ -1026,7 +1149,7 @@ Tcl_WrongNumArgs(interp, objc, objv, message)
 }
 
 
-#define DStringSV(svp) ((*svp) ? (*svp = ForceScalar(*svp)) : (*svp = newSVpv("",0), *svp))
+#define DStringSV(svp) ((*svp) ? (*svp = ForceScalar(aTHX_ *svp)) : (*svp = newSVpv("",0), *svp))
 
 #undef Tcl_DStringInit
 void
@@ -1047,6 +1170,7 @@ Tcl_DStringFree(Tcl_DString *svp)
  SV *sv;
  if ((sv = *svp))
   {
+   dTHX;
    SvREFCNT_dec(sv);
    *svp = Nullsv;
   }
@@ -1055,6 +1179,7 @@ Tcl_DStringFree(Tcl_DString *svp)
 void
 Tcl_DStringResult(Tcl_Interp *interp, Tcl_DString *svp)
 {
+ dTHX;
  SV *sv = DStringSV(svp);
  /* Tcl8.1+ strings are UTF-8 */
  Tcl_SetObjResult(interp,sv_maybe_utf8(sv));
@@ -1065,6 +1190,7 @@ Tcl_DStringResult(Tcl_Interp *interp, Tcl_DString *svp)
 char *
 Tcl_DStringAppend(Tcl_DString *svp, CONST char *s, int len)
 {
+ dTHX;
  SV *sv = DStringSV(svp);
  Tcl_AppendToObj(sv,(char *)s,len);
  return SvPVX(sv);
@@ -1073,14 +1199,16 @@ Tcl_DStringAppend(Tcl_DString *svp, CONST char *s, int len)
 int
 Tcl_DStringLength(Tcl_DString *svp)
 {
+ dTHX;
  return (int) ((*svp) ? SvCUR(DStringSV(svp)) : 0);
 }
 
 void
 Tcl_DStringSetLength(Tcl_DString *svp,int len)
 {
+ dTHX;
  SV *sv = DStringSV(svp);
- char *s = SvGROW(sv,len+1);
+ char *s = SvGROW(sv,(Size_t)(len+1));
  s[len] = '\0';
  SvCUR(sv) = len;
 }
@@ -1191,14 +1319,14 @@ TclObj_get(pTHX_ SV *sv, MAGIC *mg)
   {
    SvIV_set(sv,info->internalRep.longValue);
    SvIOK_on(sv);
-   LangDebug(__FUNCTION__ " %p %s %ld'\n",sv,info->type->name,SvIV(sv));
+   LangDebug("%s %p %s %ld'\n",__FUNCTION__,sv,info->type->name,SvIV(sv));
    return 0;
   }
  else if (info->type == &tclDoubleType)
   {
    SvNV_set(sv,info->internalRep.doubleValue);
    SvNOK_on(sv);
-   LangDebug(__FUNCTION__ " %p %s %g'\n",sv,info->type->name,SvNV(sv));
+   LangDebug("%s %p %s %g'\n",__FUNCTION__,sv,info->type->name,SvNV(sv));
    return 0;
   }
  else if (SvROK(sv) || info->type == &perlDummyType)
@@ -1217,7 +1345,7 @@ TclObj_get(pTHX_ SV *sv, MAGIC *mg)
    Tcl_GetString(sv);
    SvPOK_on(sv);
 #if 0
-   LangDebug(__FUNCTION__ " %p %s '%s'\n",sv,info->type->name,SvPV_nolen(sv));
+   LangDebug("%s %p %s '%s'\n",__FUNCTION__,sv,info->type->name,SvPV_nolen(sv));
 #endif
   }
  return 0;
@@ -1230,7 +1358,7 @@ TclObj_free(pTHX_ SV *sv, MAGIC *mg)
  if (info->type)
   {
 #ifdef DEBUG_TCLOBJ
-   LangDebug(__FUNCTION__ " %p %s\n",sv,info->type->name);
+   LangDebug("%s %p %s\n",__FUNCTION__,sv,info->type->name);
 #endif
    if (info->type->freeIntRepProc != NULL)
     {
@@ -1248,7 +1376,7 @@ TclObj_free(pTHX_ SV *sv, MAGIC *mg)
   {
    /* We can have pretened we are double or int without setting a type */
 #if 0
-   LangDebug(__FUNCTION__ " %p NULL\n",sv);
+   LangDebug("%s %p NULL\n",__FUNCTION__,sv);
    sv_dump(sv);
 #endif
   }
@@ -1260,9 +1388,9 @@ TclObj_set(pTHX_ SV *sv, MAGIC *mg)
 {
 #ifdef DEBUG_TCLOBJ
  TclObjMagic_t *info = (TclObjMagic_t *)SvPVX(mg->mg_obj);
- LangDebug(__FUNCTION__ " %p %s\n",sv,info->type->name);
+ LangDebug("%s %p %s\n",__FUNCTION__,sv,info->type->name);
 #endif
- sv_unmagic(sv,'~');  /* sv_unmagic calls free proc */
+ sv_unmagic(sv,PERL_MAGIC_ext);  /* sv_unmagic calls free proc */
  return 0;
 }
 
@@ -1271,7 +1399,7 @@ TclObj_len(pTHX_ SV *sv, MAGIC *mg)
 {
 #ifdef DEBUG_TCLOBJ
  TclObjMagic_t *info = (TclObjMagic_t *)SvPVX(mg->mg_obj);
- LangDebug(__FUNCTION__ " %s\n",info->type->name);
+ LangDebug("%s %s\n",__FUNCTION__,info->type->name);
 #endif
  return 0;
 }
@@ -1281,9 +1409,9 @@ TclObj_clear(pTHX_ SV *sv, MAGIC *mg)
 {
 #ifdef DEBUG_TCLOBJ
  TclObjMagic_t *info = (TclObjMagic_t *)SvPVX(mg->mg_obj);
- LangDebug(__FUNCTION__ " %p %s\n",sv,info->type->name);
+ LangDebug("%s %p %s\n",__FUNCTION__,sv,info->type->name);
 #endif
- sv_unmagic(sv,'~');  /* sv_unmagic calls free proc */
+ sv_unmagic(sv,PERL_MAGIC_ext);  /* sv_unmagic calls free proc */
  return 0;
 }
 
@@ -1300,7 +1428,7 @@ static TclObjMagic_t *
 Tcl_ObjMagic(Tcl_Obj *obj,int add)
 {
  dTHX;
- MAGIC *mg = (SvTYPE(obj) >= SVt_PVMG) ? mg_find(obj,'~') : NULL;
+ MAGIC *mg = (SvTYPE(obj) >= SVt_PVMG) ? mg_find(obj,PERL_MAGIC_ext) : NULL;
  SV *data = NULL;
  TclObjMagic_t *iv;
  if (mg)
@@ -1328,10 +1456,10 @@ Tcl_ObjMagic(Tcl_Obj *obj,int add)
    if (rdonly)
     SvREADONLY_off(obj);
    sv_upgrade(obj,SVt_PVMG);
-   sv_magic(obj,data,'~',NULL,0);
+   sv_magic(obj,data,PERL_MAGIC_ext,NULL,0);
    SvREFCNT_dec(data);
    SvRMAGICAL_off(obj);
-   mg = mg_find(obj,'~');
+   mg = mg_find(obj,PERL_MAGIC_ext);
    if (mg->mg_obj != data)
     abort();
    mg->mg_virtual = &TclObj_vtab;
@@ -1370,7 +1498,7 @@ Tcl_DuplicateObj(Tcl_Obj *src)
   {
    abort();
   }
- if (!object && SvROK(src) && SvTYPE(SvRV(src)) == SVt_PVAV)
+ else if (!object && SvROK(src) && SvTYPE(SvRV(src)) == SVt_PVAV)
   {
    AV *av  = (AV *) SvRV(src);
    IV max  = av_len(av);
@@ -1413,7 +1541,7 @@ Tcl_GetObjType(CONST char *name)
   return &tclIntType;
  if (strEQ(name,"double"))
   return &tclDoubleType;
- LangDebug(__FUNCTION__ " wanted %s\n",name);
+ LangDebug("%s wanted %s\n",__FUNCTION__,name);
  return &perlDummyType;
 }
 
@@ -1421,7 +1549,7 @@ static void
 NoFreeProc(Tcl_Obj *obj)
 {
  TclObjMagic_t *m = Tcl_ObjMagic(obj,1);
- LangDebug(__FUNCTION__ " %p %s\n",obj,m->type->name);
+ LangDebug("%s %p %s\n",__FUNCTION__,obj,m->type->name);
 }
 
 Tcl_ObjType *
@@ -1463,13 +1591,31 @@ TclObjSetType(Tcl_Obj *obj,Tcl_ObjType *type)
 #ifdef DEBUG_TCLOBJ
  if (m->type)
   {
-   LangDebug(__FUNCTION__ " %p was %s\n",obj,m->type->name);
+   LangDebug("%s %p was %s\n",__FUNCTION__,obj,m->type->name);
   }
- LangDebug(__FUNCTION__ " %p now %s\n",obj,type->name);
+ LangDebug("%s %p now %s\n",__FUNCTION__,obj,type->name);
 #endif
  m->type = type;
 }
 
+int
+Tcl_ConvertToType(Tcl_Interp * interp, Tcl_Obj * objPtr,
+                  Tcl_ObjType * typePtr)
+{
+    if (TclObjGetType(objPtr) == typePtr) {
+	return TCL_OK;
+    }
+
+    /*
+     * Use the target type's Tcl_SetFromAnyProc to set "objPtr"s internal
+     * form as appropriate for the target type. This frees the old internal
+     * representation.
+     */
+
+    return typePtr->setFromAnyProc(interp, objPtr);
+}
+
+
 Tcl_InternalRep *
 TclObjInternal(Tcl_Obj *obj)
 {
@@ -1483,18 +1629,22 @@ Tcl_RegisterObjType(Tcl_ObjType *type)
 }
 
 
-Arg
+Tcl_Obj *
 LangCopyArg(sv)
 SV *sv;
 {
  if (sv)
   {
    dTHX;
-   MAGIC *mg = (SvTYPE(sv) >= SVt_PVMG) ? mg_find(sv,'~') : NULL;
+   MAGIC *mg = (SvTYPE(sv) >= SVt_PVMG) ? mg_find(sv,PERL_MAGIC_ext) : NULL;
    if (mg && mg->mg_virtual == &TclObj_vtab)
     {
      return Tcl_DuplicateObj(sv);
     }
+   if (SvROK(sv) && SvTYPE(SvRV(sv)) == SVt_PVCV)
+    {
+     return LangMakeCallback(sv);
+    }
    sv = newSVsv(sv);
   }
  return sv;
