@@ -1,7 +1,7 @@
 /*
  * tkCanvWind.c --
  *
- *	This file implements window items for canvas widgets.
+ *      This file implements window items for canvas widgets.
  *
  * Copyright (c) 1992-1994 The Regents of the University of California.
  * Copyright (c) 1994-1997 Sun Microsystems, Inc.
@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkCanvWind.c,v 1.7 2002/08/05 04:30:38 dgp Exp $
+ * RCS: @(#) $Id: tkCanvWind.c,v 1.9 2003/02/09 07:48:22 hobbs Exp $
  */
 
 #include "tkPort.h"
@@ -21,19 +21,19 @@
  */
 
 typedef struct WindowItem  {
-    Tk_Item header;		/* Generic stuff that's the same for all
+    Tk_Item header;             /* Generic stuff that's the same for all
 				 * types.  MUST BE FIRST IN STRUCTURE. */
-    double x, y;		/* Coordinates of positioning point for
+    double x, y;                /* Coordinates of positioning point for
 				 * window. */
-    Tk_Window tkwin;		/* Window associated with item.  NULL means
+    Tk_Window tkwin;            /* Window associated with item.  NULL means
 				 * window has been destroyed. */
-    int width;			/* Width to use for window (<= 0 means use
+    int width;                  /* Width to use for window (<= 0 means use
 				 * window's requested width). */
-    int height;			/* Width to use for window (<= 0 means use
+    int height;                 /* Width to use for window (<= 0 means use
 				 * window's requested width). */
-    Tk_Anchor anchor;		/* Where to anchor window relative to
+    Tk_Anchor anchor;           /* Where to anchor window relative to
 				 * (x,y). */
-    Tk_Canvas canvas;		/* Canvas containing this item. */
+    Tk_Canvas canvas;           /* Canvas containing this item. */
 } WindowItem;
 
 /*
@@ -73,44 +73,44 @@ static Tk_ConfigSpec configSpecs[] = {
  * Prototypes for procedures defined in this file:
  */
 
-static void		ComputeWindowBbox _ANSI_ARGS_((Tk_Canvas canvas,
+static void             ComputeWindowBbox _ANSI_ARGS_((Tk_Canvas canvas,
 			    WindowItem *winItemPtr));
-static int		ConfigureWinItem _ANSI_ARGS_((Tcl_Interp *interp,
+static int              ConfigureWinItem _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, int objc,
 			    Tcl_Obj *CONST objv[], int flags));
-static int		CreateWinItem _ANSI_ARGS_((Tcl_Interp *interp,
+static int              CreateWinItem _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, struct Tk_Item *itemPtr,
 			    int objc, Tcl_Obj *CONST objv[]));
-static void		DeleteWinItem _ANSI_ARGS_((Tk_Canvas canvas,
+static void             DeleteWinItem _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, Display *display));
-static void		DisplayWinItem _ANSI_ARGS_((Tk_Canvas canvas,
+static void             DisplayWinItem _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, Display *display, Drawable dst,
 			    int x, int y, int width, int height));
-static void		ScaleWinItem _ANSI_ARGS_((Tk_Canvas canvas,
+static void             ScaleWinItem _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, double originX, double originY,
 			    double scaleX, double scaleY));
-static void		TranslateWinItem _ANSI_ARGS_((Tk_Canvas canvas,
+static void             TranslateWinItem _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, double deltaX, double deltaY));
-static int		WinItemCoords _ANSI_ARGS_((Tcl_Interp *interp,
+static int              WinItemCoords _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, int objc,
 			    Tcl_Obj *CONST objv[]));
-static void		WinItemLostSlaveProc _ANSI_ARGS_((
+static void             WinItemLostSlaveProc _ANSI_ARGS_((
 			    ClientData clientData, Tk_Window tkwin));
-static void		WinItemRequestProc _ANSI_ARGS_((ClientData clientData,
+static void             WinItemRequestProc _ANSI_ARGS_((ClientData clientData,
 			    Tk_Window tkwin));
-static void		WinItemStructureProc _ANSI_ARGS_((
+static void             WinItemStructureProc _ANSI_ARGS_((
 			    ClientData clientData, XEvent *eventPtr));
-static int		WinItemToArea _ANSI_ARGS_((Tk_Canvas canvas,
+static int              WinItemToArea _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, double *rectPtr));
-static int		WinItemToPostscript _ANSI_ARGS_((Tcl_Interp *interp,
+static int              WinItemToPostscript _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, int prepass));
-static double		WinItemToPoint _ANSI_ARGS_((Tk_Canvas canvas,
+static double           WinItemToPoint _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, double *pointPtr));
 #ifdef X_GetImage
-static int		xerrorhandler _ANSI_ARGS_((ClientData clientData,
+static int              xerrorhandler _ANSI_ARGS_((ClientData clientData,
 			    XErrorEvent *e));
 #endif
-static int		CanvasPsWindow _ANSI_ARGS_((Tcl_Interp *interp,
+static int              CanvasPsWindow _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Window tkwin, Tk_Canvas canvas, double x,
 			    double y, int width, int height));
 
@@ -120,26 +120,26 @@ static int		CanvasPsWindow _ANSI_ARGS_((Tcl_Interp *interp,
  */
 
 Tk_ItemType tkWindowType = {
-    "window",				/* name */
-    sizeof(WindowItem),			/* itemSize */
-    CreateWinItem,			/* createProc */
-    configSpecs,			/* configSpecs */
-    ConfigureWinItem,			/* configureProc */
-    WinItemCoords,			/* coordProc */
-    DeleteWinItem,			/* deleteProc */
-    DisplayWinItem,			/* displayProc */
-    1|TK_CONFIG_OBJS,			/* flags */
-    WinItemToPoint,			/* pointProc */
-    WinItemToArea,			/* areaProc */
-    WinItemToPostscript,	        /* postscriptProc */
-    ScaleWinItem,			/* scaleProc */
-    TranslateWinItem,			/* translateProc */
-    (Tk_ItemIndexProc *) NULL,		/* indexProc */
-    (Tk_ItemCursorProc *) NULL,		/* cursorProc */
-    (Tk_ItemSelectionProc *) NULL,	/* selectionProc */
-    (Tk_ItemInsertProc *) NULL,		/* insertProc */
-    (Tk_ItemDCharsProc *) NULL,		/* dTextProc */
-    (Tk_ItemType *) NULL,		/* nextPtr */
+    "window",                           /* name */
+    sizeof(WindowItem),                 /* itemSize */
+    CreateWinItem,                      /* createProc */
+    configSpecs,                        /* configSpecs */
+    ConfigureWinItem,                   /* configureProc */
+    WinItemCoords,                      /* coordProc */
+    DeleteWinItem,                      /* deleteProc */
+    DisplayWinItem,                     /* displayProc */
+    1|TK_CONFIG_OBJS,                   /* flags */
+    WinItemToPoint,                     /* pointProc */
+    WinItemToArea,                      /* areaProc */
+    WinItemToPostscript,                /* postscriptProc */
+    ScaleWinItem,                       /* scaleProc */
+    TranslateWinItem,                   /* translateProc */
+    (Tk_ItemIndexProc *) NULL,          /* indexProc */
+    (Tk_ItemCursorProc *) NULL,         /* cursorProc */
+    (Tk_ItemSelectionProc *) NULL,      /* selectionProc */
+    (Tk_ItemInsertProc *) NULL,         /* insertProc */
+    (Tk_ItemDCharsProc *) NULL,         /* dTextProc */
+    (Tk_ItemType *) NULL,               /* nextPtr */
 };
 
 
@@ -149,9 +149,9 @@ Tk_ItemType tkWindowType = {
  */
 
 static Tk_GeomMgr canvasGeomType = {
-    "canvas",				/* name */
-    WinItemRequestProc,			/* requestProc */
-    WinItemLostSlaveProc,		/* lostSlaveProc */
+    "canvas",                           /* name */
+    WinItemRequestProc,                 /* requestProc */
+    WinItemLostSlaveProc,               /* lostSlaveProc */
 };
 
 /*
@@ -159,49 +159,36 @@ static Tk_GeomMgr canvasGeomType = {
  *
  * CreateWinItem --
  *
- *	This procedure is invoked to create a new window
- *	item in a canvas.
+ *      This procedure is invoked to create a new window
+ *      item in a canvas.
  *
  * Results:
- *	A standard Tcl return value.  If an error occurred in
- *	creating the item, then an error message is left in
- *	the interp's result;  in this case itemPtr is
- *	left uninitialized, so it can be safely freed by the
- *	caller.
+ *      A standard Tcl return value.  If an error occurred in
+ *      creating the item, then an error message is left in
+ *      the interp's result;  in this case itemPtr is
+ *      left uninitialized, so it can be safely freed by the
+ *      caller.
  *
  * Side effects:
- *	A new window item is created.
+ *      A new window item is created.
  *
  *--------------------------------------------------------------
  */
 
 static int
 CreateWinItem(interp, canvas, itemPtr, objc, objv)
-    Tcl_Interp *interp;			/* Interpreter for error reporting. */
-    Tk_Canvas canvas;			/* Canvas to hold new item. */
-    Tk_Item *itemPtr;			/* Record to hold new item;  header
+    Tcl_Interp *interp;                 /* Interpreter for error reporting. */
+    Tk_Canvas canvas;                   /* Canvas to hold new item. */
+    Tk_Item *itemPtr;                   /* Record to hold new item;  header
 					 * has been initialized by caller. */
-    int objc;				/* Number of arguments in objv. */
-    Tcl_Obj *CONST objv[];		/* Arguments describing window. */
+    int objc;                           /* Number of arguments in objv. */
+    Tcl_Obj *CONST objv[];              /* Arguments describing window. */
 {
     WindowItem *winItemPtr = (WindowItem *) itemPtr;
-    int i = 2;
+    int i;
 
-    if (objc == 1) {
-	i = 1;
-    } else if (objc > 1) {
-	char *arg = Tcl_GetString(objv[1]);
-	if ((arg[0] == '-') && (arg[1] >= 'a') && (arg[1] <= 'z')) {
-	    i = 1;
-	}
-    }
-
-    if (objc < i) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		Tk_PathName(Tk_CanvasTkwin(canvas)), " create ",
-		itemPtr->typePtr->name, " x y ?options?\"",
-		(char *) NULL);
-	return TCL_ERROR;
+    if (objc == 0) {
+	panic("canvas did not pass any coords\n");
     }
 
     /*
@@ -216,12 +203,23 @@ CreateWinItem(interp, canvas, itemPtr, objc, objv)
 
     /*
      * Process the arguments to fill in the item record.
+     * Only 1 (list) or 2 (x y) coords are allowed.
      */
 
-    if ((WinItemCoords(interp, canvas, itemPtr, i, objv) != TCL_OK)) {
+    if (objc == 1) {
+	i = 1;
+    } else {
+	char *arg = Tcl_GetString(objv[1]);
+	i = 2;
+	if ((arg[0] == '-') && (arg[1] >= 'a') && (arg[1] <= 'z')) {
+	    i = 1;
+	}
+    }
+    if (WinItemCoords(interp, canvas, itemPtr, i, objv) != TCL_OK) {
 	goto error;
     }
-    if (ConfigureWinItem(interp, canvas, itemPtr, objc-i, objv+i, 0) == TCL_OK) {
+    if (ConfigureWinItem(interp, canvas, itemPtr, objc-i, objv+i, 0)
+	    == TCL_OK) {
 	return TCL_OK;
     }
 
@@ -235,28 +233,28 @@ CreateWinItem(interp, canvas, itemPtr, objc, objv)
  *
  * WinItemCoords --
  *
- *	This procedure is invoked to process the "coords" widget
- *	command on window items.  See the user documentation for
- *	details on what it does.
+ *      This procedure is invoked to process the "coords" widget
+ *      command on window items.  See the user documentation for
+ *      details on what it does.
  *
  * Results:
- *	Returns TCL_OK or TCL_ERROR, and sets the interp's result.
+ *      Returns TCL_OK or TCL_ERROR, and sets the interp's result.
  *
  * Side effects:
- *	The coordinates for the given item may be changed.
+ *      The coordinates for the given item may be changed.
  *
  *--------------------------------------------------------------
  */
 
 static int
 WinItemCoords(interp, canvas, itemPtr, objc, objv)
-    Tcl_Interp *interp;			/* Used for error reporting. */
-    Tk_Canvas canvas;			/* Canvas containing item. */
-    Tk_Item *itemPtr;			/* Item whose coordinates are to be
+    Tcl_Interp *interp;                 /* Used for error reporting. */
+    Tk_Canvas canvas;                   /* Canvas containing item. */
+    Tk_Item *itemPtr;                   /* Item whose coordinates are to be
 					 * read or modified. */
-    int objc;				/* Number of coordinates supplied in
+    int objc;                           /* Number of coordinates supplied in
 					 * objv. */
-    Tcl_Obj *CONST objv[];		/* Array of coordinates: x1, y1,
+    Tcl_Obj *CONST objv[];              /* Array of coordinates: x1, y1,
 					 * x2, y2, ... */
 {
     WindowItem *winItemPtr = (WindowItem *) itemPtr;
@@ -302,27 +300,27 @@ WinItemCoords(interp, canvas, itemPtr, objc, objv)
  *
  * ConfigureWinItem --
  *
- *	This procedure is invoked to configure various aspects
- *	of a window item, such as its anchor position.
+ *      This procedure is invoked to configure various aspects
+ *      of a window item, such as its anchor position.
  *
  * Results:
- *	A standard Tcl result code.  If an error occurs, then
- *	an error message is left in the interp's result.
+ *      A standard Tcl result code.  If an error occurs, then
+ *      an error message is left in the interp's result.
  *
  * Side effects:
- *	Configuration information may be set for itemPtr.
+ *      Configuration information may be set for itemPtr.
  *
  *--------------------------------------------------------------
  */
 
 static int
 ConfigureWinItem(interp, canvas, itemPtr, objc, objv, flags)
-    Tcl_Interp *interp;		/* Used for error reporting. */
-    Tk_Canvas canvas;		/* Canvas containing itemPtr. */
-    Tk_Item *itemPtr;		/* Window item to reconfigure. */
-    int objc;			/* Number of elements in objv.  */
-    Tcl_Obj *CONST objv[];	/* Arguments describing things to configure. */
-    int flags;			/* Flags to pass to Tk_ConfigureWidget. */
+    Tcl_Interp *interp;         /* Used for error reporting. */
+    Tk_Canvas canvas;           /* Canvas containing itemPtr. */
+    Tk_Item *itemPtr;           /* Window item to reconfigure. */
+    int objc;                   /* Number of elements in objv.  */
+    Tcl_Obj *CONST objv[];      /* Arguments describing things to configure. */
+    int flags;                  /* Flags to pass to Tk_ConfigureWidget. */
 {
     WindowItem *winItemPtr = (WindowItem *) itemPtr;
     Tk_Window oldWindow;
@@ -396,23 +394,23 @@ ConfigureWinItem(interp, canvas, itemPtr, objc, objv, flags)
  *
  * DeleteWinItem --
  *
- *	This procedure is called to clean up the data structure
- *	associated with a window item.
+ *      This procedure is called to clean up the data structure
+ *      associated with a window item.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Resources associated with itemPtr are released.
+ *      Resources associated with itemPtr are released.
  *
  *--------------------------------------------------------------
  */
 
 static void
 DeleteWinItem(canvas, itemPtr, display)
-    Tk_Canvas canvas;			/* Overall info about widget. */
-    Tk_Item *itemPtr;			/* Item that is being deleted. */
-    Display *display;			/* Display containing window for
+    Tk_Canvas canvas;                   /* Overall info about widget. */
+    Tk_Item *itemPtr;                   /* Item that is being deleted. */
+    Display *display;                   /* Display containing window for
 					 * canvas. */
 {
     WindowItem *winItemPtr = (WindowItem *) itemPtr;
@@ -435,25 +433,25 @@ DeleteWinItem(canvas, itemPtr, display)
  *
  * ComputeWindowBbox --
  *
- *	This procedure is invoked to compute the bounding box of
- *	all the pixels that may be drawn as part of a window item.
- *	This procedure is where the child window's placement is
- *	computed.
+ *      This procedure is invoked to compute the bounding box of
+ *      all the pixels that may be drawn as part of a window item.
+ *      This procedure is where the child window's placement is
+ *      computed.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The fields x1, y1, x2, and y2 are updated in the header
- *	for itemPtr.
+ *      The fields x1, y1, x2, and y2 are updated in the header
+ *      for itemPtr.
  *
  *--------------------------------------------------------------
  */
 
 static void
 ComputeWindowBbox(canvas, winItemPtr)
-    Tk_Canvas canvas;			/* Canvas that contains item. */
-    WindowItem *winItemPtr;		/* Item whose bbox is to be
+    Tk_Canvas canvas;                   /* Canvas that contains item. */
+    WindowItem *winItemPtr;             /* Item whose bbox is to be
 					 * recomputed. */
 {
     int width, height, x, y;
@@ -548,21 +546,21 @@ ComputeWindowBbox(canvas, winItemPtr)
  *
  * DisplayWinItem --
  *
- *	This procedure is invoked to "draw" a window item in a given
- *	drawable.  Since the window draws itself, we needn't do any
- *	actual redisplay here.  However, this procedure takes care
- *	of actually repositioning the child window so that it occupies
- *	the correct screen position.
+ *      This procedure is invoked to "draw" a window item in a given
+ *      drawable.  Since the window draws itself, we needn't do any
+ *      actual redisplay here.  However, this procedure takes care
+ *      of actually repositioning the child window so that it occupies
+ *      the correct screen position.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The child window's position may get changed.  Note: this
- *	procedure gets called both when a window needs to be displayed
- *	and when it ceases to be visible on the screen (e.g. it was
- *	scrolled or moved off-screen or the enclosing canvas is
- *	unmapped).
+ *      The child window's position may get changed.  Note: this
+ *      procedure gets called both when a window needs to be displayed
+ *      and when it ceases to be visible on the screen (e.g. it was
+ *      scrolled or moved off-screen or the enclosing canvas is
+ *      unmapped).
  *
  *--------------------------------------------------------------
  */
@@ -570,10 +568,10 @@ ComputeWindowBbox(canvas, winItemPtr)
 static void
 DisplayWinItem(canvas, itemPtr, display, drawable, regionX, regionY,
 	regionWidth, regionHeight)
-    Tk_Canvas canvas;			/* Canvas that contains item. */
-    Tk_Item *itemPtr;			/* Item to be displayed. */
-    Display *display;			/* Display on which to draw item. */
-    Drawable drawable;			/* Pixmap or window in which to draw
+    Tk_Canvas canvas;                   /* Canvas that contains item. */
+    Tk_Item *itemPtr;                   /* Item to be displayed. */
+    Display *display;                   /* Display on which to draw item. */
+    Drawable drawable;                  /* Pixmap or window in which to draw
 					 * item. */
     int regionX, regionY, regionWidth, regionHeight;
 					/* Describes region of canvas that
@@ -637,26 +635,26 @@ DisplayWinItem(canvas, itemPtr, display, drawable, regionX, regionY,
  *
  * WinItemToPoint --
  *
- *	Computes the distance from a given point to a given
- *	window, in canvas units.
+ *      Computes the distance from a given point to a given
+ *      window, in canvas units.
  *
  * Results:
- *	The return value is 0 if the point whose x and y coordinates
- *	are coordPtr[0] and coordPtr[1] is inside the window.  If the
- *	point isn't inside the window then the return value is the
- *	distance from the point to the window.
+ *      The return value is 0 if the point whose x and y coordinates
+ *      are coordPtr[0] and coordPtr[1] is inside the window.  If the
+ *      point isn't inside the window then the return value is the
+ *      distance from the point to the window.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *--------------------------------------------------------------
  */
 
 static double
 WinItemToPoint(canvas, itemPtr, pointPtr)
-    Tk_Canvas canvas;		/* Canvas containing item. */
-    Tk_Item *itemPtr;		/* Item to check against point. */
-    double *pointPtr;		/* Pointer to x and y coordinates. */
+    Tk_Canvas canvas;           /* Canvas containing item. */
+    Tk_Item *itemPtr;           /* Item to check against point. */
+    double *pointPtr;           /* Pointer to x and y coordinates. */
 {
     WindowItem *winItemPtr = (WindowItem *) itemPtr;
     double x1, x2, y1, y2, xDiff, yDiff;
@@ -694,26 +692,26 @@ WinItemToPoint(canvas, itemPtr, pointPtr)
  *
  * WinItemToArea --
  *
- *	This procedure is called to determine whether an item
- *	lies entirely inside, entirely outside, or overlapping
- *	a given rectangle.
+ *      This procedure is called to determine whether an item
+ *      lies entirely inside, entirely outside, or overlapping
+ *      a given rectangle.
  *
  * Results:
- *	-1 is returned if the item is entirely outside the area
- *	given by rectPtr, 0 if it overlaps, and 1 if it is entirely
- *	inside the given area.
+ *      -1 is returned if the item is entirely outside the area
+ *      given by rectPtr, 0 if it overlaps, and 1 if it is entirely
+ *      inside the given area.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *--------------------------------------------------------------
  */
 
 static int
 WinItemToArea(canvas, itemPtr, rectPtr)
-    Tk_Canvas canvas;		/* Canvas containing item. */
-    Tk_Item *itemPtr;		/* Item to check against rectangle. */
-    double *rectPtr;		/* Pointer to array of four coordinates
+    Tk_Canvas canvas;           /* Canvas containing item. */
+    Tk_Item *itemPtr;           /* Item to check against rectangle. */
+    double *rectPtr;            /* Pointer to array of four coordinates
 				 * (x1, y1, x2, y2) describing rectangular
 				 * area.  */
 {
@@ -739,14 +737,14 @@ WinItemToArea(canvas, itemPtr, rectPtr)
  *
  * xerrorhandler --
  *
- *	This is a dummy function to catch X11 errors during an
- *	attempt to print a canvas window.
+ *      This is a dummy function to catch X11 errors during an
+ *      attempt to print a canvas window.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *--------------------------------------------------------------
  */
@@ -767,30 +765,30 @@ xerrorhandler(clientData, e)
  *
  * WinItemToPostscript --
  *
- *	This procedure is called to generate Postscript for
- *	window items.
+ *      This procedure is called to generate Postscript for
+ *      window items.
  *
  * Results:
- *	The return value is a standard Tcl result.  If an error
- *	occurs in generating Postscript then an error message is
- *	left in interp->result, replacing whatever used to be there.
- *	If no error occurs, then Postscript for the item is appended
- *	to the result.
+ *      The return value is a standard Tcl result.  If an error
+ *      occurs in generating Postscript then an error message is
+ *      left in interp->result, replacing whatever used to be there.
+ *      If no error occurs, then Postscript for the item is appended
+ *      to the result.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *--------------------------------------------------------------
  */
 
 static int
 WinItemToPostscript(interp, canvas, itemPtr, prepass)
-    Tcl_Interp *interp;			/* Leave Postscript or error message
+    Tcl_Interp *interp;                 /* Leave Postscript or error message
 					 * here. */
-    Tk_Canvas canvas;			/* Information about overall canvas. */
-    Tk_Item *itemPtr;			/* Item for which Postscript is
+    Tk_Canvas canvas;                   /* Information about overall canvas. */
+    Tk_Item *itemPtr;                   /* Item for which Postscript is
 					 * wanted. */
-    int prepass;			/* 1 means this is a prepass to
+    int prepass;                        /* 1 means this is a prepass to
 					 * collect font information;  0 means
 					 * final Postscript is being created.*/
 {
@@ -801,7 +799,7 @@ WinItemToPostscript(interp, canvas, itemPtr, prepass)
     Tk_Window tkwin = winItemPtr->tkwin;
 
     if (prepass || winItemPtr->tkwin == NULL) {
-        return TCL_OK;
+	return TCL_OK;
     }
 
     width = Tk_Width(tkwin);
@@ -816,15 +814,15 @@ WinItemToPostscript(interp, canvas, itemPtr, prepass)
     y = Tk_CanvasPsY(canvas, winItemPtr->y);
 
     switch (winItemPtr->anchor) {
-	case TK_ANCHOR_NW:			y -= height;		break;
-	case TK_ANCHOR_N:	x -= width/2.0; y -= height;		break;
-	case TK_ANCHOR_NE:	x -= width;	y -= height;		break;
-	case TK_ANCHOR_E:	x -= width;	y -= height/2.0;	break;
-	case TK_ANCHOR_SE:	x -= width;				break;
-	case TK_ANCHOR_S:	x -= width/2.0;				break;
-	case TK_ANCHOR_SW:						break;
-	case TK_ANCHOR_W:			y -= height/2.0;	break;
-	case TK_ANCHOR_CENTER:	x -= width/2.0; y -= height/2.0;	break;
+	case TK_ANCHOR_NW:                      y -= height;            break;
+	case TK_ANCHOR_N:       x -= width/2.0; y -= height;            break;
+	case TK_ANCHOR_NE:      x -= width;     y -= height;            break;
+	case TK_ANCHOR_E:       x -= width;     y -= height/2.0;        break;
+	case TK_ANCHOR_SE:      x -= width;                             break;
+	case TK_ANCHOR_S:       x -= width/2.0;                         break;
+	case TK_ANCHOR_SW:                                              break;
+	case TK_ANCHOR_W:                       y -= height/2.0;        break;
+	case TK_ANCHOR_CENTER:  x -= width/2.0; y -= height/2.0;        break;
     }
 
     return CanvasPsWindow(interp, tkwin, canvas, x, y, width, height);
@@ -832,20 +830,19 @@ WinItemToPostscript(interp, canvas, itemPtr, prepass)
 
 static int
 CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
-    Tcl_Interp *interp;			/* Leave Postscript or error message
+    Tcl_Interp *interp;                 /* Leave Postscript or error message
 					 * here. */
-    Tk_Window tkwin;			/* window to be printed */
-    Tk_Canvas canvas;			/* Information about overall canvas. */
-    double x, y;			/* origin of window. */
-    int width, height;			/* width/height of window. */
+    Tk_Window tkwin;                    /* window to be printed */
+    Tk_Canvas canvas;                   /* Information about overall canvas. */
+    double x, y;                        /* origin of window. */
+    int width, height;                  /* width/height of window. */
 {
     char buffer[256];
-    TkWindow *winPtr;
     XImage *ximage;
     int result;
     Tcl_Obj *savePtr, *buffer1;
 #ifdef X_GetImage
-    Tk_ErrorHandler	handle;
+    Tk_ErrorHandler     handle;
 #endif
     Tcl_Obj *obj = NULL;
 
@@ -887,14 +884,8 @@ CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
 		"1.000 1.000 1.000 setrgbcolor AdjustColor\nfill\ngrestore\n",
 		Tcl_GetString(buffer1), "\nrestore\nend\n\n\n",
 		(char *) NULL);
-#if 0
-	for (winPtr = ((TkWindow *) tkwin)->childList; winPtr != NULL;
-		winPtr = winPtr->nextPtr) {
-	    if (Tk_IsMapped(winPtr)) {
-/*		printf("child window: %s\n", winPtr->pathName);*/
-	    }
-	}
-#endif
+	Tcl_DStringFree(&buffer1);
+
 	return result;
     }
     Tcl_DecrRefCount(buffer1);
@@ -936,28 +927,28 @@ CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
  *
  * ScaleWinItem --
  *
- *	This procedure is invoked to rescale a window item.
+ *      This procedure is invoked to rescale a window item.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The window referred to by itemPtr is rescaled
- *	so that the following transformation is applied to all
- *	point coordinates:
- *		x' = originX + scaleX*(x-originX)
- *		y' = originY + scaleY*(y-originY)
+ *      The window referred to by itemPtr is rescaled
+ *      so that the following transformation is applied to all
+ *      point coordinates:
+ *              x' = originX + scaleX*(x-originX)
+ *              y' = originY + scaleY*(y-originY)
  *
  *--------------------------------------------------------------
  */
 
 static void
 ScaleWinItem(canvas, itemPtr, originX, originY, scaleX, scaleY)
-    Tk_Canvas canvas;			/* Canvas containing window. */
-    Tk_Item *itemPtr;			/* Window to be scaled. */
-    double originX, originY;		/* Origin about which to scale window. */
-    double scaleX;			/* Amount to scale in X direction. */
-    double scaleY;			/* Amount to scale in Y direction. */
+    Tk_Canvas canvas;                   /* Canvas containing window. */
+    Tk_Item *itemPtr;                   /* Window to be scaled. */
+    double originX, originY;            /* Origin about which to scale window. */
+    double scaleX;                      /* Amount to scale in X direction. */
+    double scaleY;                      /* Amount to scale in Y direction. */
 {
     WindowItem *winItemPtr = (WindowItem *) itemPtr;
 
@@ -977,24 +968,24 @@ ScaleWinItem(canvas, itemPtr, originX, originY, scaleX, scaleY)
  *
  * TranslateWinItem --
  *
- *	This procedure is called to move a window by a given amount.
+ *      This procedure is called to move a window by a given amount.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The position of the window is offset by (xDelta, yDelta),
- *	and the bounding box is updated in the generic part of the
- *	item structure.
+ *      The position of the window is offset by (xDelta, yDelta),
+ *      and the bounding box is updated in the generic part of the
+ *      item structure.
  *
  *--------------------------------------------------------------
  */
 
 static void
 TranslateWinItem(canvas, itemPtr, deltaX, deltaY)
-    Tk_Canvas canvas;			/* Canvas containing item. */
-    Tk_Item *itemPtr;			/* Item that is being moved. */
-    double deltaX, deltaY;		/* Amount by which item is to be
+    Tk_Canvas canvas;                   /* Canvas containing item. */
+    Tk_Item *itemPtr;                   /* Item that is being moved. */
+    double deltaX, deltaY;              /* Amount by which item is to be
 					 * moved. */
 {
     WindowItem *winItemPtr = (WindowItem *) itemPtr;
@@ -1009,25 +1000,25 @@ TranslateWinItem(canvas, itemPtr, deltaX, deltaY)
  *
  * WinItemStructureProc --
  *
- *	This procedure is invoked whenever StructureNotify events
- *	occur for a window that's managed as part of a canvas window
- *	item.  This procudure's only purpose is to clean up when
- *	windows are deleted.
+ *      This procedure is invoked whenever StructureNotify events
+ *      occur for a window that's managed as part of a canvas window
+ *      item.  This procudure's only purpose is to clean up when
+ *      windows are deleted.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The window is disassociated from the window item when it is
- *	deleted.
+ *      The window is disassociated from the window item when it is
+ *      deleted.
  *
  *--------------------------------------------------------------
  */
 
 static void
 WinItemStructureProc(clientData, eventPtr)
-    ClientData clientData;	/* Pointer to record describing window item. */
-    XEvent *eventPtr;		/* Describes what just happened. */
+    ClientData clientData;      /* Pointer to record describing window item. */
+    XEvent *eventPtr;           /* Describes what just happened. */
 {
     WindowItem *winItemPtr = (WindowItem *) clientData;
 
@@ -1041,23 +1032,23 @@ WinItemStructureProc(clientData, eventPtr)
  *
  * WinItemRequestProc --
  *
- *	This procedure is invoked whenever a window that's associated
- *	with a window canvas item changes its requested dimensions.
+ *      This procedure is invoked whenever a window that's associated
+ *      with a window canvas item changes its requested dimensions.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The size and location on the screen of the window may change,
- *	depending on the options specified for the window item.
+ *      The size and location on the screen of the window may change,
+ *      depending on the options specified for the window item.
  *
  *--------------------------------------------------------------
  */
 
 static void
 WinItemRequestProc(clientData, tkwin)
-    ClientData clientData;		/* Pointer to record for window item. */
-    Tk_Window tkwin;			/* Window that changed its desired
+    ClientData clientData;              /* Pointer to record for window item. */
+    Tk_Window tkwin;                    /* Window that changed its desired
 					 * size. */
 {
     WindowItem *winItemPtr = (WindowItem *) clientData;
@@ -1072,14 +1063,14 @@ WinItemRequestProc(clientData, tkwin)
  *
  * WinItemLostSlaveProc --
  *
- *	This procedure is invoked by Tk whenever some other geometry
- *	claims control over a slave that used to be managed by us.
+ *      This procedure is invoked by Tk whenever some other geometry
+ *      claims control over a slave that used to be managed by us.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Forgets all canvas-related information about the slave.
+ *      Forgets all canvas-related information about the slave.
  *
  *--------------------------------------------------------------
  */
@@ -1087,9 +1078,9 @@ WinItemRequestProc(clientData, tkwin)
 	/* ARGSUSED */
 static void
 WinItemLostSlaveProc(clientData, tkwin)
-    ClientData clientData;	/* WindowItem structure for slave window that
+    ClientData clientData;      /* WindowItem structure for slave window that
 				 * was stolen away. */
-    Tk_Window tkwin;		/* Tk's handle for the slave window. */
+    Tk_Window tkwin;            /* Tk's handle for the slave window. */
 {
     WindowItem *winItemPtr = (WindowItem *) clientData;
     Tk_Window canvasTkwin = Tk_CanvasTkwin(winItemPtr->canvas);
