@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkEvent.c,v 1.16 2002/06/19 19:37:54 mdejong Exp $
+ * RCS: @(#) $Id: tkEvent.c,v 1.17.2.1 2003/07/19 01:03:25 hobbs Exp $
  */
 
 #include "tkPort.h"
@@ -648,6 +648,7 @@ Tk_HandleEvent(eventPtr)
 
     if (eventPtr->type==ButtonPress) {
 	dispPtr = TkGetDisplay(eventPtr->xbutton.display);
+	dispPtr->mouseButtonWindow = eventPtr->xbutton.window;
 	eventPtr->xbutton.state |= dispPtr->mouseButtonState;
 	switch (eventPtr->xbutton.button) {
 	    case 1: dispPtr->mouseButtonState |= Button1Mask; break;
@@ -656,6 +657,7 @@ Tk_HandleEvent(eventPtr)
 	}
     } else if (eventPtr->type==ButtonRelease) {
 	dispPtr = TkGetDisplay(eventPtr->xbutton.display);
+	dispPtr->mouseButtonWindow = 0;
 	switch (eventPtr->xbutton.button) {
 	    case 1: dispPtr->mouseButtonState &= ~Button1Mask; break;
 	    case 2: dispPtr->mouseButtonState &= ~Button2Mask; break;
@@ -664,7 +666,20 @@ Tk_HandleEvent(eventPtr)
 	eventPtr->xbutton.state |= dispPtr->mouseButtonState;
     } else if (eventPtr->type==MotionNotify) {
 	dispPtr = TkGetDisplay(eventPtr->xmotion.display);
-	eventPtr->xmotion.state |= dispPtr->mouseButtonState;
+	if (dispPtr->mouseButtonState & (Button1Mask|Button2Mask|Button3Mask)) {
+	    if (eventPtr->xbutton.window != dispPtr->mouseButtonWindow) {
+	        /*
+	         * This motion event should not be interpreted as a button
+	         * press + motion event since this is not the same window
+	         * the button was pressed down in.
+	         */
+	        dispPtr->mouseButtonState &=
+	                ~(Button1Mask|Button2Mask|Button3Mask);
+	        dispPtr->mouseButtonWindow = 0;
+	    } else {
+	        eventPtr->xmotion.state |= dispPtr->mouseButtonState;
+	    }
+	}
     }
 
     /*
@@ -910,8 +925,7 @@ Tk_HandleEvent(eventPtr)
 			    XNPreeditAttributes, preedit_attr,
 			    NULL);
 		    XFree(preedit_attr);
-		} else
-#endif
+		} else {
 		    if (winPtr->inputContext != NULL)
 		        panic("inputContext not NULL");
 		    winPtr->inputContext = XCreateIC(dispPtr->inputMethod,
@@ -919,6 +933,16 @@ Tk_HandleEvent(eventPtr)
 			    XNClientWindow, winPtr->window,
 			    XNFocusWindow, winPtr->window,
 			    NULL);
+		}
+#else
+		if (winPtr->inputContext != NULL)
+		    panic("inputContext not NULL");
+		winPtr->inputContext = XCreateIC(dispPtr->inputMethod,
+			XNInputStyle, XIMPreeditNothing|XIMStatusNothing,
+			XNClientWindow, winPtr->window,
+			XNFocusWindow, winPtr->window,
+			NULL);
+#endif
 	    }
 	}
 	if (XFilterEvent(eventPtr, None)) {
@@ -940,11 +964,16 @@ Tk_HandleEvent(eventPtr)
      * There's a potential interaction here with Tk_DeleteEventHandler.
      * Read the documentation for pendingPtr.
      */
-
     ip.eventPtr = eventPtr;
     ip.winPtr = winPtr;
     ip.nextHandler = NULL;
     ip.nextPtr = tsdPtr->pendingPtr;
+
+    /* WARNING: this stores stack address - longjmp() and
+       similar "exception" mechanisms need to take steps
+       to avoid bypassing the cleanup in this code
+     */
+
     tsdPtr->pendingPtr = &ip;
     if (mask == 0) {
 	if ((eventPtr->type == SelectionClear)
@@ -1035,7 +1064,6 @@ Tk_HandleEvent(eventPtr)
     }
     tsdPtr->pendingPtr = ip.nextPtr;
 done:
-
     /*
      * Release the interpreter for this window so that it can be potentially
      * deleted if requested.
@@ -1119,18 +1147,18 @@ TkEventDeadWindow(winPtr)
  */
 
 Time
-TkCurrentTime(dispPtr)
+TkCurrentTime(dispPtr, fallbackCurrent)
     TkDisplay *dispPtr;		/* Display for which the time is desired. */
 {
     register XEvent *eventPtr;
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
             Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+    InProgress *pending;
 
-    if (tsdPtr->pendingPtr == NULL) {
-	return dispPtr->lastEventTime;
-    }
-    eventPtr = tsdPtr->pendingPtr->eventPtr;
-    switch (eventPtr->type) {
+    pending = tsdPtr->pendingPtr;
+    while (pending != NULL) {
+	eventPtr = pending->eventPtr;
+	switch (eventPtr->type) {
 	case ButtonPress:
 	case ButtonRelease:
 	    return eventPtr->xbutton.time;
@@ -1144,8 +1172,10 @@ TkCurrentTime(dispPtr)
 	    return eventPtr->xcrossing.time;
 	case PropertyNotify:
 	    return eventPtr->xproperty.time;
+	}
+	pending = pending->nextPtr;
     }
-    return dispPtr->lastEventTime;
+    return (fallbackCurrent) ? CurrentTime : dispPtr->lastEventTime;
 }
 
 /*
