@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: $Id: tkInt.h,v 1.54 2002/08/31 06:12:20 das Exp $
+ * RCS: $Id: tkInt.h,v 1.56.2.1 2003/10/13 03:30:05 hobbs Exp $
  */
 
 #ifndef _TKINT
@@ -491,6 +491,8 @@ typedef struct TkDisplay {
      */
     int mouseButtonState;	/* current mouse button state for this
 				 * display */
+    Window mouseButtonWindow;	/* Window the button state was set in,
+				 * added in Tk 8.4. */
     Window warpWindow;
     int warpX;
     int warpY;
@@ -498,7 +500,6 @@ typedef struct TkDisplay {
     /*
      * The following field(s) were all added for Tk8.4
      */
-    long deletionEpoch;		/* Incremented by window deletions */
     unsigned int flags;		/* Various flag values:  these are all
 				 * defined in below. */
     TkCaret caret;		/* information about the caret for this
@@ -599,6 +600,7 @@ typedef struct TkMainInfo {
     Tcl_HashTable nameTable;	/* Hash table mapping path names to TkWindow
 				 * structs for all windows related to this
 				 * main window.  Managed by tkWindow.c. */
+    long deletionEpoch;		/* Incremented by window deletions */
     Tk_BindingTable bindingTable;
 				/* Used in conjunction with "bind" command
 				 * to bind events to Tcl commands. */
@@ -1111,9 +1113,6 @@ EXTERN Tcl_Obj *	TkCanvasDashPrintProc _ANSI_ARGS_((
 EXTERN int		TkGetDoublePixels _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Window tkwin, CONST char *string,
 			    double *doublePtr));
-EXTERN CONST Tk_OptionSpec *
-			TkGetOptionSpec _ANSI_ARGS_((CONST char *name,
-			    Tk_OptionTable optionTable));
 EXTERN int		TkOffsetParseProc _ANSI_ARGS_((
 			    ClientData clientData, Tcl_Interp *interp,
 			    Tk_Window tkwin, Tcl_Obj *value, char *widgRec,
