@@ -14,7 +14,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkText.c,v 1.31 2002/08/31 06:12:27 das Exp $
+ * RCS: @(#) $Id: tkText.c,v 1.33 2003/02/18 21:53:59 hobbs Exp $
  */
 #define NO_COREXT
 
@@ -185,7 +185,7 @@ int tkTextDebug = 0;
  */
 
 static int		WrapModeParseProc _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, Tk_Window tkwin, 
+			    Tcl_Interp *interp, Tk_Window tkwin,
 			    Tcl_Obj *value, char *widgRec, int offset));
 static Tcl_Obj *	WrapModePrintProc _ANSI_ARGS_((ClientData clientData,
 			    Tk_Window tkwin, char *widgRec, int offset,
@@ -280,7 +280,7 @@ WrapModeParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
  *--------------------------------------------------------------
  */
 
-static Arg
+static Tcl_Obj *
 WrapModePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;		/* Ignored. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
@@ -337,10 +337,10 @@ static int		TextDumpCmd _ANSI_ARGS_((TkText *textPtr,
 			    Tcl_Interp *interp, int argc, char **argv));
 static void		DumpLine _ANSI_ARGS_((Tcl_Interp *interp,
 			    TkText *textPtr, int what, TkTextLine *linePtr,
-			    int start, int end, int lineno, 
+			    int start, int end, int lineno,
 			    LangCallback *command));
 static int		DumpSegment _ANSI_ARGS_((Tcl_Interp *interp, char *key,
-			    char *value, Tcl_Obj *arg, LangCallback *command, 
+			    char *value, Tcl_Obj *arg, LangCallback *command,
 			    TkTextIndex *index, int what));
 static int		TextEditUndo _ANSI_ARGS_((TkText *textPtr));
 static int		TextEditRedo _ANSI_ARGS_((TkText *textPtr));
@@ -638,7 +638,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 		 * handle partial and fully overlapping ranges.  We have to
 		 * do this with multiple passes.
 		 */
-		TkTextIndex *indices, *ixStart, *ixEnd, *lastStart, *lastEnd;
+		TkTextIndex *indices, *ixStart, *ixEnd, *lastStart;
 		char *useIdx;
 
 		argc -= 2;
@@ -673,7 +673,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 		 */
 		qsort((VOID *) indices, (unsigned) (argc / 2),
 			2 * sizeof(TkTextIndex), TextIndexSortProc);
-		lastStart = lastEnd = NULL;
+		lastStart = NULL;
 		/*
 		 * Second pass will handle bogus ranges (end < start) and
 		 * overlapping ranges.
@@ -704,7 +704,6 @@ TextWidgetCmd(clientData, interp, argc, argv)
 			}
 		    }
 		    lastStart = ixStart;
-		    lastEnd   = ixEnd;
 		    useIdx[i]   = 1;
 		}
 		/*
@@ -741,9 +740,18 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	if (TkTextDLineInfo(textPtr, &index1, &x, &y, &width, &height, &base)
 		== 0) {
 	    char buf[TCL_INTEGER_SPACE * 5];
+	    Tcl_Obj *el;
+	    el = Tcl_GetObjResult(interp);
 
+	    Tcl_ListObjAppendElement(interp,el,Tcl_NewIntObj(x));
+	    Tcl_ListObjAppendElement(interp,el,Tcl_NewIntObj(y));
+	    Tcl_ListObjAppendElement(interp,el,Tcl_NewIntObj(width));
+	    Tcl_ListObjAppendElement(interp,el,Tcl_NewIntObj(height));
+	    Tcl_ListObjAppendElement(interp,el,Tcl_NewIntObj(base));
+	    /*
 	    sprintf(buf, "%d %d %d %d %d", x, y, width, height, base);
 	    Tcl_SetResult(interp, buf, TCL_VOLATILE);
+	    */
 	}
     } else if ((c == 'e') && (strncmp(argv[1], "edit", length) == 0)) {
         result = TextEditCmd(textPtr, interp, argc, argv);
@@ -774,7 +782,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 		result = TCL_ERROR;
 		goto done;
 	    }
-	    /* Using <= gives perl/Tk null strings for equal case and undef 
+	    /* Using <= gives perl/Tk null strings for equal case and undef
 	       for reversed range - which seems "correct"
 	     */
 	    if (TkTextIndexCmp(&index1, &index2) <= 0) {
@@ -806,7 +814,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	}
 	if (found > 1) {
 	    Tcl_SetObjResult(interp, objPtr);
-	} 
+	}
     } else if ((c == 'i') && (strncmp(argv[1], "index", length) == 0)
 	    && (length >= 3)) {
 	char buf[200];
@@ -827,7 +835,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
     } else if ((c == 'i') && (strncmp(argv[1], "insert", length) == 0)
 	    && (length >= 3)) {
 	int i, j, numTags;
-	Arg *tagNames;
+	Tcl_Obj * *tagNames;
 	TkTextTag **oldTagArrayPtr;
 
 	if (argc < 4) {
@@ -1136,8 +1144,6 @@ ConfigureText(interp, textPtr, argc, argv, flags)
 	    || (textPtr->selTagPtr->spacing2String != NULL)
 	    || (textPtr->selTagPtr->spacing3String != NULL)
 	    || (textPtr->selTagPtr->tabString != NULL)
-	    || (textPtr->selTagPtr->state != TK_STATE_NULL)
-	    || (textPtr->selTagPtr->elideString != NULL)
 	    || (textPtr->selTagPtr->underlineString != NULL)
 	    || (textPtr->selTagPtr->elideString != NULL)
 	    || (textPtr->selTagPtr->wrapMode != TEXT_WRAPMODE_NULL)) {
@@ -1972,9 +1978,10 @@ TextBlinkProc(clientData)
 		textPtr->insertOnTime, TextBlinkProc, (ClientData) textPtr);
     }
     TkTextMarkSegToIndex(textPtr, textPtr->insertMarkPtr, &index);
-    TkTextCharBbox(textPtr, &index, &x, &y, &w, &h);
-    TkTextRedrawRegion(textPtr, x - textPtr->insertWidth / 2, y,
-	    textPtr->insertWidth, h);
+    if (TkTextCharBbox(textPtr, &index, &x, &y, &w, &h) == 0) {
+	TkTextRedrawRegion(textPtr, x - textPtr->insertWidth / 2, y,
+		textPtr->insertWidth, h);
+    }
 }
 
 /*
@@ -2091,6 +2098,10 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	return TCL_ERROR;
     }
     pattern = argv[i];
+#ifdef _LANG
+    patObj  = objv[i];
+    Tcl_IncrRefCount(patObj);
+#endif
 
     /*
      * Convert the pattern to lower-case if we're supposed to ignore case.
@@ -2146,8 +2157,10 @@ TextSearchCmd(textPtr, interp, argc, argv)
     if (exact) {
 	patLength = strlen(pattern);
     } else {
+#ifndef _LANG
 	patObj = Tcl_NewStringObj(pattern, -1);
 	Tcl_IncrRefCount(patObj);
+#endif
 	regexp = Tcl_GetRegExpFromObj(interp, patObj,
 		(noCase ? TCL_REG_NOCASE : 0) | TCL_REG_ADVANCED);
 	if (regexp == NULL) {
@@ -2191,7 +2204,7 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	 * If we're ignoring case, convert the line to lower case.
 	 */
 
-	if (noCase) {
+	if (noCase && exact) {
 	    Tcl_DStringSetLength(&line,
 		    Tcl_UtfToLower(Tcl_DStringValue(&line)));
 	}
@@ -2342,7 +2355,7 @@ TextSearchCmd(textPtr, interp, argc, argv)
 		}
 	    }
 	    if (varName != NULL) {
-		Tcl_Obj *temp = Tcl_NewIntObj(matchLength);
+		Tcl_Obj *temp = Tcl_NewIntObj(numChars);
 		if (Tcl_ObjSetVar2(interp, varName, NULL, temp, TCL_LEAVE_ERR_MSG)
 			    == NULL) {
 		        code = TCL_ERROR;
@@ -2390,6 +2403,11 @@ TextSearchCmd(textPtr, interp, argc, argv)
     if (patObj != NULL) {
 	Tcl_DecrRefCount(patObj);
     }
+#ifdef _LANG
+    if (regexp) {
+	Lang_FreeRegExp(regexp);
+    }
+#endif
     return code;
 }
 
@@ -2419,7 +2437,7 @@ TkTextGetTabs(interp, tkwin, arg)
     Tcl_Interp *interp;			/* Used for error reporting. */
     Tk_Window tkwin;			/* Window in which the tabs will be
 					 * used. */
-    Arg arg;				/* Description of the tab stops.  See
+    Tcl_Obj * arg;				/* Description of the tab stops.  See
 					 * the text manual entry for details. */
 {
     int argc, i, count, c;
@@ -2752,7 +2770,7 @@ DumpSegment(interp, key, value, arg, command, index, what)
     Tcl_Interp *interp;
     char *key;			/* Segment type key */
     char *value;		/* Segment value */
-    Tcl_Obj *arg;		/* Segment as an Arg */
+    Tcl_Obj *arg;		/* Segment as an Tcl_Obj * */
     LangCallback *command;	/* Script callback */
     TkTextIndex *index;         /* index with line/byte position info */
     int what;			/* Look for TK_DUMP_INDEX bit */
