@@ -1,9 +1,9 @@
 /*
  * tclEvent.c --
  *
- *	This file implements some general event related interfaces including
- *	background errors, exit handlers, and the "vwait" and "update"
- *	command procedures.
+ *      This file implements some general event related interfaces including
+ *      background errors, exit handlers, and the "vwait" and "update"
+ *      command procedures.
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
  * Copyright (c) 1994-1998 Sun Microsystems, Inc.
@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclEvent.c,v 1.23 2002/08/05 03:24:40 dgp Exp $
+ * RCS: @(#) $Id: tclEvent.c,v 1.28.2.1 2003/05/13 12:44:07 dkf Exp $
  */
 #include "tkPort.h"
 #include "tkInt.h"
@@ -25,17 +25,17 @@
  */
 
 typedef struct BgError {
-    Tcl_Interp *interp;		/* Interpreter in which error occurred.  NULL
+    Tcl_Interp *interp;         /* Interpreter in which error occurred.  NULL
 				 * means this error report has been cancelled
 				 * (a previous report generated a break). */
-    char *errorMsg;		/* Copy of the error message (the interp's
+    char *errorMsg;             /* Copy of the error message (the interp's
 				 * result when the error occurred).
 				 * Malloc-ed. */
-    char *errorInfo;		/* Value of the errorInfo variable
+    char *errorInfo;            /* Value of the errorInfo variable
 				 * (malloc-ed). */
-    char *errorCode;		/* Value of the errorCode variable
+    char *errorCode;            /* Value of the errorCode variable
 				 * (malloc-ed). */
-    struct BgError *nextPtr;	/* Next in list of all pending error
+    struct BgError *nextPtr;    /* Next in list of all pending error
 				 * reports for this interpreter, or NULL
 				 * for end of list. */
 } BgError;
@@ -47,10 +47,10 @@ typedef struct BgError {
  */
 
 typedef struct ErrAssocData {
-    BgError *firstBgPtr;	/* First in list of all background errors
+    BgError *firstBgPtr;        /* First in list of all background errors
 				 * waiting to be processed for this
 				 * interpreter (NULL if none). */
-    BgError *lastBgPtr;		/* Last in list of all background errors
+    BgError *lastBgPtr;         /* Last in list of all background errors
 				 * waiting to be processed for this
 				 * interpreter (NULL if none). */
 } ErrAssocData;
@@ -61,8 +61,8 @@ typedef struct ErrAssocData {
  */
 
 typedef struct ExitHandler {
-    Tcl_ExitProc *proc;		/* Procedure to call when process exits. */
-    ClientData clientData;	/* One word of information to pass to proc. */
+    Tcl_ExitProc *proc;         /* Procedure to call when process exits. */
+    ClientData clientData;      /* One word of information to pass to proc. */
     struct ExitHandler *nextPtr;/* Next in list of all exit handlers for
 				 * this application, or NULL for end of list. */
 } ExitHandler;
@@ -92,10 +92,10 @@ static int subsystemsInitialized = 0;
 typedef struct ThreadSpecificData {
     ExitHandler *firstExitPtr;  /* First in list of all exit handlers for
 				 * this thread. */
-    int inExit;			/* True when this thread is exiting. This
+    int inExit;                 /* True when this thread is exiting. This
 				 * is used as a hack to decide to close
 				 * the standard channels. */
-    Tcl_Obj *tclLibraryPath;	/* Path(s) to the Tcl library */
+    Tcl_Obj *tclLibraryPath;    /* Path(s) to the Tcl library */
 } ThreadSpecificData;
 static Tcl_ThreadDataKey dataKey;
 
@@ -108,11 +108,11 @@ char *tclLibraryPathStr;
  * Prototypes for procedures referenced only in this file:
  */
 
-static void		BgErrorDeleteProc _ANSI_ARGS_((ClientData clientData,
+static void             BgErrorDeleteProc _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp));
-static void		HandleBgErrors _ANSI_ARGS_((ClientData clientData));
+static void             HandleBgErrors _ANSI_ARGS_((ClientData clientData));
 #else
-static char *		VwaitVarProc _ANSI_ARGS_((ClientData clientData,
+static char *           VwaitVarProc _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, Var name1, CONST char *name2,
 			    int flags));
 
@@ -126,24 +126,24 @@ static char *		VwaitVarProc _ANSI_ARGS_((ClientData clientData,
  *
  * Tcl_BackgroundError --
  *
- *	This procedure is invoked to handle errors that occur in Tcl
- *	commands that are invoked in "background" (e.g. from event or
- *	timer bindings).
+ *      This procedure is invoked to handle errors that occur in Tcl
+ *      commands that are invoked in "background" (e.g. from event or
+ *      timer bindings).
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The command "bgerror" is invoked later as an idle handler to
- *	process the error, passing it the error message.  If that fails,
- *	then an error message is output on stderr.
+ *      The command "bgerror" is invoked later as an idle handler to
+ *      process the error, passing it the error message.  If that fails,
+ *      then an error message is output on stderr.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_BackgroundError(interp)
-    Tcl_Interp *interp;		/* Interpreter in which an error has
+    Tcl_Interp *interp;         /* Interpreter in which an error has
 				 * occurred. */
 {
     BgError *errPtr;
@@ -212,21 +212,21 @@ Tcl_BackgroundError(interp)
  *
  * HandleBgErrors --
  *
- *	This procedure is invoked as an idle handler to process all of
- *	the accumulated background errors.
+ *      This procedure is invoked as an idle handler to process all of
+ *      the accumulated background errors.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Depends on what actions "bgerror" takes for the errors.
+ *      Depends on what actions "bgerror" takes for the errors.
  *
  *----------------------------------------------------------------------
  */
 
 static void
 HandleBgErrors(clientData)
-    ClientData clientData;	/* Pointer to ErrAssocData structure. */
+    ClientData clientData;      /* Pointer to ErrAssocData structure. */
 {
     Tcl_Interp *interp;
     CONST char *argv[2];
@@ -261,59 +261,59 @@ HandleBgErrors(clientData)
 	argv[1] = assocPtr->firstBgPtr->errorMsg;
 
 	Tcl_AllowExceptions(interp);
-        Tcl_Preserve((ClientData) interp);
+	Tcl_Preserve((ClientData) interp);
 	code = TclGlobalInvoke(interp, 2, argv, 0);
 	if (code == TCL_ERROR) {
 
-            /*
-             * If the interpreter is safe, we look for a hidden command
-             * named "bgerror" and call that with the error information.
-             * Otherwise, simply ignore the error. The rationale is that
-             * this could be an error caused by a malicious applet trying
-             * to cause an infinite barrage of error messages. The hidden
-             * "bgerror" command can be used by a security policy to
-             * interpose on such attacks and e.g. kill the applet after a
-             * few attempts.
-             */
+	    /*
+	     * If the interpreter is safe, we look for a hidden command
+	     * named "bgerror" and call that with the error information.
+	     * Otherwise, simply ignore the error. The rationale is that
+	     * this could be an error caused by a malicious applet trying
+	     * to cause an infinite barrage of error messages. The hidden
+	     * "bgerror" command can be used by a security policy to
+	     * interpose on such attacks and e.g. kill the applet after a
+	     * few attempts.
+	     */
 
-            if (Tcl_IsSafe(interp)) {
+	    if (Tcl_IsSafe(interp)) {
 		Tcl_SavedResult save;
 
 		Tcl_SaveResult(interp, &save);
-                TclGlobalInvoke(interp, 2, argv, TCL_INVOKE_HIDDEN);
+		TclGlobalInvoke(interp, 2, argv, TCL_INVOKE_HIDDEN);
 		Tcl_RestoreResult(interp, &save);
 
-                goto doneWithInterp;
-            }
+		goto doneWithInterp;
+	    }
 
-            /*
-             * We have to get the error output channel at the latest possible
-             * time, because the eval (above) might have changed the channel.
-             */
+	    /*
+	     * We have to get the error output channel at the latest possible
+	     * time, because the eval (above) might have changed the channel.
+	     */
 
-            errChannel = Tcl_GetStdChannel(TCL_STDERR);
-            if (errChannel != (Tcl_Channel) NULL) {
+	    errChannel = Tcl_GetStdChannel(TCL_STDERR);
+	    if (errChannel != (Tcl_Channel) NULL) {
 		char *string;
 		int len;
 
 		string = Tcl_GetStringFromObj(Tcl_GetObjResult(interp), &len);
 		if (Tcl_FindCommand(interp, "bgerror", NULL, TCL_GLOBAL_ONLY) == NULL) {
-                    Tcl_WriteChars(errChannel, assocPtr->firstBgPtr->errorInfo, -1);
-                    Tcl_WriteChars(errChannel, "\n", -1);
-                } else {
-                    Tcl_WriteChars(errChannel,
-                            "bgerror failed to handle background error.\n",
-                            -1);
-                    Tcl_WriteChars(errChannel, "    Original error: ", -1);
-                    Tcl_WriteChars(errChannel, assocPtr->firstBgPtr->errorMsg,
-                            -1);
-                    Tcl_WriteChars(errChannel, "\n", -1);
-                    Tcl_WriteChars(errChannel, "    Error in bgerror: ", -1);
-                    Tcl_WriteChars(errChannel, string, len);
-                    Tcl_WriteChars(errChannel, "\n", -1);
-                }
-                Tcl_Flush(errChannel);
-            }
+		    Tcl_WriteChars(errChannel, assocPtr->firstBgPtr->errorInfo, -1);
+		    Tcl_WriteChars(errChannel, "\n", -1);
+		} else {
+		    Tcl_WriteChars(errChannel,
+			    "bgerror failed to handle background error.\n",
+			    -1);
+		    Tcl_WriteChars(errChannel, "    Original error: ", -1);
+		    Tcl_WriteChars(errChannel, assocPtr->firstBgPtr->errorMsg,
+			    -1);
+		    Tcl_WriteChars(errChannel, "\n", -1);
+		    Tcl_WriteChars(errChannel, "    Error in bgerror: ", -1);
+		    Tcl_WriteChars(errChannel, string, len);
+		    Tcl_WriteChars(errChannel, "\n", -1);
+		}
+		Tcl_Flush(errChannel);
+	    }
 	} else if (code == TCL_BREAK) {
 
 	    /*
@@ -344,9 +344,9 @@ doneWithInterp:
 	    assocPtr->firstBgPtr = errPtr;
 	}
 
-        if (interp != NULL) {
-            Tcl_Release((ClientData) interp);
-        }
+	if (interp != NULL) {
+	    Tcl_Release((ClientData) interp);
+	}
     }
     assocPtr->lastBgPtr = NULL;
 
@@ -358,25 +358,25 @@ doneWithInterp:
  *
  * BgErrorDeleteProc --
  *
- *	This procedure is associated with the "tclBgError" assoc data
- *	for an interpreter;  it is invoked when the interpreter is
- *	deleted in order to free the information assoicated with any
- *	pending error reports.
+ *      This procedure is associated with the "tclBgError" assoc data
+ *      for an interpreter;  it is invoked when the interpreter is
+ *      deleted in order to free the information assoicated with any
+ *      pending error reports.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Background error information is freed: if there were any
- *	pending error reports, they are cancelled.
+ *      Background error information is freed: if there were any
+ *      pending error reports, they are cancelled.
  *
  *----------------------------------------------------------------------
  */
 
 static void
 BgErrorDeleteProc(clientData, interp)
-    ClientData clientData;	/* Pointer to ErrAssocData structure. */
-    Tcl_Interp *interp;		/* Interpreter being deleted. */
+    ClientData clientData;      /* Pointer to ErrAssocData structure. */
+    Tcl_Interp *interp;         /* Interpreter being deleted. */
 {
     ErrAssocData *assocPtr = (ErrAssocData *) clientData;
     BgError *errPtr;
@@ -399,15 +399,15 @@ BgErrorDeleteProc(clientData, interp)
  *
  * Tcl_CreateExitHandler --
  *
- *	Arrange for a given procedure to be invoked just before the
- *	application exits.
+ *      Arrange for a given procedure to be invoked just before the
+ *      application exits.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Proc will be invoked with clientData as argument when the
- *	application exits.
+ *      Proc will be invoked with clientData as argument when the
+ *      application exits.
  *
  *----------------------------------------------------------------------
  */
@@ -415,8 +415,8 @@ BgErrorDeleteProc(clientData, interp)
 
 void
 Tcl_CreateExitHandler(proc, clientData)
-    Tcl_ExitProc *proc;		/* Procedure to invoke. */
-    ClientData clientData;	/* Arbitrary value to pass to proc. */
+    Tcl_ExitProc *proc;         /* Procedure to invoke. */
+    ClientData clientData;      /* Arbitrary value to pass to proc. */
 {
     ExitHandler *exitPtr;
 
@@ -434,24 +434,24 @@ Tcl_CreateExitHandler(proc, clientData)
  *
  * Tcl_DeleteExitHandler --
  *
- *	This procedure cancels an existing exit handler matching proc
- *	and clientData, if such a handler exits.
+ *      This procedure cancels an existing exit handler matching proc
+ *      and clientData, if such a handler exits.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	If there is an exit handler corresponding to proc and clientData
- *	then it is cancelled;  if no such handler exists then nothing
- *	happens.
+ *      If there is an exit handler corresponding to proc and clientData
+ *      then it is cancelled;  if no such handler exists then nothing
+ *      happens.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_DeleteExitHandler(proc, clientData)
-    Tcl_ExitProc *proc;		/* Procedure that was previously registered. */
-    ClientData clientData;	/* Arbitrary value to pass to proc. */
+    Tcl_ExitProc *proc;         /* Procedure that was previously registered. */
+    ClientData clientData;      /* Arbitrary value to pass to proc. */
 {
     ExitHandler *exitPtr, *prevPtr;
 
@@ -478,23 +478,23 @@ Tcl_DeleteExitHandler(proc, clientData)
  *
  * Tcl_CreateThreadExitHandler --
  *
- *	Arrange for a given procedure to be invoked just before the
- *	current thread exits.
+ *      Arrange for a given procedure to be invoked just before the
+ *      current thread exits.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Proc will be invoked with clientData as argument when the
- *	application exits.
+ *      Proc will be invoked with clientData as argument when the
+ *      application exits.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_CreateThreadExitHandler(proc, clientData)
-    Tcl_ExitProc *proc;		/* Procedure to invoke. */
-    ClientData clientData;	/* Arbitrary value to pass to proc. */
+    Tcl_ExitProc *proc;         /* Procedure to invoke. */
+    ClientData clientData;      /* Arbitrary value to pass to proc. */
 {
     ExitHandler *exitPtr;
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
@@ -511,24 +511,24 @@ Tcl_CreateThreadExitHandler(proc, clientData)
  *
  * Tcl_DeleteThreadExitHandler --
  *
- *	This procedure cancels an existing exit handler matching proc
- *	and clientData, if such a handler exits.
+ *      This procedure cancels an existing exit handler matching proc
+ *      and clientData, if such a handler exits.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	If there is an exit handler corresponding to proc and clientData
- *	then it is cancelled;  if no such handler exists then nothing
- *	happens.
+ *      If there is an exit handler corresponding to proc and clientData
+ *      then it is cancelled;  if no such handler exists then nothing
+ *      happens.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_DeleteThreadExitHandler(proc, clientData)
-    Tcl_ExitProc *proc;		/* Procedure that was previously registered. */
-    ClientData clientData;	/* Arbitrary value to pass to proc. */
+    Tcl_ExitProc *proc;         /* Procedure that was previously registered. */
+    ClientData clientData;      /* Arbitrary value to pass to proc. */
 {
     ExitHandler *exitPtr, *prevPtr;
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
@@ -553,21 +553,21 @@ Tcl_DeleteThreadExitHandler(proc, clientData)
  *
  * Tcl_Exit --
  *
- *	This procedure is called to terminate the application.
+ *      This procedure is called to terminate the application.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	All existing exit handlers are invoked, then the application
- *	ends.
+ *      All existing exit handlers are invoked, then the application
+ *      ends.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_Exit(status)
-    int status;			/* Exit status for application;  typically
+    int status;                 /* Exit status for application;  typically
 				 * 0 for normal return, 1 for error return. */
 {
     Tcl_Finalize();
@@ -579,19 +579,19 @@ Tcl_Exit(status)
  *
  * TclSetLibraryPath --
  *
- *	Set the path that will be used for searching for init.tcl and
- *	encodings when an interp is being created.
+ *      Set the path that will be used for searching for init.tcl and
+ *      encodings when an interp is being created.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Changing the library path will affect what directories are
- *	examined when looking for encodings for all interps from that
- *	point forward.
+ *      Changing the library path will affect what directories are
+ *      examined when looking for encodings for all interps from that
+ *      point forward.
  *
- *	The refcount of the new library path is incremented and the
- *	refcount of the old path is decremented.
+ *      The refcount of the new library path is incremented and the
+ *      refcount of the old path is decremented.
  *
  *-------------------------------------------------------------------------
  */
@@ -599,7 +599,7 @@ Tcl_Exit(status)
 #ifndef _LANG
 void
 TclSetLibraryPath(pathPtr)
-    Tcl_Obj *pathPtr;		/* A Tcl list object whose elements are
+    Tcl_Obj *pathPtr;           /* A Tcl list object whose elements are
 				 * the new library path. */
 {
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
@@ -625,14 +625,14 @@ TclSetLibraryPath(pathPtr)
  *
  * TclGetLibraryPath --
  *
- *	Return a Tcl list object whose elements are the library path.
- *	The caller should not modify the contents of the returned object.
+ *      Return a Tcl list object whose elements are the library path.
+ *      The caller should not modify the contents of the returned object.
  *
  * Results:
- *	As above.
+ *      As above.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *-------------------------------------------------------------------------
  */
@@ -662,34 +662,35 @@ TclGetLibraryPath()
  *
  * TclInitSubsystems --
  *
- *	Initialize various subsytems in Tcl.  This should be called the
- *	first time an interp is created, or before any of the subsystems
- *	are used.  This function ensures an order for the initialization
- *	of subsystems:
+ *      Initialize various subsytems in Tcl.  This should be called the
+ *      first time an interp is created, or before any of the subsystems
+ *      are used.  This function ensures an order for the initialization
+ *      of subsystems:
  *
- *	1. that cannot be initialized in lazy order because they are
- *	mutually dependent.
+ *      1. that cannot be initialized in lazy order because they are
+ *      mutually dependent.
  *
- *	2. so that they can be finalized in a known order w/o causing
- *	the subsequent re-initialization of a subsystem in the act of
- *	shutting down another.
+ *      2. so that they can be finalized in a known order w/o causing
+ *      the subsequent re-initialization of a subsystem in the act of
+ *      shutting down another.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Varied, see the respective initialization routines.
+ *      Varied, see the respective initialization routines.
  *
  *-------------------------------------------------------------------------
  */
 
 void
 TclInitSubsystems(argv0)
-    CONST char *argv0;		/* Name of executable from argv[0] to main()
+    CONST char *argv0;          /* Name of executable from argv[0] to main()
 				 * in native multi-byte encoding. */
 {
     ThreadSpecificData *tsdPtr;
 
+
     if (inFinalize != 0) {
 	panic("TclInitSubsystems called while finalizing");
     }
@@ -762,16 +763,16 @@ TclInitSubsystems(argv0)
  *
  * Tcl_Finalize --
  *
- *	Shut down Tcl.  First calls registered exit handlers, then
- *	carefully shuts down various subsystems.
- *	Called by Tcl_Exit or when the Tcl shared library is being
- *	unloaded.
+ *      Shut down Tcl.  First calls registered exit handlers, then
+ *      carefully shuts down various subsystems.
+ *      Called by Tcl_Exit or when the Tcl shared library is being
+ *      unloaded.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Varied, see the respective finalization routines.
+ *      Varied, see the respective finalization routines.
  *
  *----------------------------------------------------------------------
  */
@@ -779,15 +780,18 @@ TclInitSubsystems(argv0)
 void
 Tcl_Finalize()
 {
-#ifndef _LANG
     ExitHandler *exitPtr;
-    ThreadSpecificData *tsdPtr;
 
     TclpInitLock();
     if (subsystemsInitialized != 0) {
 	subsystemsInitialized = 0;
 
-	tsdPtr = TCL_TSD_INIT(&dataKey);
+	/*
+	 * Ensure the thread-specific data is initialised as it is
+	 * used in Tcl_FinalizeThread()
+	 */
+
+	(void) TCL_TSD_INIT(&dataKey);
 
 	/*
 	 * Invoke exit handlers first.
@@ -822,6 +826,7 @@ Tcl_Finalize()
 
 	Tcl_FinalizeThread();
 
+#ifndef _LANG
 	/*
 	 * Now finalize the Tcl execution environment.  Note that this
 	 * must be done after the exit handlers, because there are
@@ -831,6 +836,18 @@ Tcl_Finalize()
 	TclFinalizeCompExecEnv();
 	TclFinalizeEnvironment();
 
+	/*
+	 * Finalizing the filesystem must come after anything which
+	 * might conceivably interact with the 'Tcl_FS' API.
+	 */
+	TclFinalizeFilesystem();
+
+	/*
+	 * We must be sure the encoding finalization doesn't need
+	 * to examine the filesystem in any way.  Since it only
+	 * needs to clean up internal data structures, this is
+	 * fine.
+	 */
 	TclFinalizeEncodingSubsystem();
 
 	if (tclExecutableName != NULL) {
@@ -855,22 +872,33 @@ Tcl_Finalize()
 
 	TclFinalizeSynchronization();
 
-	/**
-	 * Finalizing the filesystem must come after anything which
-	 * might conceivably interact with the 'Tcl_FS' API.  This
-	 * will also unload any extensions which have been loaded.
+	/*
+	 * We defer unloading of packages until very late
+	 * to avoid memory access issues.  Both exit callbacks and
+	 * synchronization variables may be stored in packages.
+	 *
+	 * Note that TclFinalizeLoad unloads packages in the reverse
+	 * of the order they were loaded in (i.e. last to be loaded
+	 * is the first to be unloaded).  This can be important for
+	 * correct unloading when dependencies exist.
+	 *
+	 * Once load has been finalized, we will have deleted any
+	 * temporary copies of shared libraries and can therefore
+	 * reset the filesystem to its original state.
 	 */
-	TclFinalizeFilesystem();
+
+	TclFinalizeLoad();
+	TclResetFilesystem();
 
 	/*
 	 * There shouldn't be any malloc'ed memory after this.
 	 */
 
 	TclFinalizeMemorySubsystem();
+#endif
 	inFinalize = 0;
     }
     TclpInitUnlock();
-#endif
 }
 
 /*
@@ -878,14 +906,14 @@ Tcl_Finalize()
  *
  * Tcl_FinalizeThread --
  *
- *	Runs the exit handlers to allow Tcl to clean up its state
- *	about a particular thread.
+ *      Runs the exit handlers to allow Tcl to clean up its state
+ *      about a particular thread.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Varied, see the respective finalization routines.
+ *      Varied, see the respective finalization routines.
  *
  *----------------------------------------------------------------------
  */
@@ -899,10 +927,6 @@ Tcl_FinalizeThread()
 	    (ThreadSpecificData *)TclThreadDataKeyGet(&dataKey);
 
     if (tsdPtr != NULL) {
-	/*
-	 * Invoke thread exit handlers first.
-	 */
-
 	tsdPtr->inExit = 1;
 
 	/*
@@ -930,13 +954,19 @@ Tcl_FinalizeThread()
 	TclFinalizeIOSubsystem();
 	TclFinalizeNotifier();
 	TclFinalizeAsync();
+    }
 
 	/*
 	 * Blow away all thread local storage blocks.
+     *
+     * Note that Tcl API allows creation of threads which do not use any
+     * Tcl interp or other Tcl subsytems. Those threads might, however,
+     * use thread local storage, so we must unconditionally finalize it.
+     *
+     * Fix [Bug #571002]
 	 */
 
 	TclFinalizeThreadData();
-    }
 #endif
 }
 
@@ -945,24 +975,46 @@ Tcl_FinalizeThread()
  *
  * TclInExit --
  *
- *	Determines if we are in the middle of exit-time cleanup.
+ *      Determines if we are in the middle of exit-time cleanup.
  *
  * Results:
- *	If we are in the middle of exiting, 1, otherwise 0.
+ *      If we are in the middle of exiting, 1, otherwise 0.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
 
 int
 TclInExit()
+{
+    return inFinalize;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TclInThreadExit --
+ *
+ *      Determines if we are in the middle of thread exit-time cleanup.
+ *
+ * Results:
+ *      If we are in the middle of exiting this thread, 1, otherwise 0.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+TclInThreadExit()
 {
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    TclThreadDataKeyGet(&dataKey);
     if (tsdPtr == NULL) {
-	return inFinalize;
+	return 0;
     } else {
 	return tsdPtr->inExit;
     }
@@ -974,14 +1026,14 @@ TclInExit()
  *
  * Tcl_VwaitObjCmd --
  *
- *	This procedure is invoked to process the "vwait" Tcl command.
- *	See the user documentation for details on what it does.
+ *      This procedure is invoked to process the "vwait" Tcl command.
+ *      See the user documentation for details on what it does.
  *
  * Results:
- *	A standard Tcl result.
+ *      A standard Tcl result.
  *
  * Side effects:
- *	See the user documentation.
+ *      See the user documentation.
  *
  *----------------------------------------------------------------------
  */
@@ -989,15 +1041,15 @@ TclInExit()
 	/* ARGSUSED */
 int
 Tcl_VwaitObjCmd(clientData, interp, objc, objv)
-    ClientData clientData;	/* Not used. */
-    Tcl_Interp *interp;		/* Current interpreter. */
-    int objc;			/* Number of arguments. */
-    Tcl_Obj *CONST objv[];	/* Argument objects. */
+    ClientData clientData;      /* Not used. */
+    Tcl_Interp *interp;         /* Current interpreter. */
+    int objc;                   /* Number of arguments. */
+    Tcl_Obj *CONST objv[];      /* Argument objects. */
 {
     int done, foundEvent;
 
     if (objc != 2) {
-        Tcl_WrongNumArgs(interp, 1, objv, "name");
+	Tcl_WrongNumArgs(interp, 1, objv, "name");
 	return TCL_ERROR;
     }
     if (Lang_TraceVar(interp, objv[1],
@@ -1031,11 +1083,11 @@ Tcl_VwaitObjCmd(clientData, interp, objc, objv)
 	/* ARGSUSED */
 static char *
 VwaitVarProc(clientData, interp, name1, name2, flags)
-    ClientData clientData;	/* Pointer to integer to set to 1. */
-    Tcl_Interp *interp;		/* Interpreter containing variable. */
-    Var name1;			/* Name of variable. */
-    CONST char *name2;		/* Second part of variable name. */
-    int flags;			/* Information about what happened. */
+    ClientData clientData;      /* Pointer to integer to set to 1. */
+    Tcl_Interp *interp;         /* Interpreter containing variable. */
+    Var name1;                  /* Name of variable. */
+    CONST char *name2;          /* Second part of variable name. */
+    int flags;                  /* Information about what happened. */
 {
     int *donePtr = (int *) clientData;
 
@@ -1048,14 +1100,14 @@ VwaitVarProc(clientData, interp, name1, name2, flags)
  *
  * Tcl_UpdateObjCmd --
  *
- *	This procedure is invoked to process the "update" Tcl command.
- *	See the user documentation for details on what it does.
+ *      This procedure is invoked to process the "update" Tcl command.
+ *      See the user documentation for details on what it does.
  *
  * Results:
- *	A standard Tcl result.
+ *      A standard Tcl result.
  *
  * Side effects:
- *	See the user documentation.
+ *      See the user documentation.
  *
  *----------------------------------------------------------------------
  */
@@ -1063,13 +1115,13 @@ VwaitVarProc(clientData, interp, name1, name2, flags)
 	/* ARGSUSED */
 int
 Tcl_UpdateObjCmd(clientData, interp, objc, objv)
-    ClientData clientData;	/* Not used. */
-    Tcl_Interp *interp;		/* Current interpreter. */
-    int objc;			/* Number of arguments. */
-    Tcl_Obj *CONST objv[];	/* Argument objects. */
+    ClientData clientData;      /* Not used. */
+    Tcl_Interp *interp;         /* Current interpreter. */
+    int objc;                   /* Number of arguments. */
+    Tcl_Obj *CONST objv[];      /* Argument objects. */
 {
     int optionIndex;
-    int flags = 0;		/* Initialized to avoid compiler warning. */
+    int flags = 0;              /* Initialized to avoid compiler warning. */
     static CONST char *updateOptions[] = {"idletasks", (char *) NULL};
     enum updateOptions {REGEXP_IDLETASKS};
 
@@ -1090,7 +1142,7 @@ Tcl_UpdateObjCmd(clientData, interp, objc, objv)
 	    }
 	}
     } else {
-        Tcl_WrongNumArgs(interp, 1, objv, "?idletasks?");
+	Tcl_WrongNumArgs(interp, 1, objv, "?idletasks?");
 	return TCL_ERROR;
     }
 
