@@ -14,7 +14,9 @@
 #include "tkPort.h"
 #include "Lang.h"
 
+#ifndef UCHAR
 #define UCHAR(x) ((unsigned char) (x))
+#endif
 
 /*
  * This flag indicates whether this module has been initialized.
@@ -836,7 +838,6 @@ processInteger:
 	    afterPtr->commandPtr = LangMakeCallback(objPtr);
 	    Tcl_DecrRefCount(objPtr);
 	}
-	Tcl_IncrRefCount(afterPtr->commandPtr);
 	/*
 	 * The variable below is used to generate unique identifiers for
 	 * after commands.  This id can wrap around, which can potentially
@@ -880,6 +881,10 @@ processInteger:
 	    }
 	    if (objc == 3) {
 		commandPtr = objv[2];
+		afterPtr = GetAfterEvent(assocPtr, commandPtr);
+		if (afterPtr != NULL) {
+		    goto got_after;
+		}
 	    } else {
 		commandPtr = Tcl_ConcatObj(objc-2, (Tcl_Obj **)objv+2);;
 	    }
@@ -900,6 +905,7 @@ processInteger:
 	    if (objc != 3) {
 		Tcl_DecrRefCount(commandPtr);
 	    }
+	got_after:
 	    if (afterPtr != NULL) {
 		if (afterPtr->token != NULL) {
 		    Tcl_DeleteTimerHandler(afterPtr->token);
