@@ -10,11 +10,12 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclUnixChan.c,v 1.39 2002/09/03 02:01:25 hobbs Exp $
+ * RCS: @(#) $Id: tclUnixChan.c,v 1.42.2.1 2003/10/23 17:49:06 andreas_kupries Exp $
  */
 
 #include "tclInt.h"	/* Internal definitions for Tcl. */
 #include "tclPort.h"	/* Portability features for Tcl. */
+#include "tclIO.h"	/* To get Channel type declaration. */
 
 /*
  * sys/ioctl.h has already been included by tclPort.h.	Including termios.h
@@ -293,6 +294,10 @@ static int		TtySetOptionProc _ANSI_ARGS_((ClientData instanceData,
 #endif	/* SUPPORTS_TTY */
 static int		WaitForConnect _ANSI_ARGS_((TcpState *statePtr,
 			    int *errorCodePtr));
+static Tcl_Channel	MakeTcpClientChannelMode _ANSI_ARGS_(
+			    (ClientData tcpSocket,
+			    int mode));
+
 
 /*
  * This structure describes the channel type structure for file based IO:
@@ -549,21 +554,12 @@ FileCloseProc(instanceData, interp)
      * Do not close standard channels while in thread-exit.
      */
 
-    if (!TclInExit()
+    if (!TclInThreadExit()
 	    || ((fsPtr->fd != 0) && (fsPtr->fd != 1) && (fsPtr->fd != 2))) {
 	if (close(fsPtr->fd) < 0) {
 	    errorCode = errno;
 	}
     }
-#ifdef DEPRECATED
-    for (nextPtrPtr = &(tsdPtr->firstFilePtr); (*nextPtrPtr) != NULL;
-	 nextPtrPtr = &((*nextPtrPtr)->nextPtr)) {
-	if ((*nextPtrPtr) == fsPtr) {
-	    (*nextPtrPtr) = fsPtr->nextPtr;
-	    break;
-	}
-    }
-#endif /* DEPRECATED */
     ckfree((char *) fsPtr);
     return errorCode;
 }
@@ -1834,8 +1830,10 @@ TclpOpenFileChannel(interp, pathPtr, mode, permissions)
     }
 
 #ifdef DEPRECATED
-    fsPtr->nextPtr = tsdPtr->firstFilePtr;
-    tsdPtr->firstFilePtr = fsPtr;
+    if (channelTypePtr == &fileChannelType) {
+        fsPtr->nextPtr = tsdPtr->firstFilePtr;
+        tsdPtr->firstFilePtr = fsPtr;
+    }
 #endif /* DEPRECATED */
     fsPtr->validMask = channelPermissions | TCL_EXCEPTION;
     fsPtr->fd = fd;
@@ -1891,8 +1889,8 @@ Tcl_MakeFileChannel(handle, mode)
 #ifdef DEPRECATED
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 #endif /* DEPRECATED */
-    int socketType = 0;
-    socklen_t argLength = sizeof(int);
+    struct sockaddr sockaddr;
+    socklen_t sockaddrLen = sizeof(sockaddr);
 
     if (mode == 0) {
 	return NULL;
@@ -1913,6 +1911,8 @@ Tcl_MakeFileChannel(handle, mode)
     }
 #endif /* DEPRECATED */
 
+    sockaddr.sa_family = AF_UNSPEC;
+
 #ifdef SUPPORTS_TTY
     if (isatty(fd)) {
 	fsPtr = TtyInit(fd, 0);
@@ -1920,21 +1920,21 @@ Tcl_MakeFileChannel(handle, mode)
 	sprintf(channelName, "serial%d", fd);
     } else
 #endif /* SUPPORTS_TTY */
-    if (getsockopt(fd, SOL_SOCKET, SO_TYPE, (VOID *)&socketType,
-		   &argLength) == 0  &&	 socketType == SOCK_STREAM) {
-	/*
-	 * The mode parameter gets lost here, unfortunately.
-	 */
-	return Tcl_MakeTcpClientChannel((ClientData) fd);
+    if (getsockname(fd, (struct sockaddr *)&sockaddr, &sockaddrLen) == 0
+            && sockaddrLen > 0
+            && sockaddr.sa_family == AF_INET) {
+        return MakeTcpClientChannelMode((ClientData) fd, mode);
     } else {
-	channelTypePtr = &fileChannelType;
-	fsPtr = (FileState *) ckalloc((unsigned) sizeof(FileState));
-	sprintf(channelName, "file%d", fd);
+        channelTypePtr = &fileChannelType;
+        fsPtr = (FileState *) ckalloc((unsigned) sizeof(FileState));
+        sprintf(channelName, "file%d", fd);
     }
 
 #ifdef DEPRECATED
-    fsPtr->nextPtr = tsdPtr->firstFilePtr;
-    tsdPtr->firstFilePtr = fsPtr;
+    if (channelTypePtr == &fileChannelType) {
+        fsPtr->nextPtr = tsdPtr->firstFilePtr;
+        tsdPtr->firstFilePtr = fsPtr;
+    }
 #endif /* DEPRECATED */
     fsPtr->fd = fd;
     fsPtr->validMask = mode | TCL_EXCEPTION;
@@ -2768,6 +2768,32 @@ Tcl_OpenTcpClient(interp, port, host, myaddr, myport, async)
 Tcl_Channel
 Tcl_MakeTcpClientChannel(sock)
     ClientData sock;		/* The socket to wrap up into a channel. */
+{
+    return MakeTcpClientChannelMode(sock, (TCL_READABLE | TCL_WRITABLE));
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * MakeTcpClientChannelMode --
+ *
+ *	Creates a Tcl_Channel from an existing client TCP socket
+ *	with given mode.
+ *
+ * Results:
+ *	The Tcl_Channel wrapped around the preexisting TCP socket.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static Tcl_Channel
+MakeTcpClientChannelMode(sock, mode)
+    ClientData sock;		/* The socket to wrap up into a channel. */
+    int mode;			/* ORed combination of TCL_READABLE and
+				 * TCL_WRITABLE to indicate file mode. */
 {
     TcpState *statePtr;
     char channelName[16 + TCL_INTEGER_SPACE];
@@ -2781,7 +2807,7 @@ Tcl_MakeTcpClientChannel(sock)
     sprintf(channelName, "sock%d", statePtr->fd);
 
     statePtr->channel = Tcl_CreateChannel(&tcpChannelType, channelName,
-	    (ClientData) statePtr, (TCL_READABLE | TCL_WRITABLE));
+	    (ClientData) statePtr, mode);
     if (Tcl_SetChannelOption((Tcl_Interp *) NULL, statePtr->channel,
 	    "-translation", "auto crlf") == TCL_ERROR) {
 	Tcl_Close((Tcl_Interp *) NULL, statePtr->channel);
@@ -3243,3 +3269,97 @@ TclUnixWaitForFile(fd, mask, timeout)
     }
     return result;
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TclpCutFileChannel --
+ *
+ *	Remove any thread local refs to this channel. See
+ *	Tcl_CutChannel for more info.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Changes thread local list of valid channels.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TclpCutFileChannel(chan)
+    Tcl_Channel chan;			/* The channel being removed. Must
+                                         * not be referenced in any
+                                         * interpreter. */
+{
+#ifdef DEPRECATED
+    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+    Channel *chanPtr = (Channel *) chan;
+    FileState *fsPtr;
+    FileState **nextPtrPtr;
+    int removed = 0;
+
+    if (chanPtr->typePtr != &fileChannelType)
+        return;
+
+    fsPtr = (FileState *) chanPtr->instanceData;
+
+    for (nextPtrPtr = &(tsdPtr->firstFilePtr); (*nextPtrPtr) != NULL;
+	 nextPtrPtr = &((*nextPtrPtr)->nextPtr)) {
+	if ((*nextPtrPtr) == fsPtr) {
+	    (*nextPtrPtr) = fsPtr->nextPtr;
+	    removed = 1;
+	    break;
+	}
+    }
+
+    /*
+     * This could happen if the channel was created in one thread
+     * and then moved to another without updating the thread
+     * local data in each thread.
+     */
+
+    if (!removed)
+        panic("file info ptr not on thread channel list");
+
+#endif /* DEPRECATED */
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TclpSpliceFileChannel --
+ *
+ *	Insert thread local ref for this channel.
+ *	Tcl_SpliceChannel for more info.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Changes thread local list of valid channels.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TclpSpliceFileChannel(chan)
+    Tcl_Channel chan;			/* The channel being removed. Must
+                                         * not be referenced in any
+                                         * interpreter. */
+{
+#ifdef DEPRECATED
+    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+    Channel *chanPtr = (Channel *) chan;
+    FileState *fsPtr;
+
+    if (chanPtr->typePtr != &fileChannelType)
+        return;
+
+    fsPtr = (FileState *) chanPtr->instanceData;
+
+    fsPtr->nextPtr = tsdPtr->firstFilePtr;
+    tsdPtr->firstFilePtr = fsPtr;
+#endif /* DEPRECATED */
+}
