@@ -169,7 +169,7 @@ typedef struct SelectBlock {
  *----------------------------------------------------------------------
  */
 typedef struct Tix_GrSortItem {
-    Arg data;			/* is usually a string, but
+    Tcl_Obj * data;			/* is usually a string, but
 					 * can be a pointer to an
 					 * arbitrary data in C API */
     int index;				/* row or column */
@@ -353,7 +353,7 @@ typedef Grid * WidgetPtr;
 #define DEF_GRID_EDITNOTIFY_COMMAND	""
 #define DEF_GRID_FLOATING_ROWS		"0"
 #define DEF_GRID_FLOATING_COLS		"0"
-#define DEF_GRID_FONT	      	"-Adobe-Helvetica-Bold-R-Normal--*-120-*"
+#define DEF_GRID_FONT			"Helvetica -12 bold"
 #define DEF_GRID_FG_COLOR		BLACK
 #define DEF_GRID_FG_MONO		BLACK
 #define DEF_GRID_FORMAT_COMMAND		""
@@ -437,7 +437,7 @@ EXTERN void		TixGridDataGetGridSize _ANSI_ARGS_((
 			    int *height_ret));
 EXTERN int		TixGridDataGetIndex _ANSI_ARGS_((
 			    Tcl_Interp * interp, WidgetPtr wPtr,
-			    Arg xStr, Arg yStr, int * xPtr, int * yPtr));
+			    Tcl_Obj * xStr, Tcl_Obj * yStr, int * xPtr, int * yPtr));
 EXTERN void 		TixGridDataInsert _ANSI_ARGS_((
 			    TixGridDataSet * dataSet,
 			    int x, int y, ClientData data));
