@@ -1,3 +1,6 @@
+
+/*	$Id: tixImgXpm.c,v 1.1.1.1 2000/05/17 11:08:42 idiscovery Exp $	*/
+
 /*
  * tixImgXpm.c --
  *
@@ -10,6 +13,7 @@
  *
  */
 
+#include "tkInt.h"
 #include "tixPort.h"
 #include "tixInt.h"
 #include "tixImgXpm.h"
@@ -20,13 +24,13 @@
  */
 
 static int		ImgXpmCreate _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *name, int argc, char **argv,
+			    char *name, int argc, Tcl_Obj *CONST *objv,
 			    Tk_ImageType *typePtr, Tk_ImageMaster master,
 			    ClientData *clientDataPtr));
 static ClientData	ImgXpmGet _ANSI_ARGS_((Tk_Window tkwin,
 			    ClientData clientData));
 static void		ImgXpmDisplay _ANSI_ARGS_((ClientData clientData,
-			    Display *display, Drawable drawable, 
+			    Display *display, Drawable drawable,
 			    int imageX, int imageY, int width, int height,
 			    int drawableX, int drawableY));
 static void		ImgXpmFree _ANSI_ARGS_((ClientData clientData,
@@ -39,14 +43,14 @@ static void		ImgXpmCmdDeletedProc _ANSI_ARGS_((
 static void		ImgXpmConfigureInstance _ANSI_ARGS_((
 			    PixmapInstance *instancePtr));
 static int		ImgXpmConfigureMaster _ANSI_ARGS_((
-			    PixmapMaster *masterPtr, int argc, char **argv,
+			    PixmapMaster *masterPtr, int argc, Tcl_Obj *CONST *objv,
 			    int flags));
 static int		ImgXpmGetData _ANSI_ARGS_((Tcl_Interp *interp,
 			    PixmapMaster *masterPtr));
 static char ** 		ImgXpmGetDataFromFile _ANSI_ARGS_((Tcl_Interp * interp,
 			    char * string, int * numLines_return));
 static char ** 		ImgXpmGetDataFromId _ANSI_ARGS_((Tcl_Interp * interp,
-			    char * id));
+			    Tk_Uid id));
 static char ** 		ImgXpmGetDataFromString _ANSI_ARGS_((Tcl_Interp*interp,
 			    char * string, int * numLines_return));
 static void 		ImgXpmGetPixmapFromData _ANSI_ARGS_((
@@ -80,6 +84,7 @@ Tk_ImageType tixPixmapImageType = {
     ImgXpmDisplay,		/* displayProc */
     ImgXpmFree,			/* freeProc */
     ImgXpmDelete,		/* deleteProc */
+    NULL,			/* postscriptProc */
     (Tk_ImageType *) NULL	/* nextPtr */
 };
 
@@ -90,7 +95,7 @@ Tk_ImageType tixPixmapImageType = {
 static Tcl_HashTable xpmTable;
 static int xpmTableInited = 0;
 
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -108,12 +113,12 @@ static int xpmTableInited = 0;
  *----------------------------------------------------------------------
  */
 static int
-ImgXpmCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
+ImgXpmCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
     Tcl_Interp *interp;		/* Interpreter for application containing
 				 * image. */
     char *name;			/* Name to use for image. */
     int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings for options (doesn't
+    Tcl_Obj *CONST *objv;	/* Argument strings for options (doesn't
 				 * include image name or type). */
     Tk_ImageType *typePtr;	/* Pointer to our type record (not used). */
     Tk_ImageMaster master;	/* Token for image, to be used by us in
@@ -122,6 +127,22 @@ ImgXpmCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
 				 * it will be returned in later callbacks. */
 {
     PixmapMaster *masterPtr;
+    int i;
+
+#ifndef _LANG
+    char *argvbuf[10];
+    char **argv = argvbuf;
+
+    /*
+     * Convert the objv arguments into string equivalent.
+     */
+    if (argc > 10) {
+	argv = (char **) ckalloc(argc * sizeof(char *));
+    }
+    for (i = 0; i < argc; i++) {
+	argv[i] = TixGetStringFromObj(objv[i], NULL);
+    }
+#endif /* _LANG */
 
     masterPtr = (PixmapMaster *) ckalloc(sizeof(PixmapMaster));
     masterPtr->tkMaster = master;
@@ -136,14 +157,14 @@ ImgXpmCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
     masterPtr->isDataAlloced = 0;
     masterPtr->instancePtr = NULL;
 
-    if (ImgXpmConfigureMaster(masterPtr, argc, argv, 0) != TCL_OK) {
+    if (ImgXpmConfigureMaster(masterPtr, argc, objv, 0) != TCL_OK) {
 	ImgXpmDelete((ClientData) masterPtr);
 	return TCL_ERROR;
     }
     *clientDataPtr = (ClientData) masterPtr;
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -167,11 +188,11 @@ ImgXpmCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
  *----------------------------------------------------------------------
  */
 static int
-ImgXpmConfigureMaster(masterPtr, argc, argv, flags)
+ImgXpmConfigureMaster(masterPtr, argc, objv, flags)
     PixmapMaster *masterPtr;	/* Pointer to data structure describing
 				 * overall pixmap image to (reconfigure). */
     int argc;			/* Number of entries in argv. */
-    char **argv;		/* Pairs of configuration options for image. */
+    Tcl_Obj *CONST *objv;	/* Pairs of configuration options for image. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget,
 				 * such as TK_CONFIG_ARGV_ONLY. */
 {
@@ -184,7 +205,7 @@ ImgXpmConfigureMaster(masterPtr, argc, argv, flags)
     oldId   = masterPtr->id;
 
     if (Tk_ConfigureWidget(masterPtr->interp, Tk_MainWindow(masterPtr->interp),
-	    configSpecs, argc, argv, (char *) masterPtr, flags)
+	    configSpecs, argc, objv, (char *) masterPtr, flags)
 	    != TCL_OK) {
 	return TCL_ERROR;
     }
@@ -234,7 +255,7 @@ ImgXpmConfigureMaster(masterPtr, argc, argv, flags)
     masterPtr->id = oldId;
     return TCL_ERROR;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -290,7 +311,7 @@ ImgXpmGetData(interp, masterPtr)
     }
 
     /* Parse the first line of the data and get info about this pixmap */
-    count = sscanf(data[0],"%i %i %i %i", 
+    count = sscanf(data[0],"%i %i %i %i",
 		   &size[0], &size[1], &ncolors, &cpp);
 
     if (count != 4) {
@@ -328,10 +349,10 @@ ImgXpmGetData(interp, masterPtr)
 
     return code;
 }
-
+
 static char ** ImgXpmGetDataFromId(interp, id)
     Tcl_Interp * interp;
-    char * id;
+    Tk_Uid id;
 {
     Tcl_HashEntry * hashPtr;
 
@@ -349,7 +370,7 @@ static char ** ImgXpmGetDataFromId(interp, id)
 	return (char**)Tcl_GetHashValue(hashPtr);
     }
 }
-
+
 static char ** ImgXpmGetDataFromString(interp, string, numLines_return)
     Tcl_Interp * interp;
     char * string;
@@ -368,7 +389,7 @@ static char ** ImgXpmGetDataFromString(interp, string, numLines_return)
      * to skip the leading blanks is good for using in-line XPM data in TCL
      * scripts
      */
-    while (isspace(*string)) {
+    while (isspace(UCHAR(*string))) {
 	++ string;
     }
 
@@ -417,7 +438,7 @@ static char ** ImgXpmGetDataFromString(interp, string, numLines_return)
     }
 
     bytes += (numLines + 1) * sizeof(char *);
-    data  = (char **) ckalloc(bytes); 
+    data  = (char **) ckalloc(bytes);
 
     memset((VOID *) data, 0, bytes );
 
@@ -444,7 +465,7 @@ static char ** ImgXpmGetDataFromString(interp, string, numLines_return)
 		continue;
 	    }
 
-	    if (isspace(*p) || *p == ',') {
+	    if (isspace(UCHAR(*p)) || *p == ',') {
 	    }
 	    else if (*p == '}') {
 		break;
@@ -479,7 +500,7 @@ static char ** ImgXpmGetDataFromString(interp, string, numLines_return)
     return data;
 
   error:
-    if (data) { 
+    if (data) {
 	ckfree((char *) data);
 	data = NULL;
     }
@@ -487,7 +508,7 @@ static char ** ImgXpmGetDataFromString(interp, string, numLines_return)
     *numLines_return = 0;
     return (char**) NULL;
 }
-
+
 static char ** ImgXpmGetDataFromFile(interp, fileName, numLines_return)
     Tcl_Interp * interp;
     char * fileName;
@@ -553,34 +574,34 @@ GetType(colorDefn, type_ret)
     char * p = colorDefn;
 
     /* skip white spaces */
-    while (*p && isspace(*p)) {
+    while (*p && isspace(UCHAR(*p))) {
 	p ++;
     }
 
     /* parse the type */
     if (p[0] != '\0' && p[0] == 'm' &&
-	p[1] != '\0' && isspace(p[1])) {
+	p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_MONO;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 'g' &&
 	     p[1] != '\0' && p[1] == '4' &&
-	     p[2] != '\0' && isspace(p[2])) {
+	     p[2] != '\0' && isspace(UCHAR(p[2]))) {
 	*type_ret = XPM_GRAY_4;
 	p += 3;
     }
     else if (p[0] != '\0' && p[0] == 'g' &&
-	     p[1] != '\0' && isspace(p[1])) {
+	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_GRAY;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 'c' &&
-	     p[1] != '\0' && isspace(p[1])) {
+	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_COLOR;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 's' &&
-	     p[1] != '\0' && isspace(p[1])) {
+	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_SYMBOLIC;
 	p += 2;
     }
@@ -617,7 +638,7 @@ GetColor(colorDefn, colorName, type_ret)
     }
 
     /* skip white spaces */
-    while (*colorDefn && isspace(*colorDefn)) {
+    while (*colorDefn && isspace(UCHAR(*colorDefn))) {
 	colorDefn ++;
     }
 
@@ -626,7 +647,7 @@ GetColor(colorDefn, colorName, type_ret)
     while (1) {
 	int dummy;
 
-	while (*colorDefn && !isspace(*colorDefn)) {
+	while (*colorDefn && !isspace(UCHAR(*colorDefn))) {
 	    *p++ = *colorDefn++;
 	}
 
@@ -637,8 +658,8 @@ GetColor(colorDefn, colorName, type_ret)
 	if (GetType(colorDefn, &dummy) == NULL) {
 	    /* the next string should also be considered as a part of a color
 	     * name */
-	    
-	    while (*colorDefn && isspace(*colorDefn)) {
+
+	    while (*colorDefn && isspace(UCHAR(*colorDefn))) {
 		*p++ = *colorDefn++;
 	    }
 	} else {
@@ -654,7 +675,7 @@ GetColor(colorDefn, colorName, type_ret)
 
     return colorDefn;
 }
-
+
 /*----------------------------------------------------------------------
  * ImgXpmGetPixmapFromData --
  *
@@ -769,7 +790,7 @@ ImgXpmGetPixmapFromData(interp, masterPtr, instancePtr)
 	} else {
 	    strncpy(colors[i].cstring, masterPtr->data[i+lOffset],
 		(size_t)masterPtr->cpp);
-	} 
+	}
 
 	if (found) {
 	    if (strcasecmp(useName, "none") != 0) {
@@ -811,7 +832,7 @@ ImgXpmGetPixmapFromData(interp, masterPtr, instancePtr)
 		}
 	    } else {
 		for (k=0; k<masterPtr->ncolors; k++) {
-		    if (strncmp(p, colors[k].cstring, 
+		    if (strncmp(p, colors[k].cstring,
 			    (size_t)masterPtr->cpp) == 0) {
 			TixpXpmSetPixel(instancePtr, image, mask, j, i,
 			        colors[k].colorPtr, &isTransp);
@@ -830,7 +851,7 @@ ImgXpmGetPixmapFromData(interp, masterPtr, instancePtr)
     TixpXpmRealizePixmap(masterPtr, instancePtr, image, mask, isTransp);
     TixpXpmFreeTmpBuffer(masterPtr, instancePtr, image, mask);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -886,7 +907,7 @@ ImgXpmConfigureInstance(instancePtr)
      */
     ImgXpmGetPixmapFromData(masterPtr->interp, masterPtr, instancePtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -954,14 +975,12 @@ ImgXpmCmd(clientData, interp, argc, argv)
 	 */
 	PixmapInstance *instancePtr;
 	int count = 0;
-	char buff[30];
 
 	for (instancePtr=masterPtr->instancePtr; instancePtr;
 	     instancePtr = instancePtr->nextPtr) {
 	    count += instancePtr->refCount;
 	}
-	sprintf(buff, "%d", count);
-	Tcl_SetResult(interp, buff, TCL_VOLATILE);
+	Tcl_SetIntObj(Tcl_GetObjResult(interp), count);
 	return TCL_OK;
     } else {
 	Tcl_AppendResult(interp, "bad option \"", argv[1],
@@ -969,7 +988,7 @@ ImgXpmCmd(clientData, interp, argc, argv)
 	return TCL_ERROR;
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1043,7 +1062,7 @@ ImgXpmGet(tkwin, masterData)
 
     return (ClientData) instancePtr;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1076,7 +1095,7 @@ ImgXpmDisplay(clientData, display, drawable, imageX, imageY, width,
     TixpXpmDisplay(clientData, display, drawable, imageX, imageY, width,
 	height, drawableX, drawableY);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1141,7 +1160,7 @@ ImgXpmFree(clientData, display)
     }
     ckfree((char *) instancePtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1182,7 +1201,7 @@ ImgXpmDelete(masterData)
     Tk_FreeOptions(configSpecs, (char *) masterPtr, (Display *) NULL, 0);
     ckfree((char *) masterPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1215,7 +1234,7 @@ ImgXpmCmdDeletedProc(clientData)
     }
 }
 
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1256,3 +1275,4 @@ Tix_DefinePixmap(interp, name, data)
     Tcl_SetHashValue(hshPtr, (char*)data);
     return TCL_OK;
 }
+
