@@ -35,13 +35,6 @@
 #    endif
 #undef WIN32_LEAN_AND_MEAN
 
-struct _TixpSubRegion {
-    Pixmap pixmap;
-    int origX, origY;
-    int x, y;
-    int width, height;
-};
-
 typedef unsigned char UNSIGNED_CHAR;
 
 #endif /* _TIX_WINPORT_H_ */
