@@ -9,11 +9,14 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkUnixFont.c,v 1.15 2002/08/05 04:30:41 dgp Exp $
+ * RCS: @(#) $Id: tkUnixFont.c,v 1.18.2.3 2003/10/29 01:07:46 hobbs Exp $
  */
 
 #include "tkUnixInt.h"
 #include "tkFont.h"
+
+#ifndef USE_XFT_FONTS
+
 #include <netinet/in.h>		/* for htons() prototype */
 #include <arpa/inet.h>		/* inet_ntoa() */
 
@@ -35,16 +38,20 @@ static CONST char *encodingList[] = {
  *
  * Under Unix, there are three attributes that uniquely identify a "font
  * family": the foundry, face name, and charset.
-  
+
  * FIXME: the existance assumption above does _NOT_ hold for UNICODE fonts
- * e.g. -misc-fixed-*-*-*-*-*-*-*-*-*-*-iso10646-1 has smaller repertoire 
- * in bold/italic than in regular forms  
- 
+ * e.g. -misc-fixed-*-*-*-*-*-*-*-*-*-*-iso10646-1 has smaller repertoire
+ * in bold/italic than in regular forms
+
  */
 
+#if TCL_UTF_MAX <= 3
 #define FONTMAP_SHIFT		8
+#else
+#define FONTMAP_SHIFT		10
+#endif
 
-#define FONTMAP_PAGES	    	(1 << (sizeof(Tcl_UniChar)*8 - FONTMAP_SHIFT))
+#define FONTMAP_PAGES		(1 << (sizeof(Tcl_UniChar)*8 - FONTMAP_SHIFT))
 #define FONTMAP_BITSPERPAGE	(1 << FONTMAP_SHIFT)
 
 typedef struct FontFamily {
@@ -93,6 +100,8 @@ typedef struct SubFont {
 				 * used when displaying/measuring chars
 				 * belonging to the FontFamily. */
     FontFamily *familyPtr;	/* The FontFamily for this SubFont. */
+    char *name;			/* Name used (malloc-ed) */
+    int   trigger;		/* Character which caused us to load this subfont */
 } SubFont;
 
 /*
@@ -181,7 +190,7 @@ static Tcl_ThreadDataKey dataKey;
  */
 
 static EncodingAlias encodingAliases[] = {
-    {"gb2312",		"gb2312*"},
+    {"gb2312-raw",	"gb2312*"},
     {"big5",		"big5*"},
     {"cns11643-1",	"cns11643*-1"},
     {"cns11643-1",	"cns11643*.1-0"},
@@ -207,10 +216,12 @@ static void		FontPkgCleanup _ANSI_ARGS_((ClientData clientData));
 static FontFamily *	AllocFontFamily _ANSI_ARGS_((Display *display,
 			    XFontStruct *fontStructPtr, int base));
 static SubFont *	CanUseFallback _ANSI_ARGS_((UnixFont *fontPtr,
-			    CONST char *fallbackName, int ch));
+			    CONST char *fallbackName, int ch,
+			    SubFont **fixSubFontPtrPtr));
 static SubFont *	CanUseFallbackWithAliases _ANSI_ARGS_((
 			    UnixFont *fontPtr, char *fallbackName,
-			    int ch, Tcl_DString *nameTriedPtr));
+			    int ch, Tcl_DString *nameTriedPtr,
+			    SubFont **fixSubFontPtrPtr));
 static int		ControlUtfProc _ANSI_ARGS_((ClientData clientData,
 			    CONST char *src, int srcLen, int flags,
 			    Tcl_EncodingState *statePtr, char *dst,
@@ -220,7 +231,7 @@ static XFontStruct *	CreateClosestFont _ANSI_ARGS_((Tk_Window tkwin,
 			    CONST TkFontAttributes *faPtr,
 			    CONST TkXLFDAttributes *xaPtr));
 static SubFont *	FindSubFontForChar _ANSI_ARGS_((UnixFont *fontPtr,
-			    int ch));
+			    int ch, SubFont **fixSubFontPtrPtr));
 static void		FontMapInsert _ANSI_ARGS_((SubFont *subFontPtr,
 			    int ch));
 static void		FontMapLoadPage _ANSI_ARGS_((SubFont *subFontPtr,
@@ -247,7 +258,7 @@ static char **		ListFonts _ANSI_ARGS_((Display *display,
 static char **		ListFontOrAlias _ANSI_ARGS_((Display *display,
 			    CONST char *faceName, int *numNamesPtr));
 static unsigned int	RankAttributes _ANSI_ARGS_((FontAttributes *wantPtr,
-			    FontAttributes *gotPtr));
+			    FontAttributes *gotPtr, int ch, const char *name));
 static void		ReleaseFont _ANSI_ARGS_((UnixFont *fontPtr));
 static void		ReleaseSubFont _ANSI_ARGS_((Display *display,
 			    SubFont *subFontPtr));
@@ -421,7 +432,7 @@ ControlUtfProc(clientData, src, srcLen, flags, statePtr, dst, dstLen,
 				 * correspond to the bytes stored in the
 				 * output buffer. */
 {
-    CONST char *srcEnd;
+    CONST char *srcStart, *srcEnd;
     char *dstStart, *dstEnd;
     Tcl_UniChar ch;
     int result;
@@ -433,6 +444,7 @@ ControlUtfProc(clientData, src, srcLen, flags, statePtr, dst, dstLen,
 
     result = TCL_OK;
 
+    srcStart = src;
     srcEnd = src + srcLen;
 
     dstStart = dst;
@@ -462,7 +474,7 @@ ControlUtfProc(clientData, src, srcLen, flags, statePtr, dst, dstLen,
 	    dst += 6;
 	}
     }
-    *srcReadPtr = src - srcEnd;
+    *srcReadPtr = src - srcStart;
     *dstWrotePtr = dst - dstStart;
     *dstCharsPtr = dst - dstStart;
     return result;
@@ -878,16 +890,6 @@ TkpGetFontFamilies(interp, tkwin)
     Tcl_DeleteHashTable(&familyTable);
 }
 
-void
-FontDebug(FontFamily *familyPtr,char *s,int n)
-{
- LangDebug("-%s-%s-*-%s %s %d\n",
-	familyPtr->foundry,
-	familyPtr->faceName,
-	Tcl_GetEncodingName(familyPtr->encoding),s,n);
-}
-
-
 /*
  *-------------------------------------------------------------------------
  *
@@ -912,7 +914,7 @@ TkpGetSubFonts(interp, tkfont)
     Tk_Font tkfont;
 {
     int i;
-    Tcl_Obj *objv[3];
+    Tcl_Obj *objv[4];
     Tcl_Obj *resultPtr, *listPtr;
     UnixFont *fontPtr;
     FontFamily *familyPtr;
@@ -924,8 +926,13 @@ TkpGetSubFonts(interp, tkfont)
 	objv[0] = Tcl_NewStringObj(familyPtr->faceName, -1);
 	objv[1] = Tcl_NewStringObj(familyPtr->foundry, -1);
 	objv[2] = Tcl_NewStringObj(Tcl_GetEncodingName(familyPtr->encoding), -1);
-	listPtr = Tcl_NewListObj(3, objv);
-	Tcl_ListObjAppendElement(NULL, resultPtr, listPtr);
+	objv[3] = Tcl_NewIntObj(fontPtr->subFontArray[i].trigger);
+	listPtr = Tcl_NewListObj(4, objv);
+	if (fontPtr->subFontArray[i].name) {
+	    Tcl_ListObjAppendElement(interp, listPtr,
+                  Tcl_NewStringObj(fontPtr->subFontArray[i].name, -1));
+	}
+	Tcl_ListObjAppendElement(interp, resultPtr, listPtr);
     }
 }
 
@@ -987,9 +994,6 @@ Tk_MeasureChars(tkfont, source, numBytes, maxLength, flags, lengthPtr)
 
     fontPtr = (UnixFont *) tkfont;
 
-    /* FIXME - subFontArray can be re-allocated so lastSubFontPtr can
-       end up pointing at memory which has been reused
-     */
     lastSubFontPtr = &fontPtr->subFontArray[0];
 
     if (numBytes == 0) {
@@ -1014,7 +1018,7 @@ Tk_MeasureChars(tkfont, source, numBytes, maxLength, flags, lengthPtr)
 	end = source + numBytes;
 	for (p = source; p < end; ) {
 	    next = p + Tcl_UtfToUniChar(p, &ch);
-	    thisSubFontPtr = FindSubFontForChar(fontPtr, ch);
+	    thisSubFontPtr = FindSubFontForChar(fontPtr, ch, &lastSubFontPtr);
 	    if (thisSubFontPtr != lastSubFontPtr) {
 		familyPtr = lastSubFontPtr->familyPtr;
 		Tcl_UtfToExternalDString(familyPtr->encoding, source,
@@ -1073,7 +1077,7 @@ Tk_MeasureChars(tkfont, source, numBytes, maxLength, flags, lengthPtr)
 	    if ((ch < BASE_CHARS) && (fontPtr->widths[ch] != 0)) {
 		newX += fontPtr->widths[ch];
 	    } else {
-		lastSubFontPtr = FindSubFontForChar(fontPtr, ch);
+		lastSubFontPtr = FindSubFontForChar(fontPtr, ch, NULL);
 		familyPtr = lastSubFontPtr->familyPtr;
 		Tcl_UtfToExternal(NULL, familyPtr->encoding, p, next - p,
 			0, NULL, buf, sizeof(buf), NULL, &dstWrote, NULL);
@@ -1183,19 +1187,21 @@ Tk_DrawChars(display, drawable, gc, tkfont, source, numBytes, x, y)
     SubFont *thisSubFontPtr, *lastSubFontPtr;
     Tcl_DString runString;
     CONST char *p, *end, *next;
-    int xStart, needWidth, window_width;
+    int xStart, needWidth, window_width, do_width;
     Tcl_UniChar ch;
     FontFamily *familyPtr;
+#ifdef TK_DRAW_CHAR_XWINDOW_CHECK
     int rx, ry;
     unsigned int width, height, border_width, depth;
-    int do_width;
     Drawable root;
+#endif
 
     fontPtr = (UnixFont *) tkfont;
     lastSubFontPtr = &fontPtr->subFontArray[0];
 
     xStart = x;
 
+#ifdef TK_DRAW_CHAR_XWINDOW_CHECK
     /*
      * Get the window width so we can abort drawing outside of the window
      */
@@ -1205,13 +1211,20 @@ Tk_DrawChars(display, drawable, gc, tkfont, source, numBytes, x, y)
     } else {
 	window_width = width;
     }
+#else
+    /*
+     * This is used by default until we find a solution that doesn't
+     * round-trip to the X server (need to get Tk cached window width).
+     */
+    window_width = 32768;
+#endif
 
     end = source + numBytes;
     needWidth = fontPtr->font.fa.underline + fontPtr->font.fa.overstrike;
     for (p = source; p <= end; ) {
 	if (p < end) {
 	    next = p + Tcl_UtfToUniChar(p, &ch);
-	    thisSubFontPtr = FindSubFontForChar(fontPtr, ch);
+	    thisSubFontPtr = FindSubFontForChar(fontPtr, ch, &lastSubFontPtr);
 	} else {
 	    next = p + 1;
 	    thisSubFontPtr = lastSubFontPtr;
@@ -1326,7 +1339,7 @@ CreateClosestFont(tkwin, faPtr, xaPtr)
     if (want.fa.family == NULL) {
 	want.fa.family = Tk_GetUid("fixed");
     }
-    want.fa.size = -TkFontGetPixels(tkwin, faPtr->size);
+    want.fa.size = -TkFontGetPixels(Tk_Screen(tkwin), faPtr->size);
     if (want.xa.charset == NULL || *want.xa.charset == '\0') {
 	want.xa.charset = Tk_GetUid("iso8859-1");	/* locale. */
     }
@@ -1388,8 +1401,8 @@ CreateClosestFont(tkwin, faPtr, xaPtr)
 	}
 	IdentifySymbolEncodings(&got);
 	scalable = (got.fa.size == 0);
-	score = RankAttributes(&want, &got);
-	if (score <= bestScore[scalable]) {
+	score = RankAttributes(&want, &got, ' ', nameList[nameIdx]);
+	if (score < bestScore[scalable]) {
 	    bestIdx[scalable] = nameIdx;
 	    bestScore[scalable] = score;
 	}
@@ -1480,7 +1493,12 @@ InitFont(tkwin, fontStructPtr, fontPtr)
 
     faPtr		= &fontPtr->font.fa;
     faPtr->family	= fa.fa.family;
-    faPtr->size		= TkFontGetPoints(tkwin, fa.fa.size);
+#if 0
+    faPtr->size		= TkFontGetPoints(Tk_Screen(tkwin), fa.fa.size);
+#else
+    /* Actuals are in pixels - says NI-S */
+    faPtr->size		= -TkFontGetPixels(Tk_Screen(tkwin), fa.fa.size);
+#endif
     faPtr->weight	= fa.fa.weight;
     faPtr->slant	= fa.fa.slant;
     faPtr->underline	= 0;
@@ -1493,7 +1511,7 @@ InitFont(tkwin, fontStructPtr, fontPtr)
     fmPtr->fixed	= fixed;
 
     fontPtr->display	= display;
-    fontPtr->pixelSize	= TkFontGetPixels(tkwin, fa.fa.size);
+    fontPtr->pixelSize	= TkFontGetPixels(Tk_Screen(tkwin), fa.fa.size);
     fontPtr->xa		= fa.xa;
 
     fontPtr->numSubFonts	= 1;
@@ -1501,7 +1519,7 @@ InitFont(tkwin, fontStructPtr, fontPtr)
     InitSubFont(display, fontStructPtr, 1, &fontPtr->subFontArray[0]);
 
     fontPtr->controlSubFont	= fontPtr->subFontArray[0];
-    subFontPtr			= FindSubFontForChar(fontPtr, '0');
+    subFontPtr			= FindSubFontForChar(fontPtr, '0', NULL);
     controlPtr			= &fontPtr->controlSubFont;
     controlPtr->fontStructPtr	= subFontPtr->fontStructPtr;
     controlPtr->familyPtr	= &tsdPtr->controlFamily;
@@ -1625,6 +1643,8 @@ InitSubFont(display, fontStructPtr, base, subFontPtr)
     subFontPtr->fontStructPtr = fontStructPtr;
     subFontPtr->familyPtr   = AllocFontFamily(display, fontStructPtr, base);
     subFontPtr->fontMap	    = subFontPtr->familyPtr->fontMap;
+    subFontPtr->name        = NULL;
+    subFontPtr->trigger     = -1;
 }
 
 /*
@@ -1651,6 +1671,9 @@ ReleaseSubFont(display, subFontPtr)
 {
     XFreeFont(display, subFontPtr->fontStructPtr);
     FreeFontFamily(subFontPtr->familyPtr);
+    if (subFontPtr->name) {
+	ckfree(subFontPtr->name);
+    }
 }
 
 /*
@@ -1789,7 +1812,6 @@ FreeFontFamily(familyPtr)
             ckfree(familyPtr->fontMap[i]);
         }
     }
-
     /*
      * Delete from list.
      */
@@ -1823,16 +1845,20 @@ FreeFontFamily(familyPtr)
  * Side effects:
  *	The contents of fontPtr are modified to cache the results
  *	of the lookup and remember any SubFonts that were dynamically
- *	loaded.
+ *	loaded.  The table of SubFonts might be extended, and if a non-NULL
+ *	reference to a subfont pointer is available, it is updated if it
+ *	previously pointed into the old subfont table.
  *
  *-------------------------------------------------------------------------
  */
 
 static SubFont *
-FindSubFontForChar(fontPtr, ch)
+FindSubFontForChar(fontPtr, ch, fixSubFontPtrPtr)
     UnixFont *fontPtr;		/* The font object with which the character
 				 * will be displayed. */
     int ch;			/* The Unicode character to be displayed. */
+    SubFont **fixSubFontPtrPtr;	/* Subfont reference to fix up if we
+				 * reallocate our subfont table. */
 {
     int i, j, k, numNames;
     Tk_Uid faceName;
@@ -1861,6 +1887,7 @@ FindSubFontForChar(fontPtr, ch)
      * name multiple times if it can be reached by multiple paths.
      */
 
+    /* Why not use a hash table ! */
     Tcl_DStringInit(&ds);
 
     /*
@@ -1872,7 +1899,7 @@ FindSubFontForChar(fontPtr, ch)
 
     faceName = fontPtr->font.fa.family;
     if (SeenName(faceName, &ds) == 0) {
-	subFontPtr = CanUseFallback(fontPtr, faceName, ch);
+	subFontPtr = CanUseFallback(fontPtr, faceName, ch, fixSubFontPtrPtr);
 	if (subFontPtr != NULL) {
 	    goto end;
 	}
@@ -1912,7 +1939,8 @@ FindSubFontForChar(fontPtr, ch)
 	 */
 
 	for (j = 0; (fallback = fontFallbacks[i][j]) != NULL; j++) {
-	    subFontPtr = CanUseFallbackWithAliases(fontPtr, fallback, ch, &ds);
+	    subFontPtr = CanUseFallbackWithAliases(fontPtr, fallback, ch, &ds,
+		    fixSubFontPtrPtr);
 	    if (subFontPtr != NULL) {
 		goto end;
 	    }
@@ -1925,7 +1953,8 @@ FindSubFontForChar(fontPtr, ch)
 
     anyFallbacks = TkFontGetGlobalClass();
     for (i = 0; (fallback = anyFallbacks[i]) != NULL; i++) {
-	subFontPtr = CanUseFallbackWithAliases(fontPtr, fallback, ch, &ds);
+	subFontPtr = CanUseFallbackWithAliases(fontPtr, fallback, ch, &ds,
+		fixSubFontPtrPtr);
 	if (subFontPtr != NULL) {
 	    goto end;
 	}
@@ -1941,7 +1970,8 @@ FindSubFontForChar(fontPtr, ch)
 	fallback = strchr(nameList[i] + 1, '-') + 1;
 	strchr(fallback, '-')[0] = '\0';
 	if (SeenName(fallback, &ds) == 0) {
-	    subFontPtr = CanUseFallback(fontPtr, fallback, ch);
+	    subFontPtr = CanUseFallback(fontPtr, fallback, ch,
+		    fixSubFontPtrPtr);
 	    if (subFontPtr != NULL) {
 		XFreeFontNames(nameList);
 		goto end;
@@ -2043,13 +2073,6 @@ FontMapInsert(subFontPtr, ch)
     subFontPtr->fontMap[row][bitOffset >> 3] |= 1 << (bitOffset & 7);
 }
 
-
-void
-Whoops(void)
-{
- abort();
-}
-
 /*
  *-------------------------------------------------------------------------
  *
@@ -2104,8 +2127,6 @@ FontMapLoadPage(subFontPtr, row)
     scale	= maxLo - minLo + 1;
     checkLo	= minLo;
 
-
-
     if (! isTwoByteFont) {
 	if (minLo < 32) {
 	    checkLo = 32;
@@ -2118,6 +2139,7 @@ FontMapLoadPage(subFontPtr, row)
     end = (row + 1) << FONTMAP_SHIFT;
     for (i = row << FONTMAP_SHIFT; i < end; i++) {
 	int hi, lo;
+
         if (unicode) {
 	     buf[0] = (i >> 8);
 	     buf[1] = (i & 0xFF);
@@ -2161,13 +2183,16 @@ FontMapLoadPage(subFontPtr, row)
  * Side effects:
  *	If the name and/or one of its aliases was rejected, the
  *	rejected string is recorded in nameTriedPtr so that it won't
- *	be tried again.
+ *	be tried again.  The table of SubFonts might be extended, and if
+ *	a non-NULL reference to a subfont pointer is available, it is
+ *	updated if it previously pointed into the old subfont table.
  *
  *---------------------------------------------------------------------------
  */
 
 static SubFont *
-CanUseFallbackWithAliases(fontPtr, faceName, ch, nameTriedPtr)
+CanUseFallbackWithAliases(fontPtr, faceName, ch, nameTriedPtr,
+	fixSubFontPtrPtr)
     UnixFont *fontPtr;		/* The font object that will own the new
 				 * screen font. */
     char *faceName;		/* Desired face name for new screen font. */
@@ -2177,13 +2202,15 @@ CanUseFallbackWithAliases(fontPtr, faceName, ch, nameTriedPtr)
 				 * been tried.  It is possible for the same
 				 * face name to be queried multiple times when
 				 * trying to find a suitable screen font. */
+    SubFont **fixSubFontPtrPtr;	/* Subfont reference to fix up if we
+				 * reallocate our subfont table. */
 {
     SubFont *subFontPtr;
     char **aliases;
     int i;
 
     if (SeenName(faceName, nameTriedPtr) == 0) {
-	subFontPtr = CanUseFallback(fontPtr, faceName, ch);
+	subFontPtr = CanUseFallback(fontPtr, faceName, ch, fixSubFontPtrPtr);
 	if (subFontPtr != NULL) {
 	    return subFontPtr;
 	}
@@ -2192,7 +2219,8 @@ CanUseFallbackWithAliases(fontPtr, faceName, ch, nameTriedPtr)
     if (aliases != NULL) {
 	for (i = 0; aliases[i] != NULL; i++) {
 	    if (SeenName(aliases[i], nameTriedPtr) == 0) {
-		subFontPtr = CanUseFallback(fontPtr, aliases[i], ch);
+		subFontPtr = CanUseFallback(fontPtr, aliases[i], ch,
+			fixSubFontPtrPtr);
 		if (subFontPtr != NULL) {
 		    return subFontPtr;
 		}
@@ -2262,18 +2290,23 @@ SeenName(name, dsPtr)
  *
  * Side effects:
  *	The font object's subFontArray is updated to contain a reference
- *	to the newly allocated SubFont.
+ *	to the newly allocated SubFont.  The table of SubFonts might be
+ *	extended, and if a non-NULL reference to a subfont pointer is
+ *	available, it is updated if it previously pointed into the old
+ *	subfont table.
  *
  *-------------------------------------------------------------------------
  */
 
 static SubFont *
-CanUseFallback(fontPtr, faceName, ch)
+CanUseFallback(fontPtr, faceName, ch, fixSubFontPtrPtr)
     UnixFont *fontPtr;		/* The font object that will own the new
 				 * screen font. */
     CONST char *faceName;	/* Desired face name for new screen font. */
     int ch;			/* The Unicode character that the new
 				 * screen font must be able to display. */
+    SubFont **fixSubFontPtrPtr;	/* Subfont reference to fix up if we
+				 * reallocate our subfont table. */
 {
     int i, nameIdx, numNames, srcLen;
     Tk_Uid hateFoundry;
@@ -2347,23 +2380,24 @@ CanUseFallback(fontPtr, faceName, ch)
 	}
 	IdentifySymbolEncodings(&got);
 	charset = GetEncodingAlias(got.xa.charset);
-
- 	if (hateFoundry != NULL) {
+	if (hateFoundry != NULL) {
 	    /*
 	     * E. If the font we picked cannot actually display the
 	     * character, cross out all names with the same foundry and
 	     * encoding.
 	     */
 
-	    if ((hateFoundry == got.xa.foundry) &&
-			((strcmp(hateCharset, charset) == 0)
-#if 1
-    	    	    	/* not clear this is correct for Unicode 
-			   foundry may not always use same subset
-			*/
-			 || (strcmp(hateCharset, "ucs-2be") == 0)
-#endif
-                        )) {
+	    if ((hateFoundry == got.xa.foundry)
+		    && (strcmp(hateCharset, charset) == 0)) {
+		/* not clear this is correct for Unicode
+		   foundry may not always use same subset
+		   on the other hand if it isn't in foundary's
+		   -iso10646-1 font it probably isn't in their
+		   -iso8859-* fonts either.
+		   It is less clear with oriental and symbol fonts
+		   that other encodings might not handle
+		*/
+		if (1 || strcmp(hateCharset, "ucs-2be") != 0)
 		goto crossout;
 	    }
 	} else {
@@ -2371,9 +2405,11 @@ CanUseFallback(fontPtr, faceName, ch)
 	     * B. Cross out all names whose encodings we've already used.
 	     */
 	    if (strcmp(charset,"ucs-2be") == 0) {
-		goto dont_skip;
+		if ((unsigned int) ch <= 0xffff)
+		    goto do_ranking;
+		goto crossout;
 	    }
-	
+
 	    for (i = 0; i < fontPtr->numSubFonts; i++) {
 		encoding = fontPtr->subFontArray[i].familyPtr->encoding;
 		if (strcmp(charset, Tcl_GetEncodingName(encoding)) == 0) {
@@ -2381,8 +2417,8 @@ CanUseFallback(fontPtr, faceName, ch)
 		}
 	    }
 	}
-	dont_skip:
 	
+   do_ranking:
 	/*
 	 * C. Cross out all names whose encoding cannot handle the character.
 	 */
@@ -2415,16 +2451,24 @@ CanUseFallback(fontPtr, faceName, ch)
 	 * D. Rank each name and pick the best match.
 	 */
 
-
 	scalable = (got.fa.size == 0);
-	score = RankAttributes(&want, &got);
-	if (score <= bestScore[scalable]) {
+	score = RankAttributes(&want, &got, ch, nameList[nameIdx]);
+	if (score < bestScore[scalable]) {
 	    bestIdx[scalable] = nameIdx;
 	    bestScore[scalable] = score;
 	}
 	if (score == 0) {
 	    break;
 	}
+	/* Allow ranking to have a veto */
+	else if (score == (unsigned int) -1) {
+	    hateFoundry = got.xa.foundry;
+	    hateCharset = charset;
+	    goto crossout;
+	}
+	else if (score == (unsigned int) -2) {
+	    goto crossout;
+	}
 	continue;
 
 	crossout:
@@ -2440,6 +2484,9 @@ CanUseFallback(fontPtr, faceName, ch)
 	nameList[nameIdx] = NULL;
     }
 
+    /* There may be merit in a version of this which just does XQueryFont()
+       rather than load it server-side until we know it is of use
+     */
     fontStructPtr = GetScreenFont(display, &want, nameList, bestIdx, bestScore, &nameIdx);
 
     encodingCachePtr = (Tcl_Encoding *) Tcl_DStringValue(&dsEncodings);
@@ -2449,14 +2496,17 @@ CanUseFallback(fontPtr, faceName, ch)
     Tcl_DStringFree(&dsEncodings);
     numEncodings = 0;
 
-    if (fontStructPtr == NULL) {
+    if (fontStructPtr == NULL || nameList[nameIdx] == NULL) {
 	if (nameList != nameListOrig) {
 	    ckfree((char *) nameList);
 	}
 	XFreeFontNames(nameListOrig);
 	return NULL;
     }
-
+#ifdef _LANG
+    TkFontParseXLFD(nameList[nameIdx], &got.fa, &got.xa);
+    IdentifySymbolEncodings(&got);
+#endif
     InitSubFont(display, fontStructPtr, 0, &subFont);
     if (FontMapLookup(&subFont, ch) == 0) {
 	/*
@@ -2465,43 +2515,44 @@ CanUseFallback(fontPtr, faceName, ch)
 	 * another font.
 	 */
 	hateFoundry = got.xa.foundry;
-	hateCharset = charset;
-	if (1 || strcmp(charset,"ucs-2be") != 0) {
-	    /* Not a Unicode font discard it */
-	    ReleaseSubFont(display, &subFont);
-	}
-	else {
-	    /* Unicode font that cannot do _this_ character - keep it anyway */
-	    if (0) FontDebug(subFont.familyPtr,"retain",fontPtr->numSubFonts);
-	    if (fontPtr->numSubFonts >= SUBFONT_SPACE) {
-		SubFont *newPtr;
-		
-		newPtr = (SubFont *) ckalloc(sizeof(SubFont) * (fontPtr->numSubFonts + 1));
-		memcpy((char *) newPtr, fontPtr->subFontArray,
-			fontPtr->numSubFonts * sizeof(SubFont));
-		if (fontPtr->subFontArray != fontPtr->staticSubFonts) {
-		    ckfree((char *) fontPtr->subFontArray);
-		}
-		fontPtr->subFontArray = newPtr;
-	    }
-	    fontPtr->subFontArray[fontPtr->numSubFonts] = subFont;
-	    fontPtr->numSubFonts++;
-	}
+	hateCharset = GetEncodingAlias(got.xa.charset);
+	ReleaseSubFont(display, &subFont);
+#ifdef _LANG
+	LangFontRank((unsigned int ) -1, ch, nameList[nameIdx],
+		     want.xa.foundry, &want.fa, want.xa.charset,
+		     got.xa.foundry, &got.fa, got.xa.charset);
+#endif
 	goto retry;
     }
-    LangDebug("For %s U+%04X use %s\n",faceName,ch,nameList[nameIdx]);
+#ifdef _LANG
+    LangFontRank(0, ch, nameList[nameIdx],
+		 want.xa.foundry, &want.fa, want.xa.charset,
+		 got.xa.foundry, &got.fa, got.xa.charset);
+#endif
+    subFont.name = ckalloc(strlen(nameList[nameIdx])+1);
+    subFont.trigger = ch;
+    if (subFont.name) {
+	strcpy(subFont.name,nameList[nameIdx]);
+    }
     if (nameList != nameListOrig) {
 	ckfree((char *) nameList);
     }
     XFreeFontNames(nameListOrig);
 
-    if (0) FontDebug(subFont.familyPtr,"add",fontPtr->numSubFonts);
     if (fontPtr->numSubFonts >= SUBFONT_SPACE) {
 	SubFont *newPtr;
 
 	newPtr = (SubFont *) ckalloc(sizeof(SubFont) * (fontPtr->numSubFonts + 1));
 	memcpy((char *) newPtr, fontPtr->subFontArray,
 		fontPtr->numSubFonts * sizeof(SubFont));
+	if (fixSubFontPtrPtr != NULL) {
+	    register SubFont *fixSubFontPtr = *fixSubFontPtrPtr;
+
+	    if (fixSubFontPtr != &fontPtr->controlSubFont) {
+		*fixSubFontPtrPtr =
+			newPtr + (fixSubFontPtr - fontPtr->subFontArray);
+	    }
+	}
 	if (fontPtr->subFontArray != fontPtr->staticSubFonts) {
 	    ckfree((char *) fontPtr->subFontArray);
 	}
@@ -2531,9 +2582,11 @@ CanUseFallback(fontPtr, faceName, ch)
  */
 
 static unsigned int
-RankAttributes(wantPtr, gotPtr)
+RankAttributes(wantPtr, gotPtr, ch, name)
     FontAttributes *wantPtr;	/* The desired attributes. */
     FontAttributes *gotPtr;	/* The attributes we have to live with. */
+    int ch;			/* character we want */
+    const char *name;		/* Font we are trying */
 {
     unsigned int penalty;
 
@@ -2587,9 +2640,6 @@ RankAttributes(wantPtr, gotPtr)
 	penalty += 65000;
 	gotAlias = GetEncodingAlias(gotPtr->xa.charset);
 	wantAlias = GetEncodingAlias(wantPtr->xa.charset);
-	if (strcmp(gotPtr->xa.charset,"ucs-2be") != 0) {
-	    /* Maybe reduce penalty for Unicode ? */
-	}
 	if (strcmp(gotAlias, wantAlias) != 0) {
 	    penalty += 30000;
 	    for (i = 0; encodingList[i] != NULL; i++) {
@@ -2601,8 +2651,12 @@ RankAttributes(wantPtr, gotPtr)
 		penalty += 20000;
 	    }
 	}
-        if (0) LangDebug("wanted %s got %s penalty = %u\n",wantAlias, gotAlias,penalty);
     }
+#ifdef _LANG
+    penalty = LangFontRank(penalty, ch, name,
+                           wantPtr->xa.foundry,&wantPtr->fa, wantPtr->xa.charset,
+                           gotPtr->xa.foundry,&gotPtr->fa, gotPtr->xa.charset);
+#endif
     return penalty;
 }
 
@@ -2636,7 +2690,7 @@ GetScreenFont(display, wantPtr, nameList, bestIdx, bestScore, indexPtr)
     unsigned int bestScore[2];	/* Scores of best bitmapped and best
 				 * scalable font.  XLFD corresponding to
 				 * lowest score will be constructed. */
-    int *indexPtr;  	    	/* Return index of name we used */				 
+    int *indexPtr;  	    	/* Return index of name we used */				
 {
     XFontStruct *fontStructPtr;
 
@@ -2653,7 +2707,7 @@ GetScreenFont(display, wantPtr, nameList, bestIdx, bestScore, indexPtr)
      */
 
     fontStructPtr = NULL;
-    if (bestScore[1] < bestScore[0]) {
+    if (bestScore[1] < bestScore[0] && bestScore[1] < (unsigned int) -2) {
 	char *str, *rest;
 	char buf[256];
 	int i;
@@ -2679,9 +2733,9 @@ GetScreenFont(display, wantPtr, nameList, bestIdx, bestScore, indexPtr)
 	if (fontStructPtr != NULL) {
 	    *indexPtr = bestIdx[1];
 	}
-	bestScore[1] = INT_MAX;
+	bestScore[1] = (unsigned int) -1;
     }
-    if (fontStructPtr == NULL) {
+    if (fontStructPtr == NULL && bestScore[0] < (unsigned int) -2) {
 	fontStructPtr = XLoadQueryFont(display, nameList[bestIdx[0]]);
 	if (fontStructPtr == NULL) {
 	    /*
@@ -2689,7 +2743,7 @@ GetScreenFont(display, wantPtr, nameList, bestIdx, bestScore, indexPtr)
 	     * names that X gave us to use, but it does anyhow.
 	     */
 
-	    if (bestScore[1] < INT_MAX) {
+	    if (bestScore[1] < (unsigned int) -2) {
 		goto tryscale;
 	    }
 	    return GetSystemFont(display);
@@ -2773,6 +2827,13 @@ GetFontAttributes(display, fontStructPtr, faPtr)
     } else {
 	TkInitFontAttributes(&faPtr->fa);
 	TkInitXLFDAttributes(&faPtr->xa);
+    }
+    /*
+     * Do last ditch check for family.  It seems that some X servers can
+     * fail on the X font calls above, slipping through earlier checks.
+     * X-Win32 5.4 is one of these.
+     */
+    if (faPtr->fa.family == NULL) {
 	faPtr->fa.family = Tk_GetUid("");
 	faPtr->xa.foundry = Tk_GetUid("");
 	faPtr->xa.charset = Tk_GetUid("");
@@ -2922,6 +2983,4 @@ GetEncodingAlias(name)
     return name;
 }
 
-
-
-
+#endif /* USE_XFT_FONTS */
