@@ -510,7 +510,7 @@ Tk_GetPixmap(display, d, width, height, depth)
 {
     if (!width || !height)
      {
-      LangDebug(__FUNCTION__ " w=%d h=%d d=%d\n",width, height,depth);
+      LangDebug("%s w=%d h=%d d=%d\n",__FUNCTION__,width, height,depth);
       abort();
      }
     return XCreatePixmap(display, d, (unsigned) width, (unsigned) height,
@@ -605,11 +605,11 @@ TkpWindowWasRecentlyDeleted(win, dispPtr)
 int
 TkpScanWindowId(interp, string, idPtr)
     Tcl_Interp *interp;
-    CONST char *string;
+    Tcl_Obj *string;
     Window *idPtr;
 {
     int value;
-    if (Tcl_GetInt(interp, string, &value) != TCL_OK) {
+    if (Tcl_GetIntFromObj(interp, string, &value) != TCL_OK) {
 	return TCL_ERROR;
     }
     *idPtr = (Window) value;
