@@ -159,7 +159,7 @@ TkpUseWindow(interp, tkwin, string)
         panic("TkpUseWindow: Already assigned a window");
     }
 
-    if (Tcl_GetInt(interp, string, &id) != TCL_OK) {
+    if (Tcl_GetIntFromObj(interp, string, &id) != TCL_OK) {
         return TCL_ERROR;
     }
     hwnd = (HWND) id;
