@@ -22,10 +22,6 @@
 #  define LangDebug (*TkeventVptr->V_LangDebug)
 #endif
 
-#ifndef LangExit
-#  define LangExit (*TkeventVptr->V_LangExit)
-#endif
-
 #ifndef LangFreeCallback
 #  define LangFreeCallback (*TkeventVptr->V_LangFreeCallback)
 #endif
