@@ -34,8 +34,8 @@ static char rcsid[] = "$Header$";
 
 /* /home/nick/bin/add_protos : Added declarations with prototypes */
 static int  PropToResult _ANSI_ARGS_((Tcl_Interp *interp, Tk_Window tkwin, Atom type, unsigned char *p, int format, long unsigned int count));
-static int  ArgToProp _ANSI_ARGS_((Tcl_Interp *interp, Tk_Window tkwin, Atom type, int format, Arg arg, unsigned char **prop, long unsigned int *count));
-extern int  Tk_PropertyCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc, Arg *args));
+static int  ArgToProp _ANSI_ARGS_((Tcl_Interp *interp, Tk_Window tkwin, Atom type, int format, Tcl_Obj * arg, unsigned char **prop, long unsigned int *count));
+extern int  Tk_PropertyCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc, Tcl_Obj * *args));
 static int PropertyExists _ANSI_ARGS_((Tk_Window tkwin, Window xid, Atom key, long unsigned int *sizep));
 
 static int
@@ -121,7 +121,7 @@ Tcl_Interp *interp;
 Tk_Window tkwin;
 Atom type;
 int format;
-Arg arg;
+Tcl_Obj * arg;
 unsigned char **prop;
 long unsigned int *count;
 {
@@ -137,7 +137,7 @@ long unsigned int *count;
  else
   {
    int valc = 0;
-   Arg *valv = NULL;
+   Tcl_Obj * *valv = NULL;
    result = Tcl_ListObjGetElements(interp, arg, &valc, &valv);
    if (result == TCL_OK)
     {
@@ -213,7 +213,7 @@ ClientData clientData;            /* Main window associated with
                                      * interpreter. */
 Tcl_Interp *interp;               /* Current interpreter. */
 int argc;                         /* Number of arguments. */
-Arg *args;                        /* Argument strings. */
+Tcl_Obj * *args;                        /* Argument strings. */
 {
 
  Tk_Window tkwin = (Tk_Window) clientData;
