@@ -15,7 +15,7 @@ extern Tix_DItemInfo tix_WindowItemType;
 
 
 void
-Boot_Tix _((void))
+Boot_Tix (pTHX)
 {
  install_vtab("TixVtab",TixVGet(),sizeof(TixVtab));
  install_vtab("TixintVtab",TixintVGet(),sizeof(TixintVtab));
